@@ -2,7 +2,7 @@
 **
 ** This file is part of Qt Creator
 **
-** Copyright (c) 2011 Nokia Corporation and/or its subsidiary(-ies).
+** Copyright (c) 2012 Nokia Corporation and/or its subsidiary(-ies).
 **
 ** Contact: Nokia Corporation (qt-info@nokia.com)
 **
@@ -43,7 +43,7 @@
 // We mean it.
 //
 
-#include <QtScript/qscriptengineagent.h>
+#include <qscriptengineagent.h>
 
 #include "qt_private/qdeclarativedebugservice_p.h"
 
