@@ -103,15 +103,16 @@ public:
     virtual bool visit(DeclarationStatementAST *) { return true; }
     virtual bool visit(DeclaratorAST *) { return true; }
     virtual bool visit(DeclaratorIdAST *) { return true; }
+    virtual bool visit(DecltypeSpecifierAST *) { return true; }
     virtual bool visit(DeleteExpressionAST *) { return true; }
     virtual bool visit(DestructorNameAST *) { return true; }
     virtual bool visit(DoStatementAST *) { return true; }
+    virtual bool visit(DynamicExceptionSpecificationAST *) { return true; }
     virtual bool visit(ElaboratedTypeSpecifierAST *) { return true; }
     virtual bool visit(EmptyDeclarationAST *) { return true; }
     virtual bool visit(EnumSpecifierAST *) { return true; }
     virtual bool visit(EnumeratorAST *) { return true; }
     virtual bool visit(ExceptionDeclarationAST *) { return true; }
-    virtual bool visit(ExceptionSpecificationAST *) { return true; }
     virtual bool visit(ExpressionOrDeclarationStatementAST *) { return true; }
     virtual bool visit(ExpressionStatementAST *) { return true; }
     virtual bool visit(ForStatementAST *) { return true; }
@@ -141,6 +142,7 @@ public:
     virtual bool visit(NewInitializerAST *) { return true; }
     virtual bool visit(NewPlacementAST *) { return true; }
     virtual bool visit(NewTypeIdAST *) { return true; }
+    virtual bool visit(NoExceptSpecificationAST *) { return true; }
     virtual bool visit(NumericLiteralAST *) { return true; }
     virtual bool visit(ObjCClassDeclarationAST *) { return true; }
     virtual bool visit(ObjCClassForwardDeclarationAST *) { return true; }
@@ -172,6 +174,7 @@ public:
     virtual bool visit(ParameterDeclarationAST *) { return true; }
     virtual bool visit(ParameterDeclarationClauseAST *) { return true; }
     virtual bool visit(PointerAST *) { return true; }
+    virtual bool visit(PointerLiteralAST *) { return true; }
     virtual bool visit(PointerToMemberAST *) { return true; }
     virtual bool visit(PostIncrDecrAST *) { return true; }
     virtual bool visit(QtEnumDeclarationAST *) { return true; }
@@ -185,12 +188,14 @@ public:
     virtual bool visit(QtPropertyDeclarationAST *) { return true; }
     virtual bool visit(QtPropertyDeclarationItemAST *) { return true; }
     virtual bool visit(QualifiedNameAST *) { return true; }
+    virtual bool visit(RangeBasedForStatementAST *) { return true; }
     virtual bool visit(ReferenceAST *) { return true; }
     virtual bool visit(ReturnStatementAST *) { return true; }
     virtual bool visit(SimpleDeclarationAST *) { return true; }
     virtual bool visit(SimpleNameAST *) { return true; }
     virtual bool visit(SimpleSpecifierAST *) { return true; }
     virtual bool visit(SizeofExpressionAST *) { return true; }
+    virtual bool visit(StaticAssertDeclarationAST *) { return true; }
     virtual bool visit(StringLiteralAST *) { return true; }
     virtual bool visit(SwitchStatementAST *) { return true; }
     virtual bool visit(TemplateDeclarationAST *) { return true; }
@@ -242,15 +247,16 @@ public:
     virtual void endVisit(DeclarationStatementAST *) {}
     virtual void endVisit(DeclaratorAST *) {}
     virtual void endVisit(DeclaratorIdAST *) {}
+    virtual void endVisit(DecltypeSpecifierAST *) {}
     virtual void endVisit(DeleteExpressionAST *) {}
     virtual void endVisit(DestructorNameAST *) {}
     virtual void endVisit(DoStatementAST *) {}
+    virtual void endVisit(DynamicExceptionSpecificationAST *) {}
     virtual void endVisit(ElaboratedTypeSpecifierAST *) {}
     virtual void endVisit(EmptyDeclarationAST *) {}
     virtual void endVisit(EnumSpecifierAST *) {}
     virtual void endVisit(EnumeratorAST *) {}
     virtual void endVisit(ExceptionDeclarationAST *) {}
-    virtual void endVisit(ExceptionSpecificationAST *) {}
     virtual void endVisit(ExpressionOrDeclarationStatementAST *) {}
     virtual void endVisit(ExpressionStatementAST *) {}
     virtual void endVisit(ForStatementAST *) {}
@@ -280,6 +286,7 @@ public:
     virtual void endVisit(NewInitializerAST *) {}
     virtual void endVisit(NewPlacementAST *) {}
     virtual void endVisit(NewTypeIdAST *) {}
+    virtual void endVisit(NoExceptSpecificationAST *) {}
     virtual void endVisit(NumericLiteralAST *) {}
     virtual void endVisit(ObjCClassDeclarationAST *) {}
     virtual void endVisit(ObjCClassForwardDeclarationAST *) {}
@@ -311,6 +318,7 @@ public:
     virtual void endVisit(ParameterDeclarationAST *) {}
     virtual void endVisit(ParameterDeclarationClauseAST *) {}
     virtual void endVisit(PointerAST *) {}
+    virtual void endVisit(PointerLiteralAST *) {}
     virtual void endVisit(PointerToMemberAST *) {}
     virtual void endVisit(PostIncrDecrAST *) {}
     virtual void endVisit(QtEnumDeclarationAST *) {}
@@ -324,12 +332,14 @@ public:
     virtual void endVisit(QtPropertyDeclarationAST *) {}
     virtual void endVisit(QtPropertyDeclarationItemAST *) {}
     virtual void endVisit(QualifiedNameAST *) {}
+    virtual void endVisit(RangeBasedForStatementAST *) {}
     virtual void endVisit(ReferenceAST *) {}
     virtual void endVisit(ReturnStatementAST *) {}
     virtual void endVisit(SimpleDeclarationAST *) {}
     virtual void endVisit(SimpleNameAST *) {}
     virtual void endVisit(SimpleSpecifierAST *) {}
     virtual void endVisit(SizeofExpressionAST *) {}
+    virtual void endVisit(StaticAssertDeclarationAST *) {}
     virtual void endVisit(StringLiteralAST *) {}
     virtual void endVisit(SwitchStatementAST *) {}
     virtual void endVisit(TemplateDeclarationAST *) {}
