@@ -92,11 +92,13 @@ void UsingDeclaration::visitSymbol0(SymbolVisitor *visitor)
 
 Declaration::Declaration(TranslationUnit *translationUnit, unsigned sourceLocation, const Name *name)
     : Symbol(translationUnit, sourceLocation, name)
+    , _initializer(0)
 { }
 
 Declaration::Declaration(Clone *clone, Subst *subst, Declaration *original)
     : Symbol(clone, subst, original)
     , _type(clone->type(original->_type, subst))
+    , _initializer(clone->stringLiteral(original->_initializer))
 { }
 
 Declaration::~Declaration()
@@ -105,9 +107,19 @@ Declaration::~Declaration()
 void Declaration::setType(const FullySpecifiedType &type)
 { _type = type; }
 
+void Declaration::setInitializer(const StringLiteral *initializer)
+{
+    _initializer = initializer;
+}
+
 FullySpecifiedType Declaration::type() const
 { return _type; }
 
+const StringLiteral *Declaration::getInitializer() const
+{
+    return _initializer;
+}
+
 void Declaration::visitSymbol0(SymbolVisitor *visitor)
 { visitor->visit(this); }
 
@@ -543,10 +555,12 @@ bool Template::matchType0(const Type *otherType, TypeMatcher *matcher) const
 
 Namespace::Namespace(TranslationUnit *translationUnit, unsigned sourceLocation, const Name *name)
     : Scope(translationUnit, sourceLocation, name)
+    , _isInline(false)
 { }
 
 Namespace::Namespace(Clone *clone, Subst *subst, Namespace *original)
     : Scope(clone, subst, original)
+    , _isInline(original->_isInline)
 { }
 
 Namespace::~Namespace()
