@@ -2,7 +2,7 @@
 **
 ** This file is part of Qt Creator
 **
-** Copyright (c) 2011 Nokia Corporation and/or its subsidiary(-ies).
+** Copyright (c) 2012 Nokia Corporation and/or its subsidiary(-ies).
 **
 ** Contact: Nokia Corporation (qt-info@nokia.com)
 **
@@ -34,7 +34,7 @@
 #define CPLUSPLUS_LOOKUPITEM_H
 
 #include <FullySpecifiedType.h>
-#include <QtCore/QHash>
+#include <QHash>
 
 namespace CPlusPlus {
 
