@@ -2,7 +2,7 @@
 **
 ** This file is part of Qt Creator
 **
-** Copyright (c) 2011 Nokia Corporation and/or its subsidiary(-ies).
+** Copyright (c) 2012 Nokia Corporation and/or its subsidiary(-ies).
 **
 ** Contact: Nokia Corporation (qt-info@nokia.com)
 **
@@ -35,9 +35,9 @@
 
 #include "extensionsystem_global.h"
 
-#include <QtCore/QHash>
-#include <QtGui/QWidget>
-#include <QtGui/QIcon>
+#include <QHash>
+#include <QWidget>
+#include <QIcon>
 
 QT_BEGIN_NAMESPACE
 class QTreeWidgetItem;
