@@ -2,7 +2,7 @@
 **
 ** This file is part of Qt Creator
 **
-** Copyright (c) 2011 Nokia Corporation and/or its subsidiary(-ies).
+** Copyright (c) 2012 Nokia Corporation and/or its subsidiary(-ies).
 **
 ** Contact: Nokia Corporation (qt-info@nokia.com)
 **
@@ -33,9 +33,9 @@
 #ifndef EASINGGRAPH_H
 #define EASINGGRAPH_H
 
-#include <QtGui/QWidget>
-#include <QtCore/QEasingCurve>
-#include <QtCore/QHash>
+#include <QWidget>
+#include <QEasingCurve>
+#include <QHash>
 
 QT_BEGIN_HEADER
 
