@@ -2,7 +2,7 @@
 **
 ** This file is part of Qt Creator
 **
-** Copyright (c) 2011 Nokia Corporation and/or its subsidiary(-ies).
+** Copyright (c) 2012 Nokia Corporation and/or its subsidiary(-ies).
 **
 ** Contact: Nokia Corporation (qt-info@nokia.com)
 **
@@ -32,11 +32,11 @@
 
 #include "qmljscodeformatter.h"
 
-#include <QtCore/QDebug>
-#include <QtCore/QMetaEnum>
-#include <QtGui/QTextDocument>
-#include <QtGui/QTextCursor>
-#include <QtGui/QTextBlock>
+#include <QDebug>
+#include <QMetaEnum>
+#include <QTextDocument>
+#include <QTextCursor>
+#include <QTextBlock>
 
 using namespace QmlJS;
 
@@ -90,6 +90,7 @@ void CodeFormatter::recalculateStateAfter(const QTextBlock &block)
             switch (kind) {
             case Identifier:    enter(objectdefinition_or_js); continue;
             case Import:        enter(top_qml); continue;
+            case LeftBrace:     enter(top_js); enter(expression); continue; // if a file starts with {, it's likely json
             default:            enter(top_js); continue;
             } break;
 
@@ -155,6 +156,9 @@ void CodeFormatter::recalculateStateAfter(const QTextBlock &block)
             switch (kind) {
             case Semicolon:     leave(true); break;
             case If:            enter(if_statement); break;
+            case With:          enter(statement_with_condition); break;
+            case Try:           enter(try_statement); break;
+            case Switch:        enter(switch_statement); break;
             case LeftBrace:     enter(jsblock_open); break;
             case On:
             case As:
@@ -163,6 +167,11 @@ void CodeFormatter::recalculateStateAfter(const QTextBlock &block)
             case Signal:
             case Property:
             case Identifier:    enter(expression_or_objectdefinition); break;
+
+            // error recovery
+            case RightBracket:
+            case RightParenthesis:  leave(true); break;
+
             default:            enter(expression); continue;
             } break;
 
@@ -247,9 +256,14 @@ void CodeFormatter::recalculateStateAfter(const QTextBlock &block)
         case expression_or_objectdefinition:
             switch (kind) {
             case Dot:
-            case Identifier:    break; // need to become an objectdefinition_open in cases like "width: Qt.Foo {"
-            case LeftBrace:     turnInto(objectdefinition_open); break;
-            default:            enter(expression); continue; // really? identifier and more tokens might already be gone
+            case Identifier:        break; // need to become an objectdefinition_open in cases like "width: Qt.Foo {"
+            case LeftBrace:         turnInto(objectdefinition_open); break;
+
+            // propagate 'leave' from expression state
+            case RightBracket:
+            case RightParenthesis:  leave(); continue;
+
+            default:                enter(expression); continue; // really? identifier and more tokens might already be gone
             } break;
 
         case expression_or_label:
@@ -376,14 +390,17 @@ void CodeFormatter::recalculateStateAfter(const QTextBlock &block)
             } break;
 
         case maybe_else:
-            if (kind == Else) {
-                turnInto(else_clause);
-                enter(substatement);
-                break;
-            } else {
-                leave(true);
-                continue;
-            }
+            switch (kind) {
+            case Else:              turnInto(else_clause); enter(substatement); break;
+            default:                leave(true); continue;
+            } break;
+
+        case maybe_catch_or_finally:
+            switch (kind) {
+            case Catch:             turnInto(catch_statement); break;
+            case Finally:           turnInto(finally_statement); break;
+            default:                leave(true); continue;
+            } break;
 
         case else_clause:
             // ### shouldn't happen
@@ -406,6 +423,7 @@ void CodeFormatter::recalculateStateAfter(const QTextBlock &block)
             } break;
 
         case switch_statement:
+        case catch_statement:
         case statement_with_condition:
             switch (kind) {
             case LeftParenthesis:   enter(statement_with_condition_paren_open); break;
@@ -419,7 +437,8 @@ void CodeFormatter::recalculateStateAfter(const QTextBlock &block)
             case RightParenthesis:  turnInto(substatement); break;
             } break;
 
-        case statement_with_block:
+        case try_statement:
+        case finally_statement:
             switch (kind) {
             case LeftBrace:         enter(jsblock_open); break;
             default:                leave(true); break;
@@ -429,7 +448,7 @@ void CodeFormatter::recalculateStateAfter(const QTextBlock &block)
             switch (kind) {
             case While:             break;
             case LeftParenthesis:   enter(do_statement_while_paren_open); break;
-            default:                leave(true); break;
+            default:                leave(true); continue; // error recovery
             } break;
 
         case do_statement_while_paren_open:
@@ -632,6 +651,8 @@ void CodeFormatter::enter(int newState)
     m_currentState.push(s);
     m_newStates.push(s);
 
+    //qDebug() << "enter state" << stateToString(newState);
+
     if (newState == bracket_open)
         enter(bracket_element_start);
 }
@@ -651,10 +672,10 @@ void CodeFormatter::leave(bool statementDone)
 
     int topState = m_currentState.top().type;
 
+    //qDebug() << "left state" << stateToString(poppedState.type) << ", now in state" << stateToString(topState);
+
     // if statement is done, may need to leave recursively
     if (statementDone) {
-        if (!isExpressionEndState(topState))
-            leave(true);
         if (topState == if_statement) {
             if (poppedState.type != maybe_else)
                 enter(maybe_else);
@@ -664,6 +685,15 @@ void CodeFormatter::leave(bool statementDone)
             // leave the else *and* the surrounding if, to prevent another else
             leave();
             leave(true);
+        } else if (topState == try_statement) {
+            if (poppedState.type != maybe_catch_or_finally
+                    && poppedState.type != finally_statement) {
+                enter(maybe_catch_or_finally);
+            } else {
+                leave(true);
+            }
+        } else if (!isExpressionEndState(topState)) {
+            leave(true);
         }
     }
 }
@@ -739,8 +769,7 @@ bool CodeFormatter::tryStatement()
         enter(case_start);
         return true;
     case Try:
-    case Finally:
-        enter(statement_with_block);
+        enter(try_statement);
         return true;
     case LeftBrace:
         enter(jsblock_open);
@@ -786,8 +815,7 @@ bool CodeFormatter::isExpressionEndState(int type) const
             type == topmost_intro ||
             type == top_js ||
             type == objectdefinition_open ||
-            type == if_statement ||
-            type == else_clause ||
+            type == do_statement ||
             type == jsblock_open ||
             type == substatement_open ||
             type == bracket_open ||
@@ -986,12 +1014,341 @@ CodeFormatter::TokenKind CodeFormatter::extendedTokenKind(const QmlJS::Token &to
 
 void CodeFormatter::dump() const
 {
-    QMetaEnum metaEnum = staticMetaObject.enumerator(staticMetaObject.indexOfEnumerator("StateType"));
-
     qDebug() << "Current token index" << m_tokenIndex;
     qDebug() << "Current state:";
     foreach (const State &s, m_currentState) {
-        qDebug() << metaEnum.valueToKey(s.type) << s.savedIndentDepth;
+        qDebug() << stateToString(s.type) << s.savedIndentDepth;
     }
     qDebug() << "Current indent depth:" << m_indentDepth;
 }
+
+QString CodeFormatter::stateToString(int type) const
+{
+    const QMetaEnum &metaEnum = staticMetaObject.enumerator(staticMetaObject.indexOfEnumerator("StateType"));
+    return metaEnum.valueToKey(type);
+}
+
+QtStyleCodeFormatter::QtStyleCodeFormatter()
+    : m_indentSize(4)
+{}
+
+void QtStyleCodeFormatter::setIndentSize(int size)
+{
+    m_indentSize = size;
+}
+
+void QtStyleCodeFormatter::onEnter(int newState, int *indentDepth, int *savedIndentDepth) const
+{
+    const State &parentState = state();
+    const Token &tk = currentToken();
+    const int tokenPosition = column(tk.begin());
+    const bool firstToken = (tokenIndex() == 0);
+    const bool lastToken = (tokenIndex() == tokenCount() - 1);
+
+    switch (newState) {
+    case objectdefinition_open: {
+        // special case for things like "gradient: Gradient {"
+        if (parentState.type == binding_assignment)
+            *savedIndentDepth = state(1).savedIndentDepth;
+
+        if (firstToken)
+            *savedIndentDepth = tokenPosition;
+
+        *indentDepth = *savedIndentDepth + m_indentSize;
+        break;
+    }
+
+    case binding_or_objectdefinition:
+        if (firstToken)
+            *indentDepth = *savedIndentDepth = tokenPosition;
+        break;
+
+    case binding_assignment:
+    case objectliteral_assignment:
+        if (lastToken)
+            *indentDepth = *savedIndentDepth + 4;
+        else
+            *indentDepth = column(tokenAt(tokenIndex() + 1).begin());
+        break;
+
+    case expression_or_objectdefinition:
+        *indentDepth = tokenPosition;
+        break;
+
+    case expression_or_label:
+        if (*indentDepth == tokenPosition)
+            *indentDepth += 2*m_indentSize;
+        else
+            *indentDepth = tokenPosition;
+        break;
+
+    case expression:
+        if (*indentDepth == tokenPosition) {
+            // expression_or_objectdefinition doesn't want the indent
+            // expression_or_label already has it
+            if (parentState.type != expression_or_objectdefinition
+                    && parentState.type != expression_or_label
+                    && parentState.type != binding_assignment) {
+                *indentDepth += 2*m_indentSize;
+            }
+        }
+        // expression_or_objectdefinition and expression_or_label have already consumed the first token
+        else if (parentState.type != expression_or_objectdefinition
+                 && parentState.type != expression_or_label) {
+            *indentDepth = tokenPosition;
+        }
+        break;
+
+    case expression_maybe_continuation:
+        // set indent depth to indent we'd get if the expression ended here
+        for (int i = 1; state(i).type != topmost_intro; ++i) {
+            const int type = state(i).type;
+            if (isExpressionEndState(type) && !isBracelessState(type)) {
+                *indentDepth = state(i - 1).savedIndentDepth;
+                break;
+            }
+        }
+        break;
+
+    case bracket_open:
+        if (parentState.type == expression && state(1).type == binding_assignment) {
+            *savedIndentDepth = state(2).savedIndentDepth;
+            *indentDepth = *savedIndentDepth + m_indentSize;
+        } else if (parentState.type == objectliteral_assignment) {
+            *savedIndentDepth = parentState.savedIndentDepth;
+            *indentDepth = *savedIndentDepth + m_indentSize;
+        } else if (!lastToken) {
+            *indentDepth = tokenPosition + 1;
+        } else {
+            *indentDepth = *savedIndentDepth + m_indentSize;
+        }
+        break;
+
+    case function_start:
+        // align to the beginning of the line
+        *savedIndentDepth = *indentDepth = column(tokenAt(0).begin());
+        break;
+
+    case do_statement_while_paren_open:
+    case statement_with_condition_paren_open:
+    case signal_arglist_open:
+    case function_arglist_open:
+    case paren_open:
+    case condition_paren_open:
+        if (!lastToken)
+            *indentDepth = tokenPosition + 1;
+        else
+            *indentDepth += m_indentSize;
+        break;
+
+    case ternary_op:
+        if (!lastToken)
+            *indentDepth = tokenPosition + tk.length + 1;
+        else
+            *indentDepth += m_indentSize;
+        break;
+
+    case jsblock_open:
+        // closing brace should be aligned to case
+        if (parentState.type == case_cont) {
+            *savedIndentDepth = parentState.savedIndentDepth;
+            break;
+        }
+        // fallthrough
+    case substatement_open:
+        // special case for "foo: {" and "property int foo: {"
+        if (parentState.type == binding_assignment)
+            *savedIndentDepth = state(1).savedIndentDepth;
+        *indentDepth = *savedIndentDepth + m_indentSize;
+        break;
+
+    case substatement:
+        *indentDepth += m_indentSize;
+        break;
+
+    case objectliteral_open:
+        if (parentState.type == expression
+                || parentState.type == objectliteral_assignment) {
+            // undo the continuation indent of the expression
+            if (state(1).type == expression_or_label)
+                *indentDepth = state(1).savedIndentDepth;
+            else
+                *indentDepth = parentState.savedIndentDepth;
+            *savedIndentDepth = *indentDepth;
+        }
+        *indentDepth += m_indentSize;
+        break;
+
+
+    case statement_with_condition:
+    case try_statement:
+    case catch_statement:
+    case finally_statement:
+    case if_statement:
+    case do_statement:
+    case switch_statement:
+        if (firstToken || parentState.type == binding_assignment)
+            *savedIndentDepth = tokenPosition;
+        // ### continuation
+        *indentDepth = *savedIndentDepth; // + 2*m_indentSize;
+        // special case for 'else if'
+        if (!firstToken
+                && newState == if_statement
+                && parentState.type == substatement
+                && state(1).type == else_clause) {
+            *indentDepth = state(1).savedIndentDepth;
+            *savedIndentDepth = *indentDepth;
+        }
+        break;
+
+    case maybe_else:
+    case maybe_catch_or_finally: {
+        // set indent to where leave(true) would put it
+        int lastNonEndState = 0;
+        while (!isExpressionEndState(state(lastNonEndState + 1).type))
+            ++lastNonEndState;
+        *indentDepth = state(lastNonEndState).savedIndentDepth;
+        break;
+    }
+
+    case condition_open:
+        // fixed extra indent when continuing 'if (', but not for 'else if ('
+        if (tokenPosition <= *indentDepth + m_indentSize)
+            *indentDepth += 2*m_indentSize;
+        else
+            *indentDepth = tokenPosition + 1;
+        break;
+
+    case case_start:
+        *savedIndentDepth = tokenPosition;
+        break;
+
+    case case_cont:
+        *indentDepth += m_indentSize;
+        break;
+
+    case multiline_comment_start:
+        *indentDepth = tokenPosition + 2;
+        break;
+
+    case multiline_comment_cont:
+        *indentDepth = tokenPosition;
+        break;
+    }
+}
+
+void QtStyleCodeFormatter::adjustIndent(const QList<Token> &tokens, int startLexerState, int *indentDepth) const
+{
+    State topState = state();
+    State previousState = state(1);
+
+    // keep user-adjusted indent in multiline comments
+    if (topState.type == multiline_comment_start
+            || topState.type == multiline_comment_cont) {
+        if (!tokens.isEmpty()) {
+            *indentDepth = column(tokens.at(0).begin());
+            return;
+        }
+    }
+    // don't touch multi-line strings at all
+    if ((startLexerState & Scanner::MultiLineMask) == Scanner::MultiLineStringDQuote
+            || (startLexerState & Scanner::MultiLineMask) == Scanner::MultiLineStringSQuote) {
+        *indentDepth = -1;
+        return;
+    }
+
+    const int kind = extendedTokenKind(tokenAt(0));
+    switch (kind) {
+    case LeftBrace:
+        if (topState.type == substatement
+                || topState.type == binding_assignment
+                || topState.type == case_cont) {
+            *indentDepth = topState.savedIndentDepth;
+        }
+        break;
+    case RightBrace: {
+        if (topState.type == jsblock_open && previousState.type == case_cont) {
+            *indentDepth = previousState.savedIndentDepth;
+            break;
+        }
+        for (int i = 0; state(i).type != topmost_intro; ++i) {
+            const int type = state(i).type;
+            if (type == objectdefinition_open
+                    || type == jsblock_open
+                    || type == substatement_open
+                    || type == objectliteral_open) {
+                *indentDepth = state(i).savedIndentDepth;
+                break;
+            }
+        }
+        break;
+    }
+    case RightBracket:
+        for (int i = 0; state(i).type != topmost_intro; ++i) {
+            const int type = state(i).type;
+            if (type == bracket_open) {
+                *indentDepth = state(i).savedIndentDepth;
+                break;
+            }
+        }
+        break;
+
+    case LeftBracket:
+    case LeftParenthesis:
+    case Delimiter:
+        if (topState.type == expression_maybe_continuation)
+            *indentDepth = topState.savedIndentDepth;
+        break;
+
+    case Else:
+        if (topState.type == maybe_else) {
+            *indentDepth = state(1).savedIndentDepth;
+        } else if (topState.type == expression_maybe_continuation) {
+            bool hasElse = false;
+            for (int i = 1; state(i).type != topmost_intro; ++i) {
+                const int type = state(i).type;
+                if (type == else_clause)
+                    hasElse = true;
+                if (type == if_statement) {
+                    if (hasElse) {
+                        hasElse = false;
+                    } else {
+                        *indentDepth = state(i).savedIndentDepth;
+                        break;
+                    }
+                }
+            }
+        }
+        break;
+
+    case Catch:
+    case Finally:
+        if (topState.type == maybe_catch_or_finally)
+            *indentDepth = state(1).savedIndentDepth;
+        break;
+
+    case Colon:
+        if (topState.type == ternary_op) {
+            *indentDepth -= 2;
+        }
+        break;
+
+    case Question:
+        if (topState.type == expression_maybe_continuation)
+            *indentDepth = topState.savedIndentDepth;
+        break;
+
+    case Default:
+    case Case:
+        for (int i = 0; state(i).type != topmost_intro; ++i) {
+            const int type = state(i).type;
+            if (type == switch_statement || type == case_cont) {
+                *indentDepth = state(i).savedIndentDepth;
+                break;
+            } else if (type == topmost_intro) {
+                break;
+            }
+        }
+        break;
+    }
+}
