@@ -2,43 +2,43 @@
 **
 ** This file is part of Qt Creator
 **
-** Copyright (c) 2011 Nokia Corporation and/or its subsidiary(-ies).
+** Copyright (c) 2012 Nokia Corporation and/or its subsidiary(-ies).
 **
 ** Contact: Nokia Corporation (qt-info@nokia.com)
 **
-** No Commercial Usage
-**
-** This file contains pre-release code and may not be distributed.
-** You may use this file in accordance with the terms and conditions
-** contained in the Technology Preview License Agreement accompanying
-** this package.
 **
 ** GNU Lesser General Public License Usage
 **
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU Lesser General Public License version 2.1 requirements
-** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** This file may be used under the terms of the GNU Lesser General Public
+** License version 2.1 as published by the Free Software Foundation and
+** appearing in the file LICENSE.LGPL included in the packaging of this file.
+** Please review the following information to ensure the GNU Lesser General
+** Public License version 2.1 requirements will be met:
+** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
 ** In addition, as a special exception, Nokia gives you certain additional
-** rights.  These rights are described in the Nokia Qt LGPL Exception
+** rights. These rights are described in the Nokia Qt LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
+** Other Usage
+**
+** Alternatively, this file may be used in accordance with the terms and
+** conditions contained in a signed written agreement between you and Nokia.
+**
 ** If you have questions regarding the use of this file, please contact
 ** Nokia at qt-info@nokia.com.
 **
 **************************************************************************/
 
 #include "qmljsdelta.h"
+#include "qmljsutils.h"
 #include <qmljs/parser/qmljsast_p.h>
 #include <qmljs/parser/qmljsastvisitor_p.h>
 
 
 #include <typeinfo>
-#include <QtCore/QDebug>
-#include <QtCore/QStringList>
+#include <QDebug>
+#include <QStringList>
 
 
 using namespace QmlJS::AST;
@@ -84,11 +84,11 @@ static QString label(UiQualifiedId *id)
 {
     QString str;
     for (; id ; id = id->next) {
-        if (!id->name)
+        if (id->name.isEmpty())
             return QString();
         if (!str.isEmpty())
             str += QLatin1Char('.');
-        str += id->name->asString();
+        str += id->name;
     }
     return str;
 }
@@ -97,7 +97,7 @@ static QString label(UiQualifiedId *id)
 static QString label(UiObjectMember *member, Document::Ptr doc)
 {
     QString str;
-    if(!member)
+    if (!member)
         return str;
 
     if (UiObjectDefinition* foo = cast<UiObjectDefinition *>(member)) {
@@ -169,14 +169,8 @@ struct Map {
 static QList<UiObjectMember *> children(UiObjectMember *ast)
 {
     QList<UiObjectMember *> ret;
-    if (UiObjectDefinition* foo = cast<UiObjectDefinition *>(ast)) {
-        UiObjectMemberList* list = foo->initializer->members;
-        while (list) {
-            ret.append(list->member);
-            list = list->next;
-        }
-    } else if(UiObjectBinding *foo = cast<UiObjectBinding *>(ast)) {
-        UiObjectMemberList* list = foo->initializer->members;
+    if (UiObjectInitializer * foo = QmlJS::initializerOfObject(ast)) {
+        UiObjectMemberList* list = foo->members;
         while (list) {
             ret.append(list->member);
             list = list->next;
@@ -285,10 +279,10 @@ static QString _propertyName(UiQualifiedId *id)
     QString s;
 
     for (; id; id = id->next) {
-        if (! id->name)
+        if (id->name.isEmpty())
             return QString();
 
-        s += id->name->asString();
+        s += id->name;
 
         if (id->next)
             s += QLatin1Char('.');
@@ -301,7 +295,7 @@ static QString _methodName(UiSourceElement *source)
 {
     if (source) {
         if (FunctionDeclaration *declaration = cast<FunctionDeclaration*>(source->sourceElement)) {
-            return declaration->name->asString();
+            return declaration->name.toString();
         }
     }
     return QString();
@@ -309,10 +303,8 @@ static QString _methodName(UiSourceElement *source)
 
 static UiObjectMemberList *objectMembers(UiObjectMember *object)
 {
-    if (UiObjectDefinition *def = cast<UiObjectDefinition *>(object))
-        return def->initializer->members;
-    else if (UiObjectBinding *binding = cast<UiObjectBinding *>(object))
-        return binding->initializer->members;
+    if (UiObjectInitializer *init = QmlJS::initializerOfObject(object))
+        return init->members;
 
     return 0;
 }
@@ -351,12 +343,32 @@ void Delta::insert(UiObjectMember *member, UiObjectMember *parentMember, const Q
     if (!member || !parentMember)
         return;
 
+    bool accepted = false;
+    // initialized with garbage
+    unsigned begin = 1, end = 2, startColumn = 3, startLine = 4;
+
     // create new objects
     if (UiObjectDefinition* uiObjectDef = cast<UiObjectDefinition *>(member)) {
-        unsigned begin = uiObjectDef->firstSourceLocation().begin();
-        unsigned end = uiObjectDef->lastSourceLocation().end();
-        QString qmlText = QString(uiObjectDef->firstSourceLocation().startColumn - 1, QLatin1Char(' '));
+        begin = uiObjectDef->firstSourceLocation().begin();
+        end = uiObjectDef->lastSourceLocation().end();
+        startColumn = uiObjectDef->firstSourceLocation().startColumn;
+        startLine = uiObjectDef->firstSourceLocation().startLine;
+        accepted = true;
+    }
+
+    if (UiObjectBinding* uiObjectBind = cast<UiObjectBinding *>(member)) {
+        SourceLocation definitionLocation = uiObjectBind->qualifiedTypeNameId->identifierToken;
+        begin = definitionLocation.begin();
+        end = uiObjectBind->lastSourceLocation().end();
+        startColumn = definitionLocation.startColumn;
+        startLine = definitionLocation.startLine;
+        accepted = true;
+    }
+
+    if (accepted) {
+        QString qmlText = QString(startColumn - 1, QLatin1Char(' '));
         qmlText += doc->source().midRef(begin, end - begin);
+
         QStringList importList;
         for (UiImportList *it = doc->qmlProgram()->imports; it; it = it->next) {
             if (!it->import)
@@ -368,18 +380,25 @@ void Delta::insert(UiObjectMember *member, UiObjectMember *parentMember, const Q
         }
 
         // encode editorRevision, lineNumber in URL. See ClientProxy::buildDebugIdHashRecursive
-        QString filename = QLatin1String("file://") + doc->fileName() + QLatin1Char('_') + QString::number(doc->editorRevision())
-                         + QLatin1Char(':') + QString::number(uiObjectDef->firstSourceLocation().startLine-importList.count());
+        // Also, a relative URL is expected (no "file://" prepending!)
+        QString filename =  doc->fileName() + QLatin1Char('_') + QString::number(doc->editorRevision())
+                         + QLatin1Char(':') + QString::number(startLine-importList.count());
         foreach(DebugId debugId, debugReferences) {
             if (debugId != -1) {
-                createObject(qmlText, debugId, importList, filename);
+                int order = 0;
+                // skip children which are not objects
+                foreach (const UiObjectMember *child, children(parentMember)) {
+                    if (child == member) break;
+                    if (child->kind == AST::Node::Kind_UiObjectDefinition)
+                        order++;
+                }
+                createObject(qmlText, debugId, importList, filename, order);
             }
         }
         newObjects += member;
     }
 }
 
-
 void Delta::update(UiObjectMember* oldObject, const QmlJS::Document::Ptr& oldDoc,
                    UiObjectMember* newObject, const QmlJS::Document::Ptr& newDoc,
                    const QList<DebugId>& debugReferences)
@@ -461,6 +480,9 @@ Delta::DebugIdMap Delta::operator()(const Document::Ptr &doc1, const Document::P
     Q_ASSERT(doc1->qmlProgram());
     Q_ASSERT(doc2->qmlProgram());
 
+    m_previousDoc = doc1;
+    m_currentDoc = doc2;
+
     Delta::DebugIdMap newDebuggIds;
 
     Map M = buildMapping(doc1, doc2);
@@ -482,11 +504,16 @@ Delta::DebugIdMap Delta::operator()(const Document::Ptr &doc1, const Document::P
 
         if (!M.way2.contains(y)) {
             UiObjectMember* parent = parents2.parent.value(y);
-            if (!M.way2.contains(parent))
-                continue;
-            if (debug)
-                qDebug () << "Delta::operator():  insert " << label(y, doc2) << " to " << label(parent, doc2);
-            insert(y, parent, newDebuggIds.value(parent), doc2);
+            if (parent) {
+                if ( parent->kind == QmlJS::AST::Node::Kind_UiArrayBinding )
+                    parent = parents2.parent.value(parent);
+
+                if (M.way2.contains(parent) && newDebuggIds.value(parent).count() > 0) {
+                    if (debug)
+                        qDebug () << "Delta::operator():  insert " << label(y, doc2) << " to " << label(parent, doc2);
+                    insert(y, parent, newDebuggIds.value(parent), doc2);
+                }
+            }
             continue;
         }
         UiObjectMember *x = M.way2[y];
@@ -498,6 +525,8 @@ Delta::DebugIdMap Delta::operator()(const Document::Ptr &doc1, const Document::P
                 updateIds = debugIds[x];
                 newDebuggIds[y] = updateIds;
             }
+            if (debug)
+                qDebug () << "Delta::operator(): update " << label(x,doc1);
             update(x, doc1, y, doc2, updateIds);
         }
         //qDebug() << "Delta::operator():  match "<< label(x, doc1) << "with parent " << label(parents1.parent.value(x), doc1)
@@ -540,7 +569,7 @@ Document::Ptr Delta::previousDocument() const
     return m_previousDoc;
 }
 
-void Delta::createObject(const QString &, DebugId, const QStringList &, const QString&)
+void Delta::createObject(const QString &, DebugId, const QStringList &, const QString&, int)
 {}
 void Delta::removeObject(int)
 {}
