@@ -2,7 +2,7 @@
 **
 ** This file is part of Qt Creator
 **
-** Copyright (c) 2011 Nokia Corporation and/or its subsidiary(-ies).
+** Copyright (c) 2012 Nokia Corporation and/or its subsidiary(-ies).
 **
 ** Contact: Nokia Corporation (qt-info@nokia.com)
 **
@@ -39,12 +39,12 @@
 #include <qmljs/qmljs_global.h>
 #include <qmljs/parser/qmljsastfwd_p.h>
 
-#include <QtCore/QFileInfoList>
-#include <QtCore/QList>
-#include <QtCore/QString>
-#include <QtCore/QHash>
-#include <QtCore/QSet>
-#include <QtCore/QMutex>
+#include <QFileInfoList>
+#include <QList>
+#include <QString>
+#include <QHash>
+#include <QSet>
+#include <QMutex>
 
 namespace QmlJS {
 
@@ -58,6 +58,7 @@ class ValueOwner;
 class Value;
 class NullValue;
 class UndefinedValue;
+class UnknownValue;
 class NumberValue;
 class IntValue;
 class RealValue;
@@ -75,6 +76,12 @@ class JSImportScope;
 class Context;
 typedef QSharedPointer<const Context> ContextPtr;
 class ReferenceContext;
+class CppComponentValue;
+class ASTObjectValue;
+class QmlEnumValue;
+class QmlPrototypeReference;
+class ASTPropertyReference;
+class ASTSignal;
 
 typedef QList<const Value *> ValueList;
 
@@ -89,6 +96,7 @@ public:
 
     virtual void visit(const NullValue *);
     virtual void visit(const UndefinedValue *);
+    virtual void visit(const UnknownValue *);
     virtual void visit(const NumberValue *);
     virtual void visit(const BooleanValue *);
     virtual void visit(const StringValue *);
@@ -113,6 +121,7 @@ public:
 
     virtual const NullValue *asNullValue() const;
     virtual const UndefinedValue *asUndefinedValue() const;
+    virtual const UnknownValue *asUnknownValue() const;
     virtual const NumberValue *asNumberValue() const;
     virtual const IntValue *asIntValue() const;
     virtual const RealValue *asRealValue() const;
@@ -124,13 +133,24 @@ public:
     virtual const Reference *asReference() const;
     virtual const ColorValue *asColorValue() const;
     virtual const AnchorLineValue *asAnchorLineValue() const;
+    virtual const CppComponentValue *asCppComponentValue() const;
+    virtual const ASTObjectValue *asAstObjectValue() const;
+    virtual const QmlEnumValue *asQmlEnumValue() const;
+    virtual const QmlPrototypeReference *asQmlPrototypeReference() const;
+    virtual const ASTPropertyReference *asAstPropertyReference() const;
+    virtual const ASTSignal *asAstSignal() const;
 
     virtual void accept(ValueVisitor *) const = 0;
 
     virtual bool getSourceLocation(QString *fileName, int *line, int *column) const;
 };
 
-template <typename _RetTy> _RetTy value_cast(const Value *v);
+template <typename _RetTy> const _RetTy *value_cast(const Value *)
+{
+    // Produce a good error message if a specialization is missing.
+    _RetTy::ERROR_MissingValueCastSpecialization();
+    return 0;
+}
 
 template <> Q_INLINE_TEMPLATE const NullValue *value_cast(const Value *v)
 {
@@ -144,6 +164,12 @@ template <> Q_INLINE_TEMPLATE const UndefinedValue *value_cast(const Value *v)
     else   return 0;
 }
 
+template <> Q_INLINE_TEMPLATE const UnknownValue *value_cast(const Value *v)
+{
+    if (v) return v->asUnknownValue();
+    else   return 0;
+}
+
 template <> Q_INLINE_TEMPLATE const NumberValue *value_cast(const Value *v)
 {
     if (v) return v->asNumberValue();
@@ -210,6 +236,42 @@ template <> Q_INLINE_TEMPLATE const AnchorLineValue *value_cast(const Value *v)
     else   return 0;
 }
 
+template <> Q_INLINE_TEMPLATE const CppComponentValue *value_cast(const Value *v)
+{
+    if (v) return v->asCppComponentValue();
+    else   return 0;
+}
+
+template <> Q_INLINE_TEMPLATE const ASTObjectValue *value_cast(const Value *v)
+{
+    if (v) return v->asAstObjectValue();
+    else   return 0;
+}
+
+template <> Q_INLINE_TEMPLATE const QmlEnumValue *value_cast(const Value *v)
+{
+    if (v) return v->asQmlEnumValue();
+    else   return 0;
+}
+
+template <> Q_INLINE_TEMPLATE const QmlPrototypeReference *value_cast(const Value *v)
+{
+    if (v) return v->asQmlPrototypeReference();
+    else   return 0;
+}
+
+template <> Q_INLINE_TEMPLATE const ASTPropertyReference *value_cast(const Value *v)
+{
+    if (v) return v->asAstPropertyReference();
+    else   return 0;
+}
+
+template <> Q_INLINE_TEMPLATE const ASTSignal *value_cast(const Value *v)
+{
+    if (v) return v->asAstSignal();
+    else   return 0;
+}
+
 ////////////////////////////////////////////////////////////////////////////////
 // Value nodes
 ////////////////////////////////////////////////////////////////////////////////
@@ -227,6 +289,13 @@ public:
     virtual void accept(ValueVisitor *visitor) const;
 };
 
+class QMLJS_EXPORT UnknownValue: public Value
+{
+public:
+    virtual const UnknownValue *asUnknownValue() const;
+    virtual void accept(ValueVisitor *) const;
+};
+
 class QMLJS_EXPORT NumberValue: public Value
 {
 public:
@@ -394,41 +463,43 @@ private:
     Error m_error;
 };
 
-class QmlObjectValue;
-
 class QMLJS_EXPORT QmlEnumValue: public NumberValue
 {
 public:
-    QmlEnumValue(const QmlObjectValue *owner, int index);
+    QmlEnumValue(const CppComponentValue *owner, int index);
     virtual ~QmlEnumValue();
 
+    virtual const QmlEnumValue *asQmlEnumValue() const;
+
     QString name() const;
     QStringList keys() const;
-    const QmlObjectValue *owner() const;
+    const CppComponentValue *owner() const;
 
 private:
-    const QmlObjectValue *_owner;
+    const CppComponentValue *_owner;
     int _enumIndex;
 };
 
 
 // A ObjectValue based on a FakeMetaObject.
-// May only have other QmlObjectValues as ancestors.
-class QMLJS_EXPORT QmlObjectValue: public ObjectValue
+// May only have other CppComponentValue as ancestors.
+class QMLJS_EXPORT CppComponentValue: public ObjectValue
 {
 public:
-    QmlObjectValue(LanguageUtils::FakeMetaObject::ConstPtr metaObject, const QString &className,
+    CppComponentValue(LanguageUtils::FakeMetaObject::ConstPtr metaObject, const QString &className,
                    const QString &moduleName, const LanguageUtils::ComponentVersion &componentVersion,
                    const LanguageUtils::ComponentVersion &importVersion, int metaObjectRevision,
                    ValueOwner *valueOwner);
-    virtual ~QmlObjectValue();
+    virtual ~CppComponentValue();
+
+    virtual const CppComponentValue *asCppComponentValue() const;
 
     virtual void processMembers(MemberProcessor *processor) const;
-    const Value *propertyValue(const LanguageUtils::FakeMetaProperty &prop) const;
+    const Value *valueForCppName(const QString &typeName) const;
 
     using ObjectValue::prototype;
-    const QmlObjectValue *prototype() const;
-    QList<const QmlObjectValue *> prototypes() const;
+    const CppComponentValue *prototype() const;
+    QList<const CppComponentValue *> prototypes() const;
 
     LanguageUtils::FakeMetaObject::ConstPtr metaObject() const;
 
@@ -444,8 +515,10 @@ public:
     bool hasLocalProperty(const QString &typeName) const;
     bool hasProperty(const QString &typeName) const;
 
-    LanguageUtils::FakeMetaEnum getEnum(const QString &typeName, const QmlObjectValue **foundInScope = 0) const;
-    const QmlEnumValue *getEnumValue(const QString &typeName, const QmlObjectValue **foundInScope = 0) const;
+    LanguageUtils::FakeMetaEnum getEnum(const QString &typeName, const CppComponentValue **foundInScope = 0) const;
+    const QmlEnumValue *getEnumValue(const QString &typeName, const CppComponentValue **foundInScope = 0) const;
+
+    const ObjectValue *signalScope(const QString &signalName) const;
 protected:
     bool isDerivedFrom(LanguageUtils::FakeMetaObject::ConstPtr base) const;
 
@@ -458,63 +531,36 @@ private:
     const LanguageUtils::ComponentVersion _componentVersion;
     const LanguageUtils::ComponentVersion _importVersion;
     mutable QAtomicPointer< QList<const Value *> > _metaSignatures;
+    mutable QAtomicPointer< QHash<QString, const ObjectValue *> > _signalScopes;
     QHash<QString, const QmlEnumValue * > _enums;
     int _metaObjectRevision;
 };
 
-class QMLJS_EXPORT Activation
-{
-public:
-    explicit Activation(Context *parentContext = 0);
-    virtual ~Activation();
-
-    Context *context() const;
-    Context *parentContext() const;
-
-    bool calledAsConstructor() const;
-    void setCalledAsConstructor(bool calledAsConstructor);
-
-    bool calledAsFunction() const;
-    void setCalledAsFunction(bool calledAsFunction);
-
-    ObjectValue *thisObject() const;
-    void setThisObject(ObjectValue *thisObject);
-
-    ValueList arguments() const;
-    void setArguments(const ValueList &arguments);
-
-private:
-    ObjectValue *_thisObject;
-    ValueList _arguments;
-    bool _calledAsFunction;
-    Context *_parentContext;
-};
-
-
 class QMLJS_EXPORT FunctionValue: public ObjectValue
 {
 public:
     FunctionValue(ValueOwner *valueOwner);
     virtual ~FunctionValue();
 
-    // [[construct]]
-    const Value *construct(const ValueList &actuals = ValueList()) const;
-
-    // [[call]]
-    const Value *call(const ValueList &actuals = ValueList()) const;
-
-    const Value *call(const ObjectValue *thisObject,
-                      const ValueList &actuals = ValueList()) const;
-
-
     virtual const Value *returnValue() const;
 
-    virtual int argumentCount() const;
-    virtual const Value *argument(int index) const;
+    // Access to the names of arguments
+    // Named arguments can be optional (usually known for builtins only)
+    virtual int namedArgumentCount() const;
     virtual QString argumentName(int index) const;
+
+    // The number of optional named arguments
+    // Example: JSON.stringify(value[, replacer[, space]])
+    //          has namedArgumentCount = 3
+    //          and optionalNamedArgumentCount = 2
+    virtual int optionalNamedArgumentCount() const;
+
+    // Whether the function accepts an unlimited number of arguments
+    // after the named ones. Defaults to false.
+    // Example: Math.max(...)
     virtual bool isVariadic() const;
 
-    virtual const Value *invoke(const Activation *activation) const;
+    virtual const Value *argument(int index) const;
 
     // Value interface
     virtual const FunctionValue *asFunctionValue() const;
@@ -529,18 +575,23 @@ public:
 
     void addArgument(const Value *argument, const QString &name = QString());
     void setReturnValue(const Value *returnValue);
+    void setVariadic(bool variadic);
+    void setOptionalNamedArgumentCount(int count);
 
     // FunctionValue interface
     virtual const Value *returnValue() const;
-    virtual int argumentCount() const;
+    virtual int namedArgumentCount() const;
+    virtual int optionalNamedArgumentCount() const;
     virtual const Value *argument(int index) const;
     virtual QString argumentName(int index) const;
-    virtual const Value *invoke(const Activation *activation) const;
+    virtual bool isVariadic() const;
 
 private:
     ValueList _arguments;
     QStringList _argumentNames;
     const Value *_returnValue;
+    int _optionalNamedArgumentCount;
+    bool _isVariadic;
 };
 
 
@@ -566,7 +617,7 @@ public:
     static void parseQmlTypeDescriptions(
         const QByteArray &qmlTypes,
         BuiltinObjects *newObjects,
-        QString *errorMessage, QString *warningMessage);
+        QList<ModuleApiInfo> *newModuleApis, QString *errorMessage, QString *warningMessage);
 };
 
 class QMLJS_EXPORT CppQmlTypes
@@ -582,21 +633,25 @@ public:
     template <typename T>
     void load(const T &fakeMetaObjects, const QString &overridePackage = QString());
 
-    QList<const QmlObjectValue *> createObjectsForImport(const QString &package, LanguageUtils::ComponentVersion version);
+    QList<const CppComponentValue *> createObjectsForImport(const QString &package, LanguageUtils::ComponentVersion version);
     bool hasModule(const QString &module) const;
 
     static QString qualifiedName(const QString &module, const QString &type,
                                  LanguageUtils::ComponentVersion version);
-    const QmlObjectValue *objectByQualifiedName(const QString &fullyQualifiedName) const;
-    const QmlObjectValue *objectByQualifiedName(
+    const CppComponentValue *objectByQualifiedName(const QString &fullyQualifiedName) const;
+    const CppComponentValue *objectByQualifiedName(
             const QString &package, const QString &type,
             LanguageUtils::ComponentVersion version) const;
-    const QmlObjectValue *objectByCppName(const QString &cppName) const;
+    const CppComponentValue *objectByCppName(const QString &cppName) const;
+
+    void setCppContextProperties(const ObjectValue *contextProperties);
+    const ObjectValue *cppContextProperties() const;
 
 private:
-    // "Package.CppName ImportVersion" ->  QmlObjectValue
-    QHash<QString, const QmlObjectValue *> _objectsByQualifiedName;
+    // "Package.CppName ImportVersion" ->  CppComponentValue
+    QHash<QString, const CppComponentValue *> _objectsByQualifiedName;
     QHash<QString, QSet<LanguageUtils::FakeMetaObject::ConstPtr> > _fakeMetaObjectsByPackage;
+    const ObjectValue *_cppContextProperties;
     ValueOwner *_valueOwner;
 };
 
@@ -695,6 +750,8 @@ public:
     QmlPrototypeReference(AST::UiQualifiedId *qmlTypeName, const Document *doc, ValueOwner *valueOwner);
     virtual ~QmlPrototypeReference();
 
+    virtual const QmlPrototypeReference *asQmlPrototypeReference() const;
+
     AST::UiQualifiedId *qmlTypeName() const;
 
 private:    
@@ -723,6 +780,7 @@ class QMLJS_EXPORT ASTFunctionValue: public FunctionValue
     AST::FunctionExpression *_ast;
     const Document *_doc;
     QList<QString> _argumentNames;
+    bool _isVariadic;
 
 public:
     ASTFunctionValue(AST::FunctionExpression *ast, const Document *doc, ValueOwner *valueOwner);
@@ -730,9 +788,7 @@ public:
 
     AST::FunctionExpression *ast() const;
 
-    virtual const Value *returnValue() const;
-    virtual int argumentCount() const;
-    virtual const Value *argument(int) const;
+    virtual int namedArgumentCount() const;
     virtual QString argumentName(int index) const;
     virtual bool isVariadic() const;
 
@@ -749,6 +805,8 @@ public:
     ASTPropertyReference(AST::UiPublicMember *ast, const Document *doc, ValueOwner *valueOwner);
     virtual ~ASTPropertyReference();
 
+    virtual const ASTPropertyReference *asAstPropertyReference() const;
+
     AST::UiPublicMember *ast() const { return _ast; }
     QString onChangedSlotName() const { return _onChangedSlotName; }
 
@@ -763,16 +821,20 @@ class QMLJS_EXPORT ASTSignal: public FunctionValue
     AST::UiPublicMember *_ast;
     const Document *_doc;
     QString _slotName;
+    const ObjectValue *_bodyScope;
 
 public:
     ASTSignal(AST::UiPublicMember *ast, const Document *doc, ValueOwner *valueOwner);
     virtual ~ASTSignal();
 
+    virtual const ASTSignal *asAstSignal() const;
+
     AST::UiPublicMember *ast() const { return _ast; }
     QString slotName() const { return _slotName; }
+    const ObjectValue *bodyScope() const { return _bodyScope; }
 
     // FunctionValue interface
-    virtual int argumentCount() const;
+    virtual int namedArgumentCount() const;
     virtual const Value *argument(int index) const;
     virtual QString argumentName(int index) const;
 
@@ -796,6 +858,8 @@ public:
                    ValueOwner *valueOwner);
     virtual ~ASTObjectValue();
 
+    virtual const ASTObjectValue *asAstObjectValue() const;
+
     bool getSourceLocation(QString *fileName, int *line, int *column) const;
     virtual void processMembers(MemberProcessor *processor) const;
 
@@ -863,6 +927,8 @@ public:
     ImportInfo info;
     // uri imports: path to library, else empty
     QString libraryPath;
+    // whether the import succeeded
+    bool valid;
 };
 
 class Imports;
@@ -901,9 +967,12 @@ public:
     Imports(ValueOwner *valueOwner);
 
     void append(const Import &import);
+    void setImportFailed();
 
     ImportInfo info(const QString &name, const Context *context) const;
     QString nameForImportedObject(const ObjectValue *value, const Context *context) const;
+    bool importFailed() const;
+
     QList<Import> all() const;
 
     const TypeScope *typeScope() const;
@@ -919,6 +988,7 @@ private:
     QList<Import> _imports;
     TypeScope *_typeScope;
     JSImportScope *_jsImportScope;
+    bool _importFailed;
 };
 
 } // namespace QmlJS
