@@ -2,28 +2,31 @@
 **
 ** This file is part of Qt Creator
 **
-** Copyright (c) 2010 Nokia Corporation and/or its subsidiary(-ies).
+** Copyright (c) 2012 Nokia Corporation and/or its subsidiary(-ies).
 **
 ** Contact: Nokia Corporation (qt-info@nokia.com)
 **
-** Commercial Usage
-**
-** Licensees holding valid Qt Commercial licenses may use this file in
-** accordance with the Qt Commercial License Agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Nokia.
 **
 ** GNU Lesser General Public License Usage
 **
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU Lesser General Public License version 2.1 requirements
-** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** This file may be used under the terms of the GNU Lesser General Public
+** License version 2.1 as published by the Free Software Foundation and
+** appearing in the file LICENSE.LGPL included in the packaging of this file.
+** Please review the following information to ensure the GNU Lesser General
+** Public License version 2.1 requirements will be met:
+** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** If you are unsure which license is appropriate for your use, please
-** contact the sales department at http://qt.nokia.com/contact.
+** In addition, as a special exception, Nokia gives you certain additional
+** rights. These rights are described in the Nokia Qt LGPL Exception
+** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+**
+** Other Usage
+**
+** Alternatively, this file may be used in accordance with the terms and
+** conditions contained in a signed written agreement between you and Nokia.
+**
+** If you have questions regarding the use of this file, please contact
+** Nokia at qt-info@nokia.com.
 **
 **************************************************************************/
 
@@ -31,12 +34,13 @@
 
 #include <qmljs/parser/qmljsast_p.h>
 #include <qmljs/parser/qmljsengine_p.h>
+#include <qmljs/qmljsutils.h>
 #include <utils/changeset.h>
 
 // ### FIXME: remove these includes:
-#include <QtGui/QTextBlock>
-#include <QtGui/QTextCursor>
-#include <QtGui/QTextDocument>
+#include <QTextBlock>
+#include <QTextCursor>
+#include <QTextDocument>
 
 using namespace QmlJS;
 using namespace QmlJS::AST;
@@ -52,14 +56,23 @@ Rewriter::Rewriter(const QString &originalText,
     Q_ASSERT(changeSet);
 }
 
-void Rewriter::addBinding(AST::UiObjectInitializer *ast,
-                          const QString &propertyName,
-                          const QString &propertyValue,
-                          BindingType bindingType)
+Rewriter::Range Rewriter::addBinding(AST::UiObjectInitializer *ast,
+                                     const QString &propertyName,
+                                     const QString &propertyValue,
+                                     BindingType bindingType)
 {
     UiObjectMemberList *insertAfter = searchMemberToInsertAfter(ast->members,
                                                                 propertyName,
                                                                 m_propertyOrder);
+    return addBinding(ast, propertyName, propertyValue, bindingType, insertAfter);
+}
+
+Rewriter::Range Rewriter::addBinding(AST::UiObjectInitializer *ast,
+                                     const QString &propertyName,
+                                     const QString &propertyValue,
+                                     BindingType bindingType,
+                                     UiObjectMemberList *insertAfter)
+{
     SourceLocation endOfPreviousMember;
     SourceLocation startOfNextMember;
 
@@ -127,14 +140,16 @@ void Rewriter::addBinding(AST::UiObjectInitializer *ast,
         newPropertyTemplate.prepend(QLatin1Char('\n'));
     }
 
-    m_changeSet->replace(endOfPreviousMember.end(), 0,
+    m_changeSet->insert(endOfPreviousMember.end(),
                          newPropertyTemplate.arg(propertyName, propertyValue));
+
+    return Range(endOfPreviousMember.end(), endOfPreviousMember.end());
 }
 
 UiObjectMemberList *Rewriter::searchMemberToInsertAfter(UiObjectMemberList *members,
                                                         const QStringList &propertyOrder)
 {
-    const int objectDefinitionInsertionPoint = propertyOrder.indexOf(QString::null);
+    const int objectDefinitionInsertionPoint = propertyOrder.indexOf(QString());
 
     UiObjectMemberList *lastObjectDef = 0;
     UiObjectMemberList *lastNonObjectDef = 0;
@@ -146,11 +161,44 @@ UiObjectMemberList *Rewriter::searchMemberToInsertAfter(UiObjectMemberList *memb
         if (cast<UiObjectDefinition*>(member))
             lastObjectDef = iter;
         else if (UiArrayBinding *arrayBinding = cast<UiArrayBinding*>(member))
-            idx = propertyOrder.indexOf(flatten(arrayBinding->qualifiedId));
+            idx = propertyOrder.indexOf(toString(arrayBinding->qualifiedId));
         else if (UiObjectBinding *objectBinding = cast<UiObjectBinding*>(member))
-            idx = propertyOrder.indexOf(flatten(objectBinding->qualifiedId));
+            idx = propertyOrder.indexOf(toString(objectBinding->qualifiedId));
         else if (UiScriptBinding *scriptBinding = cast<UiScriptBinding*>(member))
-            idx = propertyOrder.indexOf(flatten(scriptBinding->qualifiedId));
+            idx = propertyOrder.indexOf(toString(scriptBinding->qualifiedId));
+        else if (cast<UiPublicMember*>(member))
+            idx = propertyOrder.indexOf(QLatin1String("property"));
+
+        if (idx < objectDefinitionInsertionPoint)
+            lastNonObjectDef = iter;
+    }
+
+    if (lastObjectDef)
+        return lastObjectDef;
+    else
+        return lastNonObjectDef;
+}
+
+UiArrayMemberList *Rewriter::searchMemberToInsertAfter(UiArrayMemberList *members,
+                                                        const QStringList &propertyOrder)
+{
+    const int objectDefinitionInsertionPoint = propertyOrder.indexOf(QString());
+
+    UiArrayMemberList *lastObjectDef = 0;
+    UiArrayMemberList *lastNonObjectDef = 0;
+
+    for (UiArrayMemberList *iter = members; iter; iter = iter->next) {
+        UiObjectMember *member = iter->member;
+        int idx = -1;
+
+        if (cast<UiObjectDefinition*>(member))
+            lastObjectDef = iter;
+        else if (UiArrayBinding *arrayBinding = cast<UiArrayBinding*>(member))
+            idx = propertyOrder.indexOf(toString(arrayBinding->qualifiedId));
+        else if (UiObjectBinding *objectBinding = cast<UiObjectBinding*>(member))
+            idx = propertyOrder.indexOf(toString(objectBinding->qualifiedId));
+        else if (UiScriptBinding *scriptBinding = cast<UiScriptBinding*>(member))
+            idx = propertyOrder.indexOf(toString(scriptBinding->qualifiedId));
         else if (cast<UiPublicMember*>(member))
             idx = propertyOrder.indexOf(QLatin1String("property"));
 
@@ -177,13 +225,13 @@ UiObjectMemberList *Rewriter::searchMemberToInsertAfter(UiObjectMemberList *memb
         UiObjectMember *member = iter->member;
 
         if (UiArrayBinding *arrayBinding = cast<UiArrayBinding*>(member))
-            orderedMembers[flatten(arrayBinding->qualifiedId)] = iter;
+            orderedMembers[toString(arrayBinding->qualifiedId)] = iter;
         else if (UiObjectBinding *objectBinding = cast<UiObjectBinding*>(member))
-            orderedMembers[flatten(objectBinding->qualifiedId)] = iter;
+            orderedMembers[toString(objectBinding->qualifiedId)] = iter;
         else if (cast<UiObjectDefinition*>(member))
             orderedMembers[QString::null] = iter;
         else if (UiScriptBinding *scriptBinding = cast<UiScriptBinding*>(member))
-            orderedMembers[flatten(scriptBinding->qualifiedId)] = iter;
+            orderedMembers[toString(scriptBinding->qualifiedId)] = iter;
         else if (cast<UiPublicMember*>(member))
             orderedMembers[QLatin1String("property")] = iter;
     }
@@ -204,24 +252,10 @@ UiObjectMemberList *Rewriter::searchMemberToInsertAfter(UiObjectMemberList *memb
     return 0;
 }
 
-QString Rewriter::flatten(UiQualifiedId *first)
-{
-    QString flatId;
-
-    for (UiQualifiedId* current = first; current; current = current->next) {
-        if (current != first)
-            flatId += '.';
-
-        flatId += current->name->asString();
-    }
-
-    return flatId;
-}
-
-void Rewriter::changeProperty(UiObjectInitializer *ast,
-                              const QString &propertyName,
-                              const QString &newValue,
-                              BindingType binding)
+void Rewriter::changeBinding(UiObjectInitializer *ast,
+                             const QString &propertyName,
+                             const QString &newValue,
+                             BindingType binding)
 {
     QString prefix, suffix;
     int dotIdx = propertyName.indexOf(QLatin1Char('.'));
@@ -257,8 +291,8 @@ void Rewriter::changeProperty(UiObjectInitializer *ast,
         // for grouped properties:
         else if (!prefix.isEmpty()) {
             if (UiObjectDefinition *def = cast<UiObjectDefinition *>(member)) {
-                if (flatten(def->qualifiedTypeNameId) == prefix) {
-                    changeProperty(def->initializer, suffix, newValue, binding);
+                if (toString(def->qualifiedTypeNameId) == prefix) {
+                    changeBinding(def->initializer, suffix, newValue, binding);
                 }
             }
         }
@@ -282,12 +316,12 @@ void Rewriter::replaceMemberValue(UiObjectMember *propertyMember,
         startOffset = arrayBinding->lbracketToken.offset;
         endOffset = arrayBinding->rbracketToken.end();
     } else if (UiPublicMember *publicMember = AST::cast<UiPublicMember*>(propertyMember)) {
-        if (publicMember->expression) {
-            startOffset = publicMember->expression->firstSourceLocation().offset;
+        if (publicMember->statement) {
+            startOffset = publicMember->statement->firstSourceLocation().offset;
             if (publicMember->semicolonToken.isValid())
                 endOffset = publicMember->semicolonToken.end();
             else
-                endOffset = publicMember->expression->lastSourceLocation().offset;
+                endOffset = publicMember->statement->lastSourceLocation().offset;
         } else {
             startOffset = publicMember->lastSourceLocation().end();
             endOffset = startOffset;
@@ -302,20 +336,20 @@ void Rewriter::replaceMemberValue(UiObjectMember *propertyMember,
     if (needsSemicolon)
         replacement += ';';
 
-    m_changeSet->replace(startOffset, endOffset - startOffset, replacement);
+    m_changeSet->replace(startOffset, endOffset, replacement);
 }
 
 bool Rewriter::isMatchingPropertyMember(const QString &propertyName,
                                         UiObjectMember *member)
 {
     if (UiPublicMember *publicMember = cast<UiPublicMember*>(member))
-        return publicMember->name->asString() == propertyName;
+        return publicMember->name == propertyName;
     else if (UiObjectBinding *objectBinding = cast<UiObjectBinding*>(member))
-        return flatten(objectBinding->qualifiedId) == propertyName;
+        return toString(objectBinding->qualifiedId) == propertyName;
     else if (UiScriptBinding *scriptBinding = cast<UiScriptBinding*>(member))
-        return flatten(scriptBinding->qualifiedId) == propertyName;
+        return toString(scriptBinding->qualifiedId) == propertyName;
     else if (UiArrayBinding *arrayBinding = cast<UiArrayBinding*>(member))
-        return flatten(arrayBinding->qualifiedId) == propertyName;
+        return toString(arrayBinding->qualifiedId) == propertyName;
     else
         return false;
 }
@@ -343,10 +377,10 @@ void Rewriter::insertIntoArray(UiArrayBinding *ast, const QString &newValue)
         return;
 
     const int insertionPoint = lastMember->lastSourceLocation().end();
-    m_changeSet->replace(insertionPoint, 0, QLatin1String(",\n") + newValue);
+    m_changeSet->insert(insertionPoint, QLatin1String(",\n") + newValue);
 }
 
-void Rewriter::removeProperty(UiObjectInitializer *ast, const QString &propertyName)
+void Rewriter::removeBindingByName(UiObjectInitializer *ast, const QString &propertyName)
 {
     QString prefix;
     int dotIdx = propertyName.indexOf(QLatin1Char('.'));
@@ -363,7 +397,7 @@ void Rewriter::removeProperty(UiObjectInitializer *ast, const QString &propertyN
         // check for grouped properties:
         else if (!prefix.isEmpty()) {
             if (UiObjectDefinition *def = cast<UiObjectDefinition *>(member)) {
-                if (flatten(def->qualifiedTypeNameId) == prefix) {
+                if (toString(def->qualifiedTypeNameId) == prefix) {
                     removeGroupedProperty(def, propertyName);
                 }
             }
@@ -404,53 +438,60 @@ void Rewriter::removeMember(UiObjectMember *member)
     int start = member->firstSourceLocation().offset;
     int end = member->lastSourceLocation().end();
 
-    includeSurroundingWhitespace(start, end);
+    includeSurroundingWhitespace(m_originalText, start, end);
 
-    m_changeSet->replace(start, end - start, QLatin1String(""));
+    m_changeSet->remove(start, end);
 }
 
-bool Rewriter::includeSurroundingWhitespace(int &start, int &end) const
+bool Rewriter::includeSurroundingWhitespace(const QString &source, int &start, int &end)
 {
     bool includeStartingWhitespace = true;
     bool paragraphFound = false;
+    bool paragraphSkipped = false;
 
     if (end >= 0) {
-        QChar c = m_originalText.at(end);
+        QChar c = source.at(end);
+
         while (c.isSpace()) {
             ++end;
-
-            if (c == QChar::ParagraphSeparator) {
+            if (c.unicode() == 10) {
                 paragraphFound = true;
+                paragraphSkipped = true;
                 break;
-            } else if (end == m_originalText.length()) {
+            } else if (end == source.length()) {
                 break;
             }
 
-            c = m_originalText.at(end);
+            c = source.at(end);
         }
 
         includeStartingWhitespace = paragraphFound;
     }
 
+    paragraphFound = false;
     if (includeStartingWhitespace) {
         while (start > 0) {
-            const QChar c = m_originalText.at(start - 1);
+            const QChar c = source.at(start - 1);
 
-            if (!c.isSpace())
+            if (c.unicode() == 10) {
+                paragraphFound = true;
                 break;
-            else if (c == QChar::ParagraphSeparator)
+            }
+            if (!c.isSpace())
                 break;
 
             --start;
         }
     }
+    if (!paragraphFound && paragraphSkipped) //keep the line ending
+        --end;
 
     return paragraphFound;
 }
 
-void Rewriter::includeLeadingEmptyLine(int &start) const
+void Rewriter::includeLeadingEmptyLine(const QString &source, int &start)
 {
-    QTextDocument doc(m_originalText);
+    QTextDocument doc(source);
 
     if (start == 0)
         return;
@@ -472,6 +513,23 @@ void Rewriter::includeLeadingEmptyLine(int &start) const
     start = prevBlock.position();
 }
 
+void Rewriter::includeEmptyGroupedProperty(UiObjectDefinition *groupedProperty, UiObjectMember *memberToBeRemoved, int &start, int &end)
+{
+    if (groupedProperty->qualifiedTypeNameId && !groupedProperty->qualifiedTypeNameId->name.isEmpty()
+            && groupedProperty->qualifiedTypeNameId->name.at(0).isLower()) {
+        // grouped property
+        UiObjectMemberList *memberIter = groupedProperty->initializer->members;
+        while (memberIter) {
+            if (memberIter->member != memberToBeRemoved) {
+                return;
+            }
+            memberIter = memberIter->next;
+        }
+        start = groupedProperty->firstSourceLocation().begin();
+        end = groupedProperty->lastSourceLocation().end();
+    }
+}
+
 #if 0
 UiObjectMemberList *QMLRewriter::searchMemberToInsertAfter(UiObjectMemberList *members, const QStringList &propertyOrder)
 {
@@ -487,11 +545,11 @@ UiObjectMemberList *QMLRewriter::searchMemberToInsertAfter(UiObjectMemberList *m
         if (cast<UiObjectDefinition*>(member))
             lastObjectDef = iter;
         else if (UiArrayBinding *arrayBinding = cast<UiArrayBinding*>(member))
-            idx = propertyOrder.indexOf(flatten(arrayBinding->qualifiedId));
+            idx = propertyOrder.indexOf(toString(arrayBinding->qualifiedId));
         else if (UiObjectBinding *objectBinding = cast<UiObjectBinding*>(member))
-            idx = propertyOrder.indexOf(flatten(objectBinding->qualifiedId));
+            idx = propertyOrder.indexOf(toString(objectBinding->qualifiedId));
         else if (UiScriptBinding *scriptBinding = cast<UiScriptBinding*>(member))
-            idx = propertyOrder.indexOf(flatten(scriptBinding->qualifiedId));
+            idx = propertyOrder.indexOf(toString(scriptBinding->qualifiedId));
         else if (cast<UiPublicMember*>(member))
             idx = propertyOrder.indexOf(QLatin1String("property"));
 
@@ -516,13 +574,13 @@ UiObjectMemberList *QMLRewriter::searchMemberToInsertAfter(UiObjectMemberList *m
         UiObjectMember *member = iter->member;
 
         if (UiArrayBinding *arrayBinding = cast<UiArrayBinding*>(member))
-            orderedMembers[flatten(arrayBinding->qualifiedId)] = iter;
+            orderedMembers[toString(arrayBinding->qualifiedId)] = iter;
         else if (UiObjectBinding *objectBinding = cast<UiObjectBinding*>(member))
-            orderedMembers[flatten(objectBinding->qualifiedId)] = iter;
+            orderedMembers[toString(objectBinding->qualifiedId)] = iter;
         else if (cast<UiObjectDefinition*>(member))
             orderedMembers[QString::null] = iter;
         else if (UiScriptBinding *scriptBinding = cast<UiScriptBinding*>(member))
-            orderedMembers[flatten(scriptBinding->qualifiedId)] = iter;
+            orderedMembers[toString(scriptBinding->qualifiedId)] = iter;
         else if (cast<UiPublicMember*>(member))
             orderedMembers[QLatin1String("property")] = iter;
     }
@@ -558,13 +616,17 @@ void Rewriter::appendToArrayBinding(UiArrayBinding *arrayBinding,
 
     const int insertionPoint = lastMember->lastSourceLocation().end();
 
-    m_changeSet->replace(insertionPoint, 0, QLatin1String(",\n") + content);
+    m_changeSet->insert(insertionPoint, QLatin1String(",\n") + content);
 }
 
-void Rewriter::addObject(UiObjectInitializer *ast, const QString &content)
+Rewriter::Range Rewriter::addObject(UiObjectInitializer *ast, const QString &content)
 {
     UiObjectMemberList *insertAfter = searchMemberToInsertAfter(ast->members, m_propertyOrder);
+    return addObject(ast, content, insertAfter);
+}
 
+Rewriter::Range Rewriter::addObject(UiObjectInitializer *ast, const QString &content, UiObjectMemberList *insertAfter)
+{
     int insertionPoint;
     QString textToInsert;
     if (insertAfter && insertAfter->member) {
@@ -575,7 +637,32 @@ void Rewriter::addObject(UiObjectInitializer *ast, const QString &content)
     }
 
     textToInsert += content;
-    m_changeSet->replace(insertionPoint, 0, QLatin1String("\n") + textToInsert);
+    m_changeSet->insert(insertionPoint, QLatin1String("\n") + textToInsert);
+
+    return Range(insertionPoint, insertionPoint);
+}
+
+Rewriter::Range Rewriter::addObject(UiArrayBinding *ast, const QString &content)
+{
+    UiArrayMemberList *insertAfter = searchMemberToInsertAfter(ast->members, m_propertyOrder);
+    return addObject(ast, content, insertAfter);
+}
+
+Rewriter::Range Rewriter::addObject(UiArrayBinding *ast, const QString &content, UiArrayMemberList *insertAfter)
+{
+    int insertionPoint;
+    QString textToInsert;
+    if (insertAfter && insertAfter->member) {
+        insertionPoint = insertAfter->member->lastSourceLocation().end();
+        textToInsert = QLatin1String(",\n") + content;
+    } else {
+        insertionPoint = ast->lbracketToken.end();
+        textToInsert += QLatin1String("\n") + content + QLatin1Char(',');
+    }
+
+    m_changeSet->insert(insertionPoint, textToInsert);
+
+    return Range(insertionPoint, insertionPoint);
 }
 
 void Rewriter::removeObjectMember(UiObjectMember *member, UiObjectMember *parent)
@@ -586,11 +673,14 @@ void Rewriter::removeObjectMember(UiObjectMember *member, UiObjectMember *parent
     if (UiArrayBinding *parentArray = cast<UiArrayBinding *>(parent)) {
         extendToLeadingOrTrailingComma(parentArray, member, start, end);
     } else {
-        includeSurroundingWhitespace(start, end);
+        if (UiObjectDefinition *parentObjectDefinition = cast<UiObjectDefinition *>(parent)) {
+            includeEmptyGroupedProperty(parentObjectDefinition, member, start, end);
+        }
+        includeSurroundingWhitespace(m_originalText, start, end);
     }
 
-    includeLeadingEmptyLine(start);
-    m_changeSet->replace(start, end - start, QLatin1String(""));
+    includeLeadingEmptyLine(m_originalText, start);
+    m_changeSet->remove(start, end);
 }
 
 void Rewriter::extendToLeadingOrTrailingComma(UiArrayBinding *parentArray,
@@ -612,16 +702,16 @@ void Rewriter::extendToLeadingOrTrailingComma(UiArrayBinding *parentArray,
     if (currentMember->commaToken.isValid()) {
         // leading comma
         start = currentMember->commaToken.offset;
-        if (includeSurroundingWhitespace(start, end))
+        if (includeSurroundingWhitespace(m_originalText, start, end))
             --end;
     } else if (currentMember->next && currentMember->next->commaToken.isValid()) {
         // trailing comma
         end = currentMember->next->commaToken.end();
-        includeSurroundingWhitespace(start, end);
+        includeSurroundingWhitespace(m_originalText, start, end);
     } else {
         // array with 1 element, so remove the complete binding
         start = parentArray->firstSourceLocation().offset;
         end = parentArray->lastSourceLocation().end();
-        includeSurroundingWhitespace(start, end);
+        includeSurroundingWhitespace(m_originalText, start, end);
     }
 }
