@@ -2,7 +2,7 @@
 **
 ** This file is part of Qt Creator
 **
-** Copyright (c) 2011 Nokia Corporation and/or its subsidiary(-ies).
+** Copyright (c) 2012 Nokia Corporation and/or its subsidiary(-ies).
 **
 ** Contact: Nokia Corporation (qt-info@nokia.com)
 **
@@ -33,14 +33,14 @@
 #include "filesearch.h"
 #include <cctype>
 
-#include <QtCore/QBuffer>
-#include <QtCore/QFile>
-#include <QtCore/QtConcurrentRun>
-#include <QtCore/QRegExp>
-#include <QtCore/QCoreApplication>
-#include <QtCore/QTextCodec>
+#include <QBuffer>
+#include <QFile>
+#include <QtConcurrentRun>
+#include <QRegExp>
+#include <QCoreApplication>
+#include <QTextCodec>
 
-#include <qtconcurrent/runextensions.h>
+#include "runextensions.h"
 
 using namespace Utils;
 
