@@ -2,7 +2,7 @@
 **
 ** This file is part of Qt Creator
 **
-** Copyright (c) 2011 Nokia Corporation and/or its subsidiary(-ies).
+** Copyright (c) 2012 Nokia Corporation and/or its subsidiary(-ies).
 **
 ** Contact: Nokia Corporation (qt-info@nokia.com)
 **
@@ -33,9 +33,9 @@
 #include "qtcprocess.h"
 #include "stringutils.h"
 
-#include <QtCore/QDir>
-#include <QtCore/QDebug>
-#include <QtCore/QCoreApplication>
+#include <QDir>
+#include <QDebug>
+#include <QCoreApplication>
 
 #ifdef Q_OS_WIN
 #include <qt_windows.h>
@@ -657,6 +657,20 @@ void QtcProcess::start()
         if (m_environment.size() == 0)
             qWarning("QtcProcess::start: Empty environment set when running '%s'.", qPrintable(m_command));
         env = m_environment;
+
+        // If the process environemnt has no libraryPath,
+        // Qt will copy creator's libraryPath into the process environment.
+        // That's brain dead, and we work around it
+#if defined(Q_OS_UNIX)
+#  if defined(Q_OS_MAC)
+        static const char libraryPathC[] = "DYLD_LIBRARY_PATH";
+#  else
+        static const char libraryPathC[] = "LD_LIBRARY_PATH";
+#  endif
+        const QString libraryPath = QLatin1String(libraryPathC);
+        if (env.constFind(libraryPath) == env.constEnd())
+            env.set(libraryPath, QString());
+#endif
         QProcess::setEnvironment(env.toStringList());
     } else {
         env = Environment::systemEnvironment();
@@ -670,7 +684,7 @@ void QtcProcess::start()
     prepareCommand(m_command, m_arguments, &command, &arguments, &env, &workDir);
     setNativeArguments(arguments);
     if (m_useCtrlCStub) {
-        argList << command;
+        argList << QDir::toNativeSeparators(command);
         command = QCoreApplication::applicationDirPath() + QLatin1String("/qtcreator_ctrlc_stub.exe");
     }
     QProcess::start(command, argList);
@@ -762,7 +776,7 @@ QT_BEGIN_NAMESPACE
 Q_DECLARE_TYPEINFO(MxSave, Q_MOVABLE_TYPE);
 QT_END_NAMESPACE
 
-#include <QtCore/QStack>
+#include <QStack>
 
 #endif
 
