@@ -2,7 +2,7 @@
 **
 ** This file is part of Qt Creator
 **
-** Copyright (c) 2011 Nokia Corporation and/or its subsidiary(-ies).
+** Copyright (c) 2012 Nokia Corporation and/or its subsidiary(-ies).
 **
 ** Contact: Nokia Corporation (qt-info@nokia.com)
 **
@@ -34,11 +34,11 @@
 
 #include "stylehelper.h"
 
-#include <QtCore/QVariant>
-#include <QtGui/QPainter>
-#include <QtGui/QPixmapCache>
-#include <QtGui/QStyle>
-#include <QtGui/QStyleOption>
+#include <QVariant>
+#include <QPainter>
+#include <QPixmapCache>
+#include <QStyle>
+#include <QStyleOption>
 
 using namespace Utils;
 
