@@ -2,7 +2,7 @@
 **
 ** This file is part of Qt Creator
 **
-** Copyright (c) 2011 Nokia Corporation and/or its subsidiary(-ies).
+** Copyright (c) 2012 Nokia Corporation and/or its subsidiary(-ies).
 **
 ** Contact: Nokia Corporation (qt-info@nokia.com)
 **
@@ -44,13 +44,14 @@
 #include <texteditor/basetexteditor.h>
 #include <utils/qtcassert.h>
 
-#include <QtCore/QDebug>
-#include <QtCore/QFileInfo>
+#include <QDebug>
+#include <QDir>
+#include <QFileInfo>
 
-#include <QtGui/QAction>
-#include <QtGui/QContextMenuEvent>
-#include <QtGui/QMenu>
-#include <QtGui/QPainter>
+#include <QAction>
+#include <QContextMenuEvent>
+#include <QMenu>
+#include <QPainter>
 
 Q_DECLARE_METATYPE(Bookmarks::Internal::Bookmark*)
 
@@ -211,10 +212,10 @@ BookmarkView::BookmarkView(QWidget *parent)  :
 {
     setWindowTitle(tr("Bookmarks"));
 
-    connect(this, SIGNAL(clicked(const QModelIndex &)),
-            this, SLOT(gotoBookmark(const QModelIndex &)));
+    connect(this, SIGNAL(clicked(QModelIndex)),
+            this, SLOT(gotoBookmark(QModelIndex)));
 
-    ICore::instance()->addContextObject(m_bookmarkContext);
+    ICore::addContextObject(m_bookmarkContext);
 
     setItemDelegate(new BookmarkDelegate(this));
     setFrameStyle(QFrame::NoFrame);
@@ -224,7 +225,7 @@ BookmarkView::BookmarkView(QWidget *parent)  :
 
 BookmarkView::~BookmarkView()
 {
-    ICore::instance()->removeContextObject(m_bookmarkContext);
+    ICore::removeContextObject(m_bookmarkContext);
 }
 
 void BookmarkView::contextMenuEvent(QContextMenuEvent *event)
@@ -317,7 +318,7 @@ BookmarkManager::BookmarkManager() :
     connect(Core::ICore::instance(), SIGNAL(contextChanged(Core::IContext*,Core::Context)),
             this, SLOT(updateActionStatus()));
 
-    connect(ProjectExplorerPlugin::instance()->session(), SIGNAL(sessionLoaded()),
+    connect(ProjectExplorerPlugin::instance()->session(), SIGNAL(sessionLoaded(QString)),
             this, SLOT(loadBookmarks()));
 
     updateActionStatus();
@@ -381,7 +382,7 @@ QVariant BookmarkManager::data(const QModelIndex &index, int role) const
     else if (role == BookmarkManager::LineText)
         return m_bookmarksList.at(index.row())->lineText();
     else if (role == Qt::ToolTipRole)
-        return m_bookmarksList.at(index.row())->filePath();
+        return QDir::toNativeSeparators(m_bookmarksList.at(index.row())->filePath());
 
     return QVariant();
 }
@@ -392,7 +393,7 @@ void BookmarkManager::toggleBookmark()
     if (!editor)
         return;
 
-    toggleBookmark(editor->file()->fileName(), editor->currentLine());
+    toggleBookmark(editor->document()->fileName(), editor->currentLine());
 }
 
 void BookmarkManager::toggleBookmark(const QString &fileName, int lineNumber)
@@ -409,6 +410,7 @@ void BookmarkManager::toggleBookmark(const QString &fileName, int lineNumber)
 
     // Add a new bookmark if no bookmark existed on this line
     Bookmark *bookmark = new Bookmark(fi.filePath(), editorLine, this);
+    bookmark->init();
     addBookmark(bookmark);
 }
 
@@ -507,7 +509,7 @@ void BookmarkManager::documentPrevNext(bool next)
 {
     TextEditor::ITextEditor *editor = currentTextEditor();
     int editorLine = editor->currentLine();
-    QFileInfo fi(editor->file()->fileName());
+    QFileInfo fi(editor->document()->fileName());
     if (!m_bookmarksMap.contains(fi.path()))
         return;
 
@@ -614,7 +616,7 @@ BookmarkManager::State BookmarkManager::state() const
     if (!editor)
         return HasBookMarks;
 
-    const QFileInfo fi(editor->file()->fileName());
+    const QFileInfo fi(editor->document()->fileName());
 
     const DirectoryFileBookmarksMap::const_iterator dit = m_bookmarksMap.constFind(fi.path());
     if (dit == m_bookmarksMap.constEnd())
@@ -715,6 +717,7 @@ void BookmarkManager::addBookmark(const QString &s)
 
         if (!filePath.isEmpty() && !findBookmark(fi.path(), fi.fileName(), lineNumber)) {
             Bookmark *b = new Bookmark(filePath, lineNumber, this);
+            b->init();
             addBookmark(b, false);
         }
     } else {
@@ -733,33 +736,33 @@ QString BookmarkManager::bookmarkToString(const Bookmark *b)
 /* Saves the bookmarks to the session settings. */
 void BookmarkManager::saveBookmarks()
 {
-    SessionManager *s = sessionManager();
-    if (!s)
-        return;
-
     QStringList list;
     foreach (const FileNameBookmarksMap *bookmarksMap, m_bookmarksMap)
         foreach (const Bookmark *bookmark, *bookmarksMap)
             list << bookmarkToString(bookmark);
 
-    s->setValue("Bookmarks", list);
+    sessionManager()->setValue("Bookmarks", list);
 }
 
 /* Loads the bookmarks from the session settings. */
 void BookmarkManager::loadBookmarks()
 {
     removeAllBookmarks();
-    SessionManager *s = sessionManager();
-    if (!s)
-        return;
-
-    const QStringList &list = s->value("Bookmarks").toStringList();
+    const QStringList &list = sessionManager()->value("Bookmarks").toStringList();
     foreach (const QString &bookmarkString, list)
         addBookmark(bookmarkString);
 
     updateActionStatus();
 }
 
+void BookmarkManager::handleBookmarkRequest(TextEditor::ITextEditor *textEditor,
+                                            int line,
+                                            TextEditor::ITextEditor::MarkRequestKind kind)
+{
+    if (kind == TextEditor::ITextEditor::BookmarkRequest && textEditor->document())
+        toggleBookmark(textEditor->document()->fileName(), line);
+}
+
 // BookmarkViewFactory
 
 BookmarkViewFactory::BookmarkViewFactory(BookmarkManager *bm)
@@ -777,9 +780,9 @@ int BookmarkViewFactory::priority() const
     return 300;
 }
 
-QString BookmarkViewFactory::id() const
+Id BookmarkViewFactory::id() const
 {
-    return QLatin1String("Bookmarks");
+    return Id("Bookmarks");
 }
 
 QKeySequence BookmarkViewFactory::activationSequence() const
