@@ -2,7 +2,7 @@
 **
 ** This file is part of Qt Creator
 **
-** Copyright (c) 2011 Nokia Corporation and/or its subsidiary(-ies).
+** Copyright (c) 2012 Nokia Corporation and/or its subsidiary(-ies).
 **
 ** Contact: Nokia Corporation (qt-info@nokia.com)
 **
@@ -33,7 +33,9 @@
 #ifndef ACTIONCONTAINER_H
 #define ACTIONCONTAINER_H
 
-#include <QtCore/QObject>
+#include "coreplugin/id.h"
+
+#include <QObject>
 
 QT_BEGIN_NAMESPACE
 class QMenu;
@@ -59,21 +61,21 @@ public:
     virtual void setOnAllDisabledBehavior(OnAllDisabledBehavior behavior) = 0;
     virtual ActionContainer::OnAllDisabledBehavior onAllDisabledBehavior() const = 0;
 
-    virtual int id() const = 0;
+    virtual Id id() const = 0;
 
     virtual QMenu *menu() const = 0;
     virtual QMenuBar *menuBar() const = 0;
 
-    virtual QAction *insertLocation(const QString &group) const = 0;
-    virtual void appendGroup(const QString &group) = 0;
-    virtual void addAction(Command *action, const QString &group = QString()) = 0;
-    virtual void addMenu(ActionContainer *menu, const QString &group = QString()) = 0;
-    virtual void addMenu(ActionContainer *before, ActionContainer *menu, const QString &group = QString()) = 0;
+    virtual QAction *insertLocation(const Id &group) const = 0;
+    virtual void appendGroup(const Id &group) = 0;
+    virtual void insertGroup(const Id &before, const Id &group) = 0;
+    virtual void addAction(Command *action, const Id &group = Id()) = 0;
+    virtual void addMenu(ActionContainer *menu, const Id &group = Id()) = 0;
+    virtual void addMenu(ActionContainer *before, ActionContainer *menu, const Id &group = Id()) = 0;
 
-    // clears this menu and submenus from all actions and submenus
-    // doesn't destroy the submenus and commands, just removes them from their parents
+    // This clears this menu and submenus from all actions and submenus.
+    // It does not destroy the submenus and commands, just removes them from their parents.
     virtual void clear() = 0;
-    virtual ~ActionContainer() {}
 };
 
 } // namespace Core
