@@ -2,7 +2,7 @@
 **
 ** This file is part of Qt Creator
 **
-** Copyright (c) 2011 Nokia Corporation and/or its subsidiary(-ies).
+** Copyright (c) 2012 Nokia Corporation and/or its subsidiary(-ies).
 **
 ** Contact: Nokia Corporation (qt-info@nokia.com)
 **
@@ -35,7 +35,7 @@
 
 #include <coreplugin/core_global.h>
 
-#include <QtCore/QObject>
+#include <QObject>
 
 namespace Core {
 
