@@ -2,7 +2,7 @@
 **
 ** This file is part of Qt Creator
 **
-** Copyright (c) 2011 Nokia Corporation and/or its subsidiary(-ies).
+** Copyright (c) 2012 Nokia Corporation and/or its subsidiary(-ies).
 **
 ** Contact: Nokia Corporation (qt-info@nokia.com)
 **
@@ -35,8 +35,8 @@
 
 #include "core_global.h"
 
-#include <QtCore/QPointer>
-#include <QtGui/QWidget>
+#include <QPointer>
+#include <QWidget>
 
 namespace Core {
 
