@@ -2,7 +2,7 @@
 **
 ** This file is part of Qt Creator
 **
-** Copyright (c) 2011 Nokia Corporation and/or its subsidiary(-ies).
+** Copyright (c) 2012 Nokia Corporation and/or its subsidiary(-ies).
 **
 ** Contact: Nokia Corporation (qt-info@nokia.com)
 **
@@ -54,18 +54,18 @@
 #include <utils/styledbar.h>
 #include <utils/qtcassert.h>
 
-#include <QtCore/QDebug>
+#include <QDebug>
 
-#include <QtGui/QAction>
-#include <QtGui/QApplication>
-#include <QtGui/QComboBox>
-#include <QtGui/QFocusEvent>
-#include <QtGui/QHBoxLayout>
-#include <QtGui/QSplitter>
-#include <QtGui/QPainter>
-#include <QtGui/QToolButton>
-#include <QtGui/QStackedWidget>
-#include <QtGui/QMenu>
+#include <QAction>
+#include <QApplication>
+#include <QComboBox>
+#include <QFocusEvent>
+#include <QHBoxLayout>
+#include <QSplitter>
+#include <QPainter>
+#include <QToolButton>
+#include <QStackedWidget>
+#include <QMenu>
 
 namespace Core {
 namespace Internal {
@@ -114,8 +114,8 @@ OutputPaneManager::OutputPaneManager(QWidget *parent) :
     m_lastIndex(-1),
     m_outputWidgetPane(new QStackedWidget),
     m_opToolBarWidgets(new QStackedWidget),
-    m_minimizeIcon(":/core/images/arrowdown.png"),
-    m_maximizeIcon(":/core/images/arrowup.png"),
+    m_minimizeIcon(QLatin1String(":/core/images/arrowdown.png")),
+    m_maximizeIcon(QLatin1String(":/core/images/arrowup.png")),
     m_maximised(false)
 {
     setWindowTitle(tr("Output"));
@@ -140,7 +140,7 @@ OutputPaneManager::OutputPaneManager(QWidget *parent) :
     m_minMaxAction->setIcon(m_maximizeIcon);
     m_minMaxAction->setText(tr("Maximize Output Pane"));
 
-    m_closeButton->setIcon(QIcon(QLatin1String(Constants::ICON_CLOSE)));
+    m_closeButton->setIcon(QIcon(QLatin1String(Constants::ICON_CLOSE_DOCUMENT)));
     connect(m_closeButton, SIGNAL(clicked()), this, SLOT(slotHide()));
 
     QVBoxLayout *mainlayout = new QVBoxLayout;
@@ -184,7 +184,7 @@ QWidget *OutputPaneManager::buttonsWidget()
 // Return shortcut as Ctrl+<number>
 static inline int paneShortCut(int number)
 {
-#ifdef Q_WS_MAC
+#ifdef Q_OS_MAC
     int modifier = Qt::CTRL;
 #else
     int modifier = Qt::ALT;
@@ -194,7 +194,7 @@ static inline int paneShortCut(int number)
 
 void OutputPaneManager::init()
 {
-    ActionManager *am = Core::ICore::instance()->actionManager();
+    ActionManager *am = Core::ICore::actionManager();
     ActionContainer *mwindow = am->actionContainer(Constants::M_WINDOW);
     const Context globalcontext(Core::Constants::C_GLOBAL);
 
@@ -212,20 +212,20 @@ void OutputPaneManager::init()
     mpanes->addAction(cmd, "Coreplugin.OutputPane.ActionsGroup");
 
     cmd = am->registerAction(m_prevAction, "Coreplugin.OutputPane.previtem", globalcontext);
-    cmd->setDefaultKeySequence(QKeySequence("Shift+F6"));
+    cmd->setDefaultKeySequence(QKeySequence(tr("Shift+F6")));
     m_prevToolButton->setDefaultAction(cmd->action());
     mpanes->addAction(cmd, "Coreplugin.OutputPane.ActionsGroup");
 
     cmd = am->registerAction(m_nextAction, "Coreplugin.OutputPane.nextitem", globalcontext);
     m_nextToolButton->setDefaultAction(cmd->action());
-    cmd->setDefaultKeySequence(QKeySequence("F6"));
+    cmd->setDefaultKeySequence(QKeySequence(tr("F6")));
     mpanes->addAction(cmd, "Coreplugin.OutputPane.ActionsGroup");
 
     cmd = am->registerAction(m_minMaxAction, "Coreplugin.OutputPane.minmax", globalcontext);
-#ifdef Q_WS_MAC
-    cmd->setDefaultKeySequence(QKeySequence("Ctrl+9"));
+#ifdef Q_OS_MAC
+    cmd->setDefaultKeySequence(QKeySequence(tr("Ctrl+9")));
 #else
-    cmd->setDefaultKeySequence(QKeySequence("Alt+9"));
+    cmd->setDefaultKeySequence(QKeySequence(tr("Alt+9")));
 #endif
     cmd->setAttribute(Command::CA_UpdateText);
     cmd->setAttribute(Command::CA_UpdateIcon);
@@ -270,11 +270,11 @@ void OutputPaneManager::init()
 
         m_opToolBarWidgets->addWidget(toolButtonsContainer);
 
-        QString actionId = QString("QtCreator.Pane.%1").arg(outPane->displayName().simplified());
+        QString actionId = QLatin1String("QtCreator.Pane.") + outPane->displayName().simplified();
         actionId.remove(QLatin1Char(' '));
         QAction *action = new QAction(outPane->displayName(), this);
 
-        Command *cmd = am->registerAction(action, actionId, Context(Constants::C_GLOBAL));
+        Command *cmd = am->registerAction(action, Id(actionId), Context(Constants::C_GLOBAL));
 
         mpanes->addAction(cmd, "Coreplugin.OutputPane.PanesGroup");
         m_actions.insert(cmd->action(), idx);
@@ -386,8 +386,12 @@ void OutputPaneManager::slotHide()
             m_buttons.value(idx)->setChecked(false);
         if (m_pageMap.value(idx))
             m_pageMap.value(idx)->visibilityChanged(false);
-        if (IEditor *editor = Core::EditorManager::instance()->currentEditor())
-            editor->widget()->setFocus();
+        if (IEditor *editor = Core::EditorManager::instance()->currentEditor()) {
+            QWidget *w = editor->widget()->focusWidget();
+            if (!w)
+                w = editor->widget();
+            w->setFocus();
+        }
     }
 }
 
@@ -446,12 +450,13 @@ void OutputPaneManager::showPage(int idx, bool focus)
         if (!OutputPanePlaceHolder::getCurrent()) {
             // In this mode we don't have a placeholder
             // switch to the output mode and switch the page
-            ModeManager::instance()->activateMode(Constants::MODE_EDIT);
+            ModeManager::activateMode(QLatin1String(Constants::MODE_EDIT));
         }
         if (OutputPanePlaceHolder::getCurrent()) {
             // make the page visible
             OutputPanePlaceHolder::getCurrent()->setVisible(true);
             ensurePageVisible(idx);
+            m_pageMap.value(idx)->visibilityChanged(true);
             if (focus && out->canFocus())
                 out->setFocus();
         }
@@ -530,16 +535,16 @@ OutputPaneToggleButton::OutputPaneToggleButton(int number, const QString &text,
     QFont fnt = QApplication::font();
     setFont(fnt);
     setStyleSheet(
-            "QToolButton { border-image: url(:/core/images/panel_button.png) 2 2 2 19;"
+            QLatin1String("QToolButton { border-image: url(:/core/images/panel_button.png) 2 2 2 19;"
                          " border-width: 2px 2px 2px 19px; padding-left: -17; padding-right: 4 } "
             "QToolButton:checked { border-image: url(:/core/images/panel_button_checked.png) 2 2 2 19 } "
             "QToolButton::menu-indicator { width:0; height:0 }"
-#ifndef Q_WS_MAC // Mac UIs usually don't hover
+#ifndef Q_OS_MAC // Mac UIs usually don't hover
             "QToolButton:checked:hover { border-image: url(:/core/images/panel_button_checked_hover.png) 2 2 2 19 } "
             "QToolButton:pressed:hover { border-image: url(:/core/images/panel_button_pressed.png) 2 2 2 19 } "
             "QToolButton:hover { border-image: url(:/core/images/panel_button_hover.png) 2 2 2 19 } "
 #endif
-            );
+            ));
     if (m_action)
         connect(m_action, SIGNAL(changed()), this, SLOT(updateToolTip()));
 }
