@@ -2,7 +2,7 @@
 **
 ** This file is part of Qt Creator
 **
-** Copyright (c) 2011 Nokia Corporation and/or its subsidiary(-ies).
+** Copyright (c) 2012 Nokia Corporation and/or its subsidiary(-ies).
 **
 ** Contact: Nokia Corporation (qt-info@nokia.com)
 **
@@ -37,9 +37,9 @@
 #include <utils/qtcassert.h>
 #include <coreplugin/icore.h>
 
-#include <QtCore/QSettings>
-#include <QtCore/QTextStream>
-#include <QtCore/QCoreApplication>
+#include <QSettings>
+#include <QTextStream>
+#include <QCoreApplication>
 
 namespace CodePaster {
 
@@ -129,7 +129,7 @@ void SettingsPage::apply()
     const Settings newSettings = m_widget->settings();
     if (newSettings != *m_settings) {
         *m_settings = newSettings;
-        m_settings->toSettings(Core::ICore::instance()->settings());
+        m_settings->toSettings(Core::ICore::settings());
     }
 }
 
