@@ -2,7 +2,7 @@
 **
 ** This file is part of Qt Creator
 **
-** Copyright (c) 2011 Nokia Corporation and/or its subsidiary(-ies).
+** Copyright (c) 2012 Nokia Corporation and/or its subsidiary(-ies).
 **
 ** Contact: Nokia Corporation (qt-info@nokia.com)
 **
@@ -34,8 +34,6 @@
 #include "cppeditorconstants.h"
 #include "cppplugin.h"
 #include "cpphighlighter.h"
-#include "cppchecksymbols.h"
-#include "cpplocalsymbols.h"
 #include "cppautocompleter.h"
 #include "cppquickfixassistant.h"
 
@@ -63,11 +61,18 @@
 
 #include <cpptools/cpptoolsplugin.h>
 #include <cpptools/cpptoolsconstants.h>
+#include <cpptools/cppchecksymbols.h>
 #include <cpptools/cppcodeformatter.h>
-#include <cpptools/cppcompletionassist.h>
+#include <cpptools/cppcompletionsupport.h>
+#include <cpptools/cpphighlightingsupport.h>
+#include <cpptools/cpplocalsymbols.h>
 #include <cpptools/cppqtstyleindenter.h>
 #include <cpptools/cppcodestylesettings.h>
 #include <cpptools/cpprefactoringchanges.h>
+#include <cpptools/cpptoolsreuse.h>
+#include <cpptools/doxygengenerator.h>
+#include <cpptools/cpptoolssettings.h>
+#include <cpptools/symbolfinder.h>
 
 #include <coreplugin/icore.h>
 #include <coreplugin/actionmanager/actionmanager.h>
@@ -92,24 +97,24 @@
 #include <texteditor/codeassist/basicproposalitem.h>
 #include <texteditor/codeassist/genericproposal.h>
 
-#include <QtCore/QDebug>
-#include <QtCore/QTime>
-#include <QtCore/QTimer>
-#include <QtCore/QStack>
-#include <QtCore/QSettings>
-#include <QtCore/QSignalMapper>
-#include <QtGui/QAction>
-#include <QtGui/QApplication>
-#include <QtGui/QHeaderView>
-#include <QtGui/QLayout>
-#include <QtGui/QMenu>
-#include <QtGui/QShortcut>
-#include <QtGui/QTextEdit>
-#include <QtGui/QComboBox>
-#include <QtGui/QToolBar>
-#include <QtGui/QTreeView>
-#include <QtGui/QSortFilterProxyModel>
-#include <QtGui/QMainWindow>
+#include <QDebug>
+#include <QTime>
+#include <QTimer>
+#include <QStack>
+#include <QSettings>
+#include <QSignalMapper>
+#include <QAction>
+#include <QApplication>
+#include <QHeaderView>
+#include <QLayout>
+#include <QMenu>
+#include <QShortcut>
+#include <QTextEdit>
+#include <QComboBox>
+#include <QToolBar>
+#include <QTreeView>
+#include <QSortFilterProxyModel>
+#include <QMainWindow>
 
 #include <sstream>
 
@@ -120,6 +125,7 @@ enum {
 };
 
 using namespace CPlusPlus;
+using namespace CppTools;
 using namespace CppEditor::Internal;
 
 namespace {
@@ -145,7 +151,7 @@ public:
 
     void adjustWidth()
     {
-        const int w = Core::ICore::instance()->mainWindow()->geometry().width();
+        const int w = Core::ICore::mainWindow()->geometry().width();
         setMaximumWidth(w);
         setMinimumWidth(qMin(qMax(sizeHintForColumn(0), minimumSizeHint().width()), w));
     }
@@ -359,7 +365,8 @@ struct CanonicalSymbol
 
     static Symbol *canonicalSymbol(Scope *scope, const QString &code, TypeOfExpression &typeOfExpression)
     {
-        const QList<LookupItem> results = typeOfExpression(code, scope, TypeOfExpression::Preprocess);
+        const QList<LookupItem> results =
+                typeOfExpression(code.toUtf8(), scope, TypeOfExpression::Preprocess);
 
         for (int i = results.size() - 1; i != -1; --i) {
             const LookupItem &r = results.at(i);
@@ -407,6 +414,8 @@ CPPEditor::CPPEditor(CPPEditorWidget *editor)
     m_context.add(TextEditor::Constants::C_TEXTEDITOR);
 }
 
+Q_GLOBAL_STATIC(CppTools::SymbolFinder, symbolFinder)
+
 CPPEditorWidget::CPPEditorWidget(QWidget *parent)
     : TextEditor::BaseTextEditorWidget(parent)
     , m_currentRenameSelection(NoCurrentRenameSelection)
@@ -414,9 +423,12 @@ CPPEditorWidget::CPPEditorWidget(QWidget *parent)
     , m_inRenameChanged(false)
     , m_firstRenameChange(false)
     , m_objcEnabled(false)
+    , m_commentsSettings(CppTools::CppToolsSettings::instance()->commentsSettings())
+    , m_completionSupport(0)
+    , m_highlightingSupport(0)
 {
     m_initialized = false;
-    qRegisterMetaType<CppEditor::Internal::SemanticInfo>("CppEditor::Internal::SemanticInfo");
+    qRegisterMetaType<SemanticInfo>("CppTools::SemanticInfo");
 
     m_semanticHighlighter = new SemanticHighlighter(this);
     m_semanticHighlighter->start();
@@ -430,10 +442,11 @@ CPPEditorWidget::CPPEditorWidget(QWidget *parent)
     baseTextDocument()->setSyntaxHighlighter(new CppHighlighter);
 
     m_modelManager = CppModelManagerInterface::instance();
-
     if (m_modelManager) {
         connect(m_modelManager, SIGNAL(documentUpdated(CPlusPlus::Document::Ptr)),
                 this, SLOT(onDocumentUpdated(CPlusPlus::Document::Ptr)));
+        m_completionSupport = m_modelManager->completionSupport(editor());
+        m_highlightingSupport = m_modelManager->highlightingSupport(editor());
     }
 
     m_highlightRevision = 0;
@@ -450,6 +463,11 @@ CPPEditorWidget::CPPEditorWidget(QWidget *parent)
     m_declDefLinkFinder = new FunctionDeclDefLinkFinder(this);
     connect(m_declDefLinkFinder, SIGNAL(foundLink(QSharedPointer<FunctionDeclDefLink>)),
             this, SLOT(onFunctionDeclDefLinkFound(QSharedPointer<FunctionDeclDefLink>)));
+
+    connect(CppTools::CppToolsSettings::instance(),
+            SIGNAL(commentsSettingsChanged(CppTools::CommentsSettings)),
+            this,
+            SLOT(onCommentsSettingsChanged(CppTools::CommentsSettings)));
 }
 
 CPPEditorWidget::~CPPEditorWidget()
@@ -462,6 +480,9 @@ CPPEditorWidget::~CPPEditorWidget()
         m_modelManager->GC();
         numberOfClosedEditors = 0;
     }
+
+    delete m_highlightingSupport;
+    delete m_completionSupport;
 }
 
 TextEditor::BaseTextEditor *CPPEditorWidget::createEditor()
@@ -529,7 +550,7 @@ void CPPEditorWidget::createToolBar(CPPEditor *editor)
     connect(m_outlineCombo, SIGNAL(currentIndexChanged(int)), this, SLOT(updateOutlineToolTip()));
     connect(document(), SIGNAL(contentsChange(int,int,int)), this, SLOT(onContentsChanged(int,int,int)));
 
-    connect(file(), SIGNAL(changed()), this, SLOT(updateFileName()));
+    connect(editorDocument(), SIGNAL(changed()), this, SLOT(updateFileName()));
 
     // set up function declaration - definition link
     connect(this, SIGNAL(cursorPositionChanged()), this, SLOT(updateFunctionDeclDefLink()));
@@ -539,8 +560,8 @@ void CPPEditorWidget::createToolBar(CPPEditor *editor)
     connect(this, SIGNAL(cursorPositionChanged()), this, SLOT(updateUses()));
     connect(this, SIGNAL(textChanged()), this, SLOT(updateUses()));
 
-    connect(m_semanticHighlighter, SIGNAL(changed(CppEditor::Internal::SemanticInfo)),
-            this, SLOT(updateSemanticInfo(CppEditor::Internal::SemanticInfo)));
+    connect(m_semanticHighlighter, SIGNAL(changed(CppTools::SemanticInfo)),
+            this, SLOT(updateSemanticInfo(CppTools::SemanticInfo)));
 
     editor->insertExtraToolBarWidget(TextEditor::BaseTextEditor::Left, m_outlineCombo);
 }
@@ -569,6 +590,27 @@ void CPPEditorWidget::cut()
     finishRename();
 }
 
+void CPPEditorWidget::selectAll()
+{
+    // if we are currently renaming a symbol
+    // and the cursor is over that symbol, select just that symbol
+    if (m_currentRenameSelection != NoCurrentRenameSelection) {
+        QTextCursor cursor = textCursor();
+        int selectionBegin = m_currentRenameSelectionBegin.position();
+        int selectionEnd = m_currentRenameSelectionEnd.position();
+
+        if (cursor.position() >= selectionBegin
+                && cursor.position() <= selectionEnd) {
+            cursor.setPosition(selectionBegin);
+            cursor.setPosition(selectionEnd, QTextCursor::KeepAnchor);
+            setTextCursor(cursor);
+            return;
+        }
+    }
+
+    BaseTextEditorWidget::selectAll();
+}
+
 CppModelManagerInterface *CPPEditorWidget::modelManager() const
 {
     return m_modelManager;
@@ -577,7 +619,7 @@ CppModelManagerInterface *CPPEditorWidget::modelManager() const
 void CPPEditorWidget::setMimeType(const QString &mt)
 {
     BaseTextEditorWidget::setMimeType(mt);
-    setObjCEnabled(mt == CppTools::Constants::OBJECTIVE_CPP_SOURCE_MIMETYPE);
+    setObjCEnabled(mt == QLatin1String(CppTools::Constants::OBJECTIVE_CPP_SOURCE_MIMETYPE));
 }
 
 void CPPEditorWidget::setObjCEnabled(bool onoff)
@@ -634,18 +676,12 @@ void CPPEditorWidget::abortRename()
     m_currentRenameSelectionEnd = QTextCursor();
     setExtraSelections(CodeSemanticsSelection, m_renameSelections);
 
-    rehighlight();
-}
-
-void CPPEditorWidget::rehighlight(bool force)
-{
-    const SemanticHighlighter::Source source = currentSource(force);
-    m_semanticHighlighter->rehighlight(source);
+    semanticRehighlight(/* force = */ true);
 }
 
 void CPPEditorWidget::onDocumentUpdated(Document::Ptr doc)
 {
-    if (doc->fileName() != file()->fileName())
+    if (doc->fileName() != editorDocument()->fileName())
         return;
 
     if (doc->editorRevision() != editorRevision())
@@ -655,9 +691,9 @@ void CPPEditorWidget::onDocumentUpdated(Document::Ptr doc)
             (Core::EditorManager::instance()->currentEditor() == editor()
              && (!m_lastSemanticInfo.doc
                  || !m_lastSemanticInfo.doc->translationUnit()->ast()
-                 || m_lastSemanticInfo.doc->fileName() != file()->fileName()))) {
+                 || m_lastSemanticInfo.doc->fileName() != editorDocument()->fileName()))) {
         m_initialized = true;
-        rehighlight(/* force = */ true);
+        semanticRehighlight(/* force = */ true);
     }
 
     m_updateOutlineTimer->start();
@@ -671,11 +707,14 @@ const Macro *CPPEditorWidget::findCanonicalMacro(const QTextCursor &cursor, Docu
     int line, col;
     convertPosition(cursor.position(), &line, &col);
 
-    if (const Macro *macro = doc->findMacroDefinitionAt(line))
-        return macro;
-
-    if (const Document::MacroUse *use = doc->findMacroUseAt(cursor.position()))
+    if (const Macro *macro = doc->findMacroDefinitionAt(line)) {
+        QTextCursor macroCursor = cursor;
+        const QByteArray name = identifierUnderCursor(&macroCursor).toLatin1();
+        if (macro->name() == name)
+            return macro;
+    } else if (const Document::MacroUse *use = doc->findMacroUseAt(cursor.position())) {
         return &use->macro();
+    }
 
     return 0;
 }
@@ -686,12 +725,13 @@ void CPPEditorWidget::findUsages()
     info.snapshot = CppModelManagerInterface::instance()->snapshot();
     info.snapshot.insert(info.doc);
 
-    CanonicalSymbol cs(this, info);
-    Symbol *canonicalSymbol = cs(textCursor());
-    if (canonicalSymbol) {
-        m_modelManager->findUsages(canonicalSymbol, cs.context());
-    } else if (const Macro *macro = findCanonicalMacro(textCursor(), info.doc)) {
+    if (const Macro *macro = findCanonicalMacro(textCursor(), info.doc)) {
         m_modelManager->findMacroUsages(*macro);
+    } else {
+        CanonicalSymbol cs(this, info);
+        Symbol *canonicalSymbol = cs(textCursor());
+        if (canonicalSymbol)
+            m_modelManager->findUsages(canonicalSymbol, cs.context());
     }
 }
 
@@ -837,7 +877,7 @@ void CPPEditorWidget::onContentsChanged(int position, int charsRemoved, int char
 }
 
 void CPPEditorWidget::updateFileName()
-{ }
+{}
 
 void CPPEditorWidget::jumpToOutlineElement(int)
 {
@@ -871,7 +911,7 @@ bool CPPEditorWidget::sortedOutline() const
 void CPPEditorWidget::updateOutlineNow()
 {
     const Snapshot snapshot = m_modelManager->snapshot();
-    Document::Ptr document = snapshot.document(file()->fileName());
+    Document::Ptr document = snapshot.document(editorDocument()->fileName());
 
     if (!document)
         return;
@@ -1017,7 +1057,7 @@ void CPPEditorWidget::switchDeclarationDefinition()
 
     const Snapshot snapshot = m_modelManager->snapshot();
 
-    if (Document::Ptr thisDocument = snapshot.document(file()->fileName())) {
+    if (Document::Ptr thisDocument = snapshot.document(editorDocument()->fileName())) {
         int line = 0, positionInBlock = 0;
         convertPosition(position(), &line, &positionInBlock);
 
@@ -1051,7 +1091,7 @@ void CPPEditorWidget::switchDeclarationDefinition()
                 openCppEditorAt(linkToSymbol(best.first()));
 
         } else if (lastVisibleSymbol && lastVisibleSymbol->isDeclaration() && lastVisibleSymbol->type()->isFunctionType()) {
-            if (Symbol *def = snapshot.findMatchingDefinition(lastVisibleSymbol))
+            if (Symbol *def = symbolFinder()->findMatchingDefinition(lastVisibleSymbol, snapshot))
                 openCppEditorAt(linkToSymbol(def));
         }
     }
@@ -1146,12 +1186,13 @@ CPPEditorWidget::Link CPPEditorWidget::attemptFuncDeclDef(const QTextCursor &cur
 
     Symbol *target = 0;
     if (FunctionDefinitionAST *funDef = declParent->asFunctionDefinition()) {
-        QList<Declaration *> candidates = findMatchingDeclaration(LookupContext(doc, snapshot),
-                                                                  funDef->symbol);
+        QList<Declaration *> candidates =
+                symbolFinder()->findMatchingDeclaration(LookupContext(doc, snapshot),
+                                                        funDef->symbol);
         if (!candidates.isEmpty()) // TODO: improve disambiguation
             target = candidates.first();
     } else if (declParent->asSimpleDeclaration()) {
-        target = snapshot.findMatchingDefinition(funcDecl->symbol);
+        target = symbolFinder()->findMatchingDefinition(funcDecl->symbol, snapshot);
     }
 
     if (target) {
@@ -1250,7 +1291,7 @@ CPPEditorWidget::Link CPPEditorWidget::findLinkAt(const QTextCursor &cursor,
     }
 
     // Now we prefer the doc from the snapshot with macros expanded.
-    Document::Ptr doc = snapshot.document(file()->fileName());
+    Document::Ptr doc = snapshot.document(editorDocument()->fileName());
     if (!doc) {
         doc = m_lastSemanticInfo.doc;
         if (!doc)
@@ -1344,6 +1385,25 @@ CPPEditorWidget::Link CPPEditorWidget::findLinkAt(const QTextCursor &cursor,
         tc.setPosition(endOfToken);
     }
 
+    // Handle macro uses
+    const Macro *macro = doc->findMacroDefinitionAt(line);
+    if (macro) {
+        QTextCursor macroCursor = cursor;
+        const QByteArray name = identifierUnderCursor(&macroCursor).toLatin1();
+        if (macro->name() == name)
+            return link;    //already on definition!
+    } else {
+        const Document::MacroUse *use = doc->findMacroUseAt(endOfToken - 1);
+        if (use && use->macro().fileName() != QLatin1String("<configuration>")) {
+            const Macro &macro = use->macro();
+            link.fileName = macro.fileName();
+            link.line = macro.line();
+            link.begin = use->begin();
+            link.end = use->end();
+            return link;
+        }
+    }
+
     // Find the last symbol up to the cursor position
     Scope *scope = doc->scopeAt(line, column);
     if (!scope)
@@ -1371,7 +1431,8 @@ CPPEditorWidget::Link CPPEditorWidget::findLinkAt(const QTextCursor &cursor,
 
     TypeOfExpression typeOfExpression;
     typeOfExpression.init(doc, snapshot);
-    const QList<LookupItem> resolvedSymbols = typeOfExpression.reference(expression, scope, TypeOfExpression::Preprocess);
+    const QList<LookupItem> resolvedSymbols =
+            typeOfExpression.reference(expression.toUtf8(), scope, TypeOfExpression::Preprocess);
 
     if (!resolvedSymbols.isEmpty()) {
         LookupItem result = skipForwardDeclarations(resolvedSymbols);
@@ -1379,7 +1440,7 @@ CPPEditorWidget::Link CPPEditorWidget::findLinkAt(const QTextCursor &cursor,
         foreach (const LookupItem &r, resolvedSymbols) {
             if (Symbol *d = r.declaration()) {
                 if (d->isDeclaration() || d->isFunction()) {
-                    if (file()->fileName() == QString::fromUtf8(d->fileName(), d->fileNameLength())) {
+                    if (editorDocument()->fileName() == QString::fromUtf8(d->fileName(), d->fileNameLength())) {
                         if (unsigned(lineNumber) == d->line() && unsigned(positionInBlock) >= d->column()) { // ### TODO: check the end
                             result = r; // take the symbol under cursor.
                             break;
@@ -1400,9 +1461,8 @@ CPPEditorWidget::Link CPPEditorWidget::findLinkAt(const QTextCursor &cursor,
                 if (def == lastVisibleSymbol)
                     def = 0; // jump to declaration then.
 
-                if (symbol->isForwardClassDeclaration()) {
-                    def = snapshot.findMatchingClassDeclaration(symbol);
-                }
+                if (symbol->isForwardClassDeclaration())
+                    def = symbolFinder()->findMatchingClassDeclaration(symbol, snapshot);
             }
 
             link = linkToSymbol(def ? def : symbol);
@@ -1438,7 +1498,7 @@ Symbol *CPPEditorWidget::findDefinition(Symbol *symbol, const Snapshot &snapshot
     else if (! symbol->type()->isFunctionType())
         return 0; // not a function declaration
 
-    return snapshot.findMatchingDefinition(symbol);
+    return symbolFinder()->findMatchingDefinition(symbol, snapshot);
 }
 
 unsigned CPPEditorWidget::editorRevision() const
@@ -1507,7 +1567,7 @@ void CPPEditorWidget::contextMenuEvent(QContextMenuEvent *e)
 
     QMenu *menu = new QMenu;
 
-    Core::ActionManager *am = Core::ICore::instance()->actionManager();
+    Core::ActionManager *am = Core::ICore::actionManager();
     Core::ActionContainer *mcontext = am->actionContainer(Constants::M_CONTEXT);
     QMenu *contextMenu = mcontext->menu();
 
@@ -1543,7 +1603,7 @@ void CPPEditorWidget::contextMenuEvent(QContextMenuEvent *e)
 
     foreach (QAction *action, contextMenu->actions()) {
         menu->addAction(action);
-        if (action->objectName() == Constants::M_REFACTORING_MENU_INSERTION_POINT)
+        if (action->objectName() == QLatin1String(Constants::M_REFACTORING_MENU_INSERTION_POINT))
             menu->addMenu(quickFixMenu);
     }
 
@@ -1557,7 +1617,8 @@ void CPPEditorWidget::contextMenuEvent(QContextMenuEvent *e)
 void CPPEditorWidget::keyPressEvent(QKeyEvent *e)
 {
     if (m_currentRenameSelection == NoCurrentRenameSelection) {
-        TextEditor::BaseTextEditorWidget::keyPressEvent(e);
+        if (!handleDocumentationComment(e))
+            TextEditor::BaseTextEditorWidget::keyPressEvent(e);
         return;
     }
 
@@ -1646,15 +1707,15 @@ Core::IEditor *CPPEditor::duplicate(QWidget *parent)
     return newEditor->editor();
 }
 
-QString CPPEditor::id() const
+Core::Id CPPEditor::id() const
 {
-    return QLatin1String(CppEditor::Constants::CPPEDITOR_ID);
+    return CppEditor::Constants::CPPEDITOR_ID;
 }
 
 bool CPPEditor::open(QString *errorString, const QString &fileName, const QString &realFileName)
 {
     bool b = TextEditor::BaseTextEditor::open(errorString, fileName, realFileName);
-    editorWidget()->setMimeType(Core::ICore::instance()->mimeDatabase()->findByFile(QFileInfo(fileName)).type());
+    editorWidget()->setMimeType(Core::ICore::mimeDatabase()->findByFile(QFileInfo(fileName)).type());
     return b;
 }
 
@@ -1685,6 +1746,8 @@ void CPPEditorWidget::setFontSettings(const TextEditor::FontSettings &fs)
             fs.toTextCharFormat(QLatin1String(TextEditor::Constants::C_STATIC));
     m_semanticHighlightFormatMap[SemanticInfo::VirtualMethodUse] =
             fs.toTextCharFormat(QLatin1String(TextEditor::Constants::C_VIRTUAL_METHOD));
+    m_semanticHighlightFormatMap[SemanticInfo::LabelUse] =
+            fs.toTextCharFormat(QLatin1String(TextEditor::Constants::C_LABEL));
     m_keywordFormat = fs.toTextCharFormat(QLatin1String(TextEditor::Constants::C_KEYWORD));
 
     // only set the background, we do not want to modify foreground properties set by the syntax highlighter or the link
@@ -1760,9 +1823,9 @@ bool CPPEditorWidget::openCppEditorAt(const Link &link)
                                                     Constants::CPPEDITOR_ID);
 }
 
-void CPPEditorWidget::semanticRehighlight()
+void CPPEditorWidget::semanticRehighlight(bool force)
 {
-    m_semanticHighlighter->rehighlight(currentSource());
+    m_semanticHighlighter->rehighlight(currentSource(force));
 }
 
 void CPPEditorWidget::updateSemanticInfo(const SemanticInfo &semanticInfo)
@@ -1784,6 +1847,11 @@ void CPPEditorWidget::updateSemanticInfo(const SemanticInfo &semanticInfo)
     m_renameSelections.clear();
     m_currentRenameSelection = NoCurrentRenameSelection;
 
+    // We can use the semanticInfo's snapshot (and avoid locking), but not its
+    // document, since it doesn't contain expanded macros.
+    LookupContext context(semanticInfo.snapshot.document(editorDocument()->fileName()),
+                          semanticInfo.snapshot);
+
     SemanticInfo::LocalUseIterator it(semanticInfo.localUses);
     while (it.hasNext()) {
         it.next();
@@ -1799,12 +1867,14 @@ void CPPEditorWidget::updateSemanticInfo(const SemanticInfo &semanticInfo)
             }
         }
 
-        if (uses.size() == 1)
-            // it's an unused declaration
-            highlightUses(uses, semanticInfo, &unusedSelections);
-
-        else if (good && m_renameSelections.isEmpty())
+        if (uses.size() == 1) {
+            if (!CppTools::isOwnershipRAIIType(it.key(), context)) {
+                // it's an unused declaration
+                highlightUses(uses, semanticInfo, &unusedSelections);
+            }
+        } else if (good && m_renameSelections.isEmpty()) {
             highlightUses(uses, semanticInfo, &m_renameSelections);
+        }
     }
 
     if (m_lastSemanticInfo.forced || previousSemanticInfo.revision != semanticInfo.revision) {
@@ -1812,11 +1882,11 @@ void CPPEditorWidget::updateSemanticInfo(const SemanticInfo &semanticInfo)
 
         if (! semanticHighlighterDisabled && semanticInfo.doc) {
             if (Core::EditorManager::instance()->currentEditor() == editor()) {
-                LookupContext context(semanticInfo.doc, semanticInfo.snapshot);
-                CheckSymbols::Future f = CheckSymbols::go(semanticInfo.doc, context);
-                m_highlighter = f;
-                m_highlightRevision = semanticInfo.revision;
-                m_highlightWatcher.setFuture(m_highlighter);
+                if (m_highlightingSupport) {
+                    m_highlighter = m_highlightingSupport->highlightingFuture(semanticInfo.doc, semanticInfo.snapshot);
+                    m_highlightRevision = semanticInfo.revision;
+                    m_highlightWatcher.setFuture(m_highlighter);
+                }
             }
         }
 
@@ -1943,7 +2013,7 @@ SemanticHighlighter::Source CPPEditorWidget::currentSource(bool force)
     convertPosition(position(), &line, &column);
 
     const Snapshot snapshot = m_modelManager->snapshot();
-    const QString fileName = file()->fileName();
+    const QString fileName = editorDocument()->fileName();
 
     QString code;
     if (force || m_lastSemanticInfo.revision != editorRevision())
@@ -2117,21 +2187,10 @@ TextEditor::IAssistInterface *CPPEditorWidget::createAssistInterface(
     TextEditor::AssistReason reason) const
 {
     if (kind == TextEditor::Completion) {
-        QStringList includePaths;
-        QStringList frameworkPaths;
-        if (ProjectExplorer::Project *project =
-                ProjectExplorer::ProjectExplorerPlugin::instance()->currentProject()) {
-            includePaths = m_modelManager->projectInfo(project).includePaths;
-            frameworkPaths = m_modelManager->projectInfo(project).frameworkPaths;
-        }
-        return new CppTools::Internal::CppCompletionAssistInterface(
-                    document(),
-                    position(),
-                    editor()->file(),
-                    reason,
-                    m_modelManager->snapshot(),
-                    includePaths,
-                    frameworkPaths);
+        if (m_completionSupport)
+            return m_completionSupport->createAssistInterface(
+                        ProjectExplorer::ProjectExplorerPlugin::currentProject(),
+                        document(), position(), reason);
     } else if (kind == TextEditor::QuickFix) {
         if (!semanticInfo().doc || isOutdated())
             return 0;
@@ -2237,4 +2296,116 @@ void CPPEditorWidget::abortDeclDefLink()
     m_declDefLink.clear();
 }
 
+bool CPPEditorWidget::handleDocumentationComment(QKeyEvent *e)
+{
+    if (!m_commentsSettings.m_enableDoxygen
+            && !m_commentsSettings.m_leadingAsterisks) {
+        return false;
+    }
+
+    if (e->key() == Qt::Key_Return
+            || e->key() == Qt::Key_Enter) {
+        QTextCursor cursor = textCursor();
+        if (!autoCompleter()->isInComment(cursor))
+            return false;
+
+        // We are interested on two particular cases:
+        //   1) The cursor is right after a /** or /*! and the user pressed enter. If Doxygen
+        //      is enabled we need to generate an entire comment block.
+        //   2) The cursor is already in the middle of a multi-line comment and the user pressed
+        //      enter. If leading asterisk(s) is set we need to write a comment continuation
+        //      with those.
+
+        if (m_commentsSettings.m_enableDoxygen
+                && cursor.positionInBlock() >= 3) {
+            const int pos = cursor.position();
+            if (characterAt(pos - 3) == QLatin1Char('/')
+                    && characterAt(pos - 2) == QLatin1Char('*')
+                    && (characterAt(pos - 1) == QLatin1Char('*')
+                        || characterAt(pos - 1) == QLatin1Char('!'))) {
+                CppTools::DoxygenGenerator doxygen;
+                doxygen.setAddLeadingAsterisks(m_commentsSettings.m_leadingAsterisks);
+                doxygen.setGenerateBrief(m_commentsSettings.m_generateBrief);
+                doxygen.setStartComment(false);
+                if (characterAt(pos - 1) == QLatin1Char('!'))
+                    doxygen.setStyle(CppTools::DoxygenGenerator::QtStyle);
+                else
+                    doxygen.setStyle(CppTools::DoxygenGenerator::JavaStyle);
+
+                // Move until we reach any possibly meaningful content.
+                while (document()->characterAt(cursor.position()).isSpace()
+                       && cursor.movePosition(QTextCursor::NextCharacter)) {
+                }
+
+                if (!cursor.atEnd()) {
+                    const QString &comment = doxygen.generate(cursor);
+                    if (!comment.isEmpty()) {
+                        cursor.beginEditBlock();
+                        cursor.setPosition(pos);
+                        cursor.insertText(comment);
+                        cursor.setPosition(pos - 3, QTextCursor::KeepAnchor);
+                        indent(document(), cursor, QChar::Null);
+                        cursor.endEditBlock();
+                        e->accept();
+                        return true;
+                    }
+                }
+                cursor.setPosition(pos);
+            }
+        }
+
+        if (!m_commentsSettings.m_leadingAsterisks)
+            return false;
+
+        // We continue the comment if the cursor is after a comment's line asterisk and if
+        // there's no asterisk immediately after the cursor (that would already be considered
+        // a leading asterisk).
+        int offset = 0;
+        const int blockPos = cursor.positionInBlock();
+        const QString &text = cursor.block().text();
+        for (; offset < blockPos; ++offset) {
+            if (!text.at(offset).isSpace())
+                break;
+        }
+
+        if (offset < blockPos
+                && (text.at(offset) == QLatin1Char('*')
+                    || (offset < blockPos - 1
+                        && text.at(offset) == QLatin1Char('/')
+                        && text.at(offset + 1) == QLatin1Char('*')))) {
+            int followinPos = blockPos;
+            for (; followinPos < text.length(); ++followinPos) {
+                if (!text.at(followinPos).isSpace())
+                    break;
+            }
+            if (followinPos == text.length()
+                    || text.at(followinPos) != QLatin1Char('*')) {
+                QString newLine(QLatin1Char('\n'));
+                QTextCursor c(cursor);
+                c.movePosition(QTextCursor::StartOfBlock);
+                c.movePosition(QTextCursor::NextCharacter, QTextCursor::KeepAnchor, offset);
+                newLine.append(c.selectedText());
+                if (text.at(offset) == QLatin1Char('/')) {
+                    newLine.append(QLatin1String(" *"));
+                } else {
+                    int start = offset;
+                    while (offset < blockPos && text.at(offset) == QLatin1Char('*'))
+                        ++offset;
+                    newLine.append(QString(offset - start, QLatin1Char('*')));
+                }
+                cursor.insertText(newLine);
+                e->accept();
+                return true;
+            }
+        }
+    }
+
+    return false;
+}
+
+void CPPEditorWidget::onCommentsSettingsChanged(const CppTools::CommentsSettings &settings)
+{
+    m_commentsSettings = settings;
+}
+
 #include "cppeditor.moc"
