@@ -2,30 +2,29 @@
 **
 ** This file is part of Qt Creator
 **
-** Copyright (c) 2011 Nokia Corporation and/or its subsidiary(-ies).
+** Copyright (c) 2012 Nokia Corporation and/or its subsidiary(-ies).
 **
 ** Contact: Nokia Corporation (qt-info@nokia.com)
 **
-** No Commercial Usage
-**
-** This file contains pre-release code and may not be distributed.
-** You may use this file in accordance with the terms and conditions
-** contained in the Technology Preview License Agreement accompanying
-** this package.
 **
 ** GNU Lesser General Public License Usage
 **
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU Lesser General Public License version 2.1 requirements
-** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** This file may be used under the terms of the GNU Lesser General Public
+** License version 2.1 as published by the Free Software Foundation and
+** appearing in the file LICENSE.LGPL included in the packaging of this file.
+** Please review the following information to ensure the GNU Lesser General
+** Public License version 2.1 requirements will be met:
+** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
 ** In addition, as a special exception, Nokia gives you certain additional
-** rights.  These rights are described in the Nokia Qt LGPL Exception
+** rights. These rights are described in the Nokia Qt LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
+** Other Usage
+**
+** Alternatively, this file may be used in accordance with the terms and
+** conditions contained in a signed written agreement between you and Nokia.
+**
 ** If you have questions regarding the use of this file, please contact
 ** Nokia at qt-info@nokia.com.
 **
@@ -33,15 +32,18 @@
 
 #include "cppelementevaluator.h"
 
-#include <coreplugin/ifile.h>
-#include <cpptools/cppmodelmanagerinterface.h>
+#include <coreplugin/idocument.h>
+#include <cpptools/cpptoolsreuse.h>
 
 #include <FullySpecifiedType.h>
+#include <Literals.h>
 #include <Names.h>
 #include <CoreTypes.h>
 #include <Scope.h>
 #include <Symbol.h>
 #include <Symbols.h>
+#include <TypeHierarchyBuilder.h>
+#include <cplusplus/ModelManagerInterface.h>
 #include <cplusplus/ExpressionUnderCursor.h>
 #include <cplusplus/Overview.h>
 #include <cplusplus/TypeOfExpression.h>
@@ -49,28 +51,16 @@
 #include <cplusplus/LookupItem.h>
 #include <cplusplus/Icons.h>
 
-#include <QtCore/QDir>
-#include <QtCore/QFileInfo>
-#include <QtCore/QSet>
-#include <QtCore/QQueue>
+#include <QDir>
+#include <QFileInfo>
+#include <QSet>
+#include <QQueue>
 
 using namespace CppEditor;
 using namespace Internal;
 using namespace CPlusPlus;
 
 namespace {
-    void moveCursorToEndOfName(QTextCursor *tc) {
-        QTextDocument *doc = tc->document();
-        if (!doc)
-            return;
-
-        QChar ch = doc->characterAt(tc->position());
-        while (ch.isLetterOrNumber() || ch == QLatin1Char('_')) {
-            tc->movePosition(QTextCursor::NextCharacter);
-            ch = doc->characterAt(tc->position());
-        }
-    }
-
     QStringList stripName(const QString &name) {
         QStringList all;
         all << name;
@@ -84,11 +74,12 @@ namespace {
     }
 }
 
-CppElementEvaluator::CppElementEvaluator(CPPEditor *editor) :
+CppElementEvaluator::CppElementEvaluator(CPPEditorWidget *editor) :
     m_editor(editor),
-    m_modelManager(CppTools::CppModelManagerInterface::instance()),
+    m_modelManager(CppModelManagerInterface::instance()),
     m_tc(editor->textCursor()),
-    m_lookupBaseClasses(false)
+    m_lookupBaseClasses(false),
+    m_lookupDerivedClasses(false)
 {}
 
 void CppElementEvaluator::setTextCursor(const QTextCursor &tc)
@@ -97,21 +88,19 @@ void CppElementEvaluator::setTextCursor(const QTextCursor &tc)
 void CppElementEvaluator::setLookupBaseClasses(const bool lookup)
 { m_lookupBaseClasses = lookup; }
 
-QSharedPointer<CppElement> CppElementEvaluator::identifyCppElement()
-{
-    m_element.clear();
-    evaluate();
-    return m_element;
-}
+void CppElementEvaluator::setLookupDerivedClasses(const bool lookup)
+{ m_lookupDerivedClasses = lookup; }
 
 // @todo: Consider refactoring code from CPPEditor::findLinkAt into here.
-void CppElementEvaluator::evaluate()
+void CppElementEvaluator::execute()
 {
+    clear();
+
     if (!m_modelManager)
         return;
 
     const Snapshot &snapshot = m_modelManager->snapshot();
-    Document::Ptr doc = snapshot.document(m_editor->file()->fileName());
+    Document::Ptr doc = snapshot.document(m_editor->editorDocument()->fileName());
     if (!doc)
         return;
 
@@ -120,37 +109,36 @@ void CppElementEvaluator::evaluate()
     const int pos = m_tc.position();
     m_editor->convertPosition(pos, &line, &column);
 
-    if (!matchDiagnosticMessage(doc, line)) {
-        if (!matchIncludeFile(doc, line) && !matchMacroInUse(doc, pos)) {
-            moveCursorToEndOfName(&m_tc);
+    checkDiagnosticMessage(pos);
 
-            // Fetch the expression's code
-            ExpressionUnderCursor expressionUnderCursor;
-            const QString &expression = expressionUnderCursor(m_tc);
-            Scope *scope = doc->scopeAt(line, column);
+    if (!matchIncludeFile(doc, line) && !matchMacroInUse(doc, pos)) {
+        CppTools::moveCursorToEndOfIdentifier(&m_tc);
 
-            TypeOfExpression typeOfExpression;
-            typeOfExpression.init(doc, snapshot);
-            const QList<LookupItem> &lookupItems = typeOfExpression(expression, scope);
-            if (lookupItems.isEmpty())
-                return;
+        // Fetch the expression's code
+        ExpressionUnderCursor expressionUnderCursor;
+        const QString &expression = expressionUnderCursor(m_tc);
+        Scope *scope = doc->scopeAt(line, column);
 
-            const LookupItem &lookupItem = lookupItems.first(); // ### TODO: select best candidate.
-            handleLookupItemMatch(snapshot, lookupItem, typeOfExpression.context());
-        }
+        TypeOfExpression typeOfExpression;
+        typeOfExpression.init(doc, snapshot);
+        const QList<LookupItem> &lookupItems = typeOfExpression(expression.toUtf8(), scope);
+        if (lookupItems.isEmpty())
+            return;
+
+        const LookupItem &lookupItem = lookupItems.first(); // ### TODO: select best candidate.
+        handleLookupItemMatch(snapshot, lookupItem, typeOfExpression.context());
     }
 }
 
-bool CppElementEvaluator::matchDiagnosticMessage(const CPlusPlus::Document::Ptr &document,
-                                                 unsigned line)
+void CppElementEvaluator::checkDiagnosticMessage(int pos)
 {
-    foreach (const Document::DiagnosticMessage &m, document->diagnosticMessages()) {
-        if (m.line() == line) {
-            m_element = QSharedPointer<CppElement>(new CppDiagnosis(m));
-            return true;
+    foreach (const QTextEdit::ExtraSelection &sel,
+             m_editor->extraSelections(TextEditor::BaseTextEditorWidget::CodeWarningsSelection)) {
+        if (pos >= sel.cursor.selectionStart() && pos <= sel.cursor.selectionEnd()) {
+            m_diagnosis = sel.format.toolTip();
+            break;
         }
     }
-    return false;
 }
 
 bool CppElementEvaluator::matchIncludeFile(const CPlusPlus::Document::Ptr &document, unsigned line)
@@ -169,8 +157,7 @@ bool CppElementEvaluator::matchMacroInUse(const CPlusPlus::Document::Ptr &docume
     foreach (const Document::MacroUse &use, document->macroUses()) {
         if (use.contains(pos)) {
             const unsigned begin = use.begin();
-            const QString &name = use.macro().name();
-            if (pos < begin + name.length()) {
+            if (pos < begin + use.macro().name().length()) {
                 m_element = QSharedPointer<CppElement>(new CppMacro(use.macro()));
                 return true;
             }
@@ -191,31 +178,67 @@ void CppElementEvaluator::handleLookupItemMatch(const Snapshot &snapshot,
         const FullySpecifiedType &type = declaration->type();
         if (declaration->isNamespace()) {
             m_element = QSharedPointer<CppElement>(new CppNamespace(declaration));
-        } else if (declaration->isClass() || declaration->isForwardClassDeclaration()) {
+        } else if (declaration->isClass()
+                   || declaration->isForwardClassDeclaration()
+                   || (declaration->isTemplate() && declaration->asTemplate()->declaration()
+                       && (declaration->asTemplate()->declaration()->isClass()
+                           || declaration->asTemplate()->declaration()->isForwardClassDeclaration()))) {
             if (declaration->isForwardClassDeclaration())
-                if (Symbol *classDeclaration = snapshot.findMatchingClassDeclaration(declaration))
+                if (Symbol *classDeclaration =
+                        m_symbolFinder.findMatchingClassDeclaration(declaration, snapshot)) {
                     declaration = classDeclaration;
+                }
             CppClass *cppClass = new CppClass(declaration);
             if (m_lookupBaseClasses)
                 cppClass->lookupBases(declaration, context);
+            if (m_lookupDerivedClasses)
+                cppClass->lookupDerived(declaration, snapshot);
             m_element = QSharedPointer<CppElement>(cppClass);
-        } else if (declaration->isEnum() || declaration->enclosingScope()->isEnum()) {
-            m_element = QSharedPointer<CppElement>(new CppEnum(declaration));
+        } else if (Enum *enumDecl = declaration->asEnum()) {
+            m_element = QSharedPointer<CppElement>(new CppEnum(enumDecl));
+        } else if (EnumeratorDeclaration *enumerator = dynamic_cast<EnumeratorDeclaration *>(declaration)) {
+            m_element = QSharedPointer<CppElement>(new CppEnumerator(enumerator));
         } else if (declaration->isTypedef()) {
             m_element = QSharedPointer<CppElement>(new CppTypedef(declaration));
-        } else if (declaration->isFunction() || (type.isValid() && type->isFunctionType())) {
+        } else if (declaration->isFunction()
+                   || (type.isValid() && type->isFunctionType())
+                   || declaration->isTemplate()) {
             m_element = QSharedPointer<CppElement>(new CppFunction(declaration));
         } else if (declaration->isDeclaration() && type.isValid()) {
             m_element = QSharedPointer<CppElement>(
                 new CppVariable(declaration, context, lookupItem.scope()));
-        } else if (declaration->isTemplate() && declaration->asTemplate()->declaration()) {
-            m_element = QSharedPointer<CppElement>(new CppTemplate(declaration));
         } else {
             m_element = QSharedPointer<CppElement>(new CppDeclarableElement(declaration));
         }
     }
 }
 
+bool CppElementEvaluator::identifiedCppElement() const
+{
+    return !m_element.isNull();
+}
+
+const QSharedPointer<CppElement> &CppElementEvaluator::cppElement() const
+{
+    return m_element;
+}
+
+bool CppElementEvaluator::hasDiagnosis() const
+{
+    return !m_diagnosis.isEmpty();
+}
+
+const QString &CppElementEvaluator::diagnosis() const
+{
+    return m_diagnosis;
+}
+
+void CppEditor::Internal::CppElementEvaluator::clear()
+{
+    m_element.clear();
+    m_diagnosis.clear();
+}
+
 // CppElement
 CppElement::CppElement() : m_helpCategory(TextEditor::HelpItem::Unknown)
 {}
@@ -244,10 +267,10 @@ void CppElement::setHelpMark(const QString &mark)
 const QString &CppElement::helpMark() const
 { return m_helpMark; }
 
-void CppElement::setLink(const CPPEditor::Link &link)
+void CppElement::setLink(const CPPEditorWidget::Link &link)
 { m_link = link; }
 
-const CPPEditor::Link &CppElement::link() const
+const CPPEditorWidget::Link &CppElement::link() const
 { return m_link; }
 
 void CppElement::setTooltip(const QString &tooltip)
@@ -269,19 +292,6 @@ Unknown::~Unknown()
 const QString &Unknown::type() const
 { return m_type; }
 
-// CppDiagnosis
-CppDiagnosis::CppDiagnosis(const Document::DiagnosticMessage &message) :
-    CppElement(), m_text(message.text())
-{
-    setTooltip(m_text);
-}
-
-CppDiagnosis::~CppDiagnosis()
-{}
-
-const QString &CppDiagnosis::text() const
-{ return m_text; }
-
 // CppInclude
 CppInclude::~CppInclude()
 {}
@@ -294,7 +304,7 @@ CppInclude::CppInclude(const Document::Include &includeFile) :
     setHelpCategory(TextEditor::HelpItem::Brief);
     setHelpIdCandidates(QStringList(m_fileName));
     setHelpMark(m_fileName);
-    setLink(CPPEditor::Link(m_path));
+    setLink(CPPEditorWidget::Link(m_path));
     setTooltip(m_path);
 }
 
@@ -308,16 +318,20 @@ const QString &CppInclude::fileName() const
 CppMacro::CppMacro(const Macro &macro) : CppElement()
 {
     setHelpCategory(TextEditor::HelpItem::Macro);
-    setHelpIdCandidates(QStringList(macro.name()));
-    setHelpMark(macro.name());
-    setLink(CPPEditor::Link(macro.fileName(), macro.line()));
-    setTooltip(macro.toString());
+    const QString macroName = QLatin1String(macro.name());
+    setHelpIdCandidates(QStringList(macroName));
+    setHelpMark(macroName);
+    setLink(CPPEditorWidget::Link(macro.fileName(), macro.line()));
+    setTooltip(macro.toStringWithLineBreaks());
 }
 
 CppMacro::~CppMacro()
 {}
 
 // CppDeclarableElement
+CppDeclarableElement::CppDeclarableElement()
+{}
+
 CppDeclarableElement::CppDeclarableElement(Symbol *declaration) : CppElement()
 {
     m_icon = Icons().iconForSymbol(declaration);
@@ -337,7 +351,7 @@ CppDeclarableElement::CppDeclarableElement(Symbol *declaration) : CppElement()
     }
 
     setTooltip(overview.prettyType(declaration->type(), m_qualifiedName));
-    setLink(CPPEditor::linkToSymbol(declaration));
+    setLink(CPPEditorWidget::linkToSymbol(declaration));
     setHelpMark(m_name);
 }
 
@@ -379,6 +393,9 @@ CppNamespace::~CppNamespace()
 {}
 
 // CppClass
+CppClass::CppClass()
+{}
+
 CppClass::CppClass(Symbol *declaration) : CppDeclarableElement(declaration)
 {
     setHelpCategory(TextEditor::HelpItem::ClassOrNamespace);
@@ -419,9 +436,32 @@ void CppClass::lookupBases(Symbol *declaration, const CPlusPlus::LookupContext &
     }
 }
 
+void CppClass::lookupDerived(CPlusPlus::Symbol *declaration, const CPlusPlus::Snapshot &snapshot)
+{
+    typedef QPair<CppClass *, TypeHierarchy> Data;
+
+    TypeHierarchyBuilder builder(declaration, snapshot);
+    const TypeHierarchy &completeHierarchy = builder.buildDerivedTypeHierarchy();
+
+    QQueue<Data> q;
+    q.enqueue(qMakePair(this, completeHierarchy));
+    while (!q.isEmpty()) {
+        const Data &current = q.dequeue();
+        CppClass *clazz = current.first;
+        const TypeHierarchy &classHierarchy = current.second;
+        foreach (const TypeHierarchy &derivedHierarchy, classHierarchy.hierarchy()) {
+            clazz->m_derived.append(CppClass(derivedHierarchy.symbol()));
+            q.enqueue(qMakePair(&clazz->m_derived.last(), derivedHierarchy));
+        }
+    }
+}
+
 const QList<CppClass> &CppClass::bases() const
 { return m_bases; }
 
+const QList<CppClass> &CppClass::derived() const
+{ return m_derived; }
+
 // CppFunction
 CppFunction::CppFunction(Symbol *declaration) : CppDeclarableElement(declaration)
 {
@@ -443,18 +483,11 @@ CppFunction::~CppFunction()
 {}
 
 // CppEnum
-CppEnum::CppEnum(Symbol *declaration) : CppDeclarableElement(declaration)
+CppEnum::CppEnum(Enum *declaration)
+    : CppDeclarableElement(declaration)
 {
     setHelpCategory(TextEditor::HelpItem::Enum);
-
-    if (declaration->enclosingScope()->isEnum()) {
-        Symbol *enumSymbol = declaration->enclosingScope()->asEnum();
-        Overview overview;
-        setHelpMark(overview.prettyName(enumSymbol->name()));
-        setTooltip(overview.prettyName(LookupContext::fullyQualifiedName(enumSymbol)));
-    } else {
-        setTooltip(qualifiedName());
-    }
+    setTooltip(qualifiedName());
 }
 
 CppEnum::~CppEnum()
@@ -513,30 +546,30 @@ CppVariable::CppVariable(Symbol *declaration, const LookupContext &context, Scop
 CppVariable::~CppVariable()
 {}
 
-// CppTemplate
-CppTemplate::CppTemplate(CPlusPlus::Symbol *declaration) : CppDeclarableElement(declaration)
+CppEnumerator::CppEnumerator(CPlusPlus::EnumeratorDeclaration *declaration)
+    : CppDeclarableElement(declaration)
 {
-    Template *templateSymbol = declaration->asTemplate();
-    if (templateSymbol->declaration()->isClass() ||
-        templateSymbol->declaration()->isForwardClassDeclaration()) {
-        m_isClassTemplate = true;
-        setHelpCategory(TextEditor::HelpItem::ClassOrNamespace);
-        setTooltip(qualifiedName());
-    } else {
-        m_isClassTemplate = false;
-        setHelpCategory(TextEditor::HelpItem::Function);
+    setHelpCategory(TextEditor::HelpItem::Enum);
+
+    Overview overview;
+
+    Symbol *enumSymbol = declaration->enclosingScope()->asEnum();
+    const QString enumName = overview.prettyName(LookupContext::fullyQualifiedName(enumSymbol));
+    const QString enumeratorName = overview.prettyName(declaration->name());
+    QString enumeratorValue;
+    if (const StringLiteral *value = declaration->constantValue()) {
+        enumeratorValue = QString::fromUtf8(value->chars(), value->size());
     }
+
+    setHelpMark(overview.prettyName(enumSymbol->name()));
+
+    QString tooltip = enumeratorName;
+    if (!enumName.isEmpty())
+        tooltip.prepend(enumName + QLatin1Char(' '));
+    if (!enumeratorValue.isEmpty())
+        tooltip.append(QLatin1String(" = ") + enumeratorValue);
+    setTooltip(tooltip);
 }
 
-CppTemplate::~CppTemplate()
+CppEnumerator::~CppEnumerator()
 {}
-
-bool CppTemplate::isClassTemplate() const
-{
-    return m_isClassTemplate;
-}
-
-bool CppTemplate::isFunctionTemplate() const
-{
-    return !m_isClassTemplate;
-}
