@@ -2,7 +2,7 @@
 **
 ** This file is part of Qt Creator
 **
-** Copyright (c) 2011 Nokia Corporation and/or its subsidiary(-ies).
+** Copyright (c) 2012 Nokia Corporation and/or its subsidiary(-ies).
 **
 ** Contact: Nokia Corporation (qt-info@nokia.com)
 **
@@ -30,10 +30,12 @@
 **
 **************************************************************************/
 
+#include "cppcompletionsupport.h"
+#include "cpphighlightingsupport.h"
 #include "cpptoolseditorsupport.h"
 #include "cppmodelmanager.h"
 
-#include <coreplugin/ifile.h>
+#include <coreplugin/idocument.h>
 
 #include <texteditor/itexteditor.h>
 #include <texteditor/basetexteditor.h>
@@ -42,8 +44,9 @@
 #include <ASTVisitor.h>
 #include <TranslationUnit.h>
 
-#include <QtCore/QTimer>
+#include <QTimer>
 
+using namespace CppTools;
 using namespace CppTools::Internal;
 using namespace CPlusPlus;
 
@@ -122,7 +125,7 @@ void CppEditorSupport::updateDocumentNow()
     } else {
         _updateDocumentTimer->stop();
 
-        QStringList sourceFiles(_textEditor->file()->fileName());
+        QStringList sourceFiles(_textEditor->document()->fileName());
         _cachedContents = _textEditor->contents().toUtf8();
         _documentParser = _modelManager->refreshSourceFiles(sourceFiles);
     }
