@@ -2,7 +2,7 @@
 **
 ** This file is part of Qt Creator
 **
-** Copyright (c) 2011 Nokia Corporation and/or its subsidiary(-ies).
+** Copyright (c) 2012 Nokia Corporation and/or its subsidiary(-ies).
 **
 ** Contact: Nokia Corporation (qt-info@nokia.com)
 **
@@ -37,16 +37,15 @@
 #include "bytearrayinputstream.h"
 #include "gdb/gdbmi.h"
 #include "disassemblerlines.h"
-#ifdef Q_OS_WIN
-#    include "shared/dbgwinutils.h"
-#endif
+#include "shared/hostutils.h"
+
 #include <utils/qtcassert.h>
 
-#include <QtCore/QByteArray>
-#include <QtCore/QVariant>
-#include <QtCore/QString>
-#include <QtCore/QDir>
-#include <QtCore/QDebug>
+#include <QByteArray>
+#include <QVariant>
+#include <QString>
+#include <QDir>
+#include <QDebug>
 
 #include <cctype>
 
@@ -264,7 +263,7 @@ static inline bool parseThread(QByteArray line, ThreadData *thread, bool *curren
         const QByteArray &pidTid = tokens.at(2);
         const int dotPos = pidTid.indexOf('.');
         if (dotPos != -1)
-            thread->targetId = QLatin1String("0x") + QString::fromAscii(pidTid.mid(dotPos + 1));
+            thread->targetId = QLatin1String("0x") + QString::fromLatin1(pidTid.mid(dotPos + 1));
     }
     case 1:
         thread->id = tokens.at(0).toInt();
