@@ -2,7 +2,7 @@
 **
 ** This file is part of Qt Creator
 **
-** Copyright (c) 2011 Nokia Corporation and/or its subsidiary(-ies).
+** Copyright (c) 2012 Nokia Corporation and/or its subsidiary(-ies).
 **
 ** Contact: Nokia Corporation (qt-info@nokia.com)
 **
@@ -50,9 +50,10 @@
 #include "stackhandler.h"
 #include "threadshandler.h"
 #include "watchhandler.h"
+#include "qtmessageloghandler.h"
 
 #include <coreplugin/icore.h>
-#include <coreplugin/ifile.h>
+#include <coreplugin/idocument.h>
 #include <coreplugin/editormanager/editormanager.h>
 #include <coreplugin/progressmanager/progressmanager.h>
 #include <coreplugin/progressmanager/futureprogress.h>
@@ -60,14 +61,20 @@
 #include <texteditor/itexteditor.h>
 #include <texteditor/basetextmark.h>
 
+#include <projectexplorer/taskhub.h>
+#include <extensionsystem/pluginmanager.h>
+
 #include <utils/savedaction.h>
 #include <utils/qtcassert.h>
+#include <utils/fileinprojectfinder.h>
 
-#include <QtCore/QDebug>
-#include <QtCore/QTimer>
-#include <QtCore/QFutureInterface>
+#include <QDebug>
+#include <QTimer>
+#include <QFile>
+#include <QFileInfo>
+#include <QFutureInterface>
 
-#include <QtGui/QMessageBox>
+#include <QMessageBox>
 
 using namespace Core;
 using namespace Debugger::Internal;
@@ -132,6 +139,15 @@ QDebug operator<<(QDebug str, const DebuggerStartParameters &sp)
 //
 //////////////////////////////////////////////////////////////////////
 
+// transitions:
+//   None->Requested
+//   Requested->Succeeded
+//   Requested->Failed
+//   Requested->Cancelled
+enum RemoteSetupState { RemoteSetupNone, RemoteSetupRequested,
+                        RemoteSetupSucceeded, RemoteSetupFailed,
+                        RemoteSetupCancelled };
+
 class DebuggerEnginePrivate : public QObject
 {
     Q_OBJECT
@@ -149,6 +165,7 @@ public:
         m_state(DebuggerNotReady),
         m_lastGoodState(DebuggerNotReady),
         m_targetState(DebuggerNotReady),
+        m_remoteSetupState(RemoteSetupNone),
         m_inferiorPid(0),
         m_modulesHandler(),
         m_registerHandler(),
@@ -158,10 +175,13 @@ public:
         m_watchHandler(engine),
         m_disassemblerAgent(engine),
         m_memoryAgent(engine),
-        m_isStateDebugging(false)
+        m_isStateDebugging(false),
+        m_testsPossible(true),
+        m_testsRunning(false),
+        m_taskHub(0)
     {
         connect(&m_locationTimer, SIGNAL(timeout()), SLOT(resetLocation()));
-        if (sp.toolChainAbi.os() == ProjectExplorer::Abi::MacOS)
+        if (sp.toolChainAbi.os() == Abi::MacOS)
             m_disassemblerAgent.setTryMixed(false);
     }
 
@@ -232,6 +252,7 @@ public slots:
     void scheduleResetLocation()
     {
         m_stackHandler.scheduleResetLocation();
+        m_watchHandler.scheduleResetLocation();
         m_threadsHandler.scheduleResetLocation();
         m_disassemblerAgent.scheduleResetLocation();
         m_locationTimer.setSingleShot(true);
@@ -243,15 +264,18 @@ public slots:
         m_locationTimer.stop();
         m_locationMark.reset();
         m_stackHandler.resetLocation();
+        m_watchHandler.resetLocation();
         m_threadsHandler.resetLocation();
         m_disassemblerAgent.resetLocation();
     }
 
 public:
     DebuggerState state() const { return m_state; }
+    RemoteSetupState remoteSetupState() const { return m_remoteSetupState; }
     bool isMasterEngine() const { return m_engine->isMasterEngine(); }
     DebuggerRunControl *runControl() const
         { return m_masterEngine ? m_masterEngine->runControl() : m_runControl; }
+    void setRemoteSetupState(RemoteSetupState state);
 
     DebuggerEngine *m_engine; // Not owned.
     DebuggerEngine *m_masterEngine; // Not owned
@@ -269,6 +293,9 @@ public:
     // The state we are aiming for.
     DebuggerState m_targetState;
 
+    // State of RemoteSetup signal/slots.
+    RemoteSetupState m_remoteSetupState;
+
     qint64 m_inferiorPid;
 
     ModulesHandler m_modulesHandler;
@@ -277,6 +304,7 @@ public:
     StackHandler m_stackHandler;
     ThreadsHandler m_threadsHandler;
     WatchHandler m_watchHandler;
+    QtMessageLogHandler m_qtMessageHandler;
     QFutureInterface<void> m_progress;
 
     DisassemblerAgent m_disassemblerAgent;
@@ -285,6 +313,19 @@ public:
     QTimer m_locationTimer;
 
     bool m_isStateDebugging;
+
+    Utils::FileInProjectFinder m_fileFinder;
+    // Testing
+    void handleAutoTests();
+    void handleAutoTestLine(int line);
+    void reportTestError(const QString &msg, int line);
+    bool m_testsPossible;
+    bool m_testsRunning;
+    bool m_breakOnError;
+    bool m_foundError;
+    QStringList m_testContents;
+    TaskHub *m_taskHub;
+    QString m_testFileName;
 };
 
 
@@ -358,6 +399,11 @@ void DebuggerEngine::frameDown()
     activateFrame(qMax(currentIndex - 1, 0));
 }
 
+void DebuggerEngine::setTargetState(DebuggerState state)
+{
+    d->m_targetState = state;
+}
+
 ModulesHandler *DebuggerEngine::modulesHandler() const
 {
     return d->m_masterEngine
@@ -374,10 +420,9 @@ RegisterHandler *DebuggerEngine::registerHandler() const
 
 StackHandler *DebuggerEngine::stackHandler() const
 {
-    //return d->m_masterEngine
-    //    ? d->m_masterEngine->stackHandler()
-    //    : &d->m_stackHandler;
-    return &d->m_stackHandler;
+    return d->m_masterEngine
+        ? d->m_masterEngine->stackHandler()
+        : &d->m_stackHandler;
 }
 
 ThreadsHandler *DebuggerEngine::threadsHandler() const
@@ -394,6 +439,11 @@ WatchHandler *DebuggerEngine::watchHandler() const
         : &d->m_watchHandler;
 }
 
+QtMessageLogHandler *DebuggerEngine::qtMessageLogHandler() const
+{
+    return &d->m_qtMessageHandler;
+}
+
 SourceFilesHandler *DebuggerEngine::sourceFilesHandler() const
 {
     return d->m_masterEngine
@@ -473,6 +523,14 @@ QAbstractItemModel *DebuggerEngine::sourceFilesModel() const
     return model;
 }
 
+QAbstractItemModel *DebuggerEngine::qtMessageLogModel() const
+{
+    QAbstractItemModel *model = qtMessageLogHandler()->model();
+    if (model->objectName().isEmpty()) // Make debugging easier.
+        model->setObjectName(objectName() + QLatin1String("QtMessageLogModel"));
+    return model;
+}
+
 void DebuggerEngine::fetchMemory(MemoryAgent *, QObject *,
         quint64 addr, quint64 length)
 {
@@ -501,6 +559,9 @@ void DebuggerEngine::showMessage(const QString &msg, int channel, int timeout) c
     }
     //if (msg.size() && msg.at(0).isUpper() && msg.at(1).isUpper())
     //    qDebug() << qPrintable(msg) << "IN STATE" << state();
+    if (channel == QtMessageLogOutput)
+        qtMessageLogHandler()->appendMessage(QtMessageLogHandler::UndefinedType, msg);
+
     debuggerCore()->showMessage(msg, channel, timeout);
     if (d->m_runControl) {
         d->m_runControl->showMessage(msg, channel);
@@ -515,7 +576,7 @@ void DebuggerEngine::startDebugger(DebuggerRunControl *runControl)
     QTC_ASSERT(!d->m_runControl, notifyEngineSetupFailed(); return);
 
     d->m_progress.setProgressRange(0, 1000);
-    Core::FutureProgress *fp = Core::ICore::instance()->progressManager()
+    Core::FutureProgress *fp = Core::ICore::progressManager()
         ->addTask(d->m_progress.future(),
         tr("Launching"), _("Debugger.Launcher"));
     fp->setKeepOnFinish(Core::FutureProgress::HideOnFinish);
@@ -531,9 +592,8 @@ void DebuggerEngine::startDebugger(DebuggerRunControl *runControl)
     if (!d->m_startParameters.environment.size())
         d->m_startParameters.environment = Utils::Environment();
 
-    const unsigned engineCapabilities = debuggerCapabilities();
     debuggerCore()->action(OperateByInstruction)
-        ->setEnabled(engineCapabilities & DisassemblerCapability);
+        ->setEnabled(hasCapability(DisassemblerCapability));
 
     QTC_ASSERT(state() == DebuggerNotReady || state() == DebuggerFinished,
          qDebug() << state());
@@ -551,6 +611,8 @@ void DebuggerEngine::resetLocation()
 
 void DebuggerEngine::gotoLocation(const Location &loc)
 {
+     d->resetLocation();
+
     if (debuggerCore()->boolSetting(OperateByInstruction) || !loc.hasDebugInfo()) {
         d->m_disassemblerAgent.setLocation(loc);
         return;
@@ -559,7 +621,6 @@ void DebuggerEngine::gotoLocation(const Location &loc)
     //if (m_shuttingDown)
     //    return;
 
-    d->resetLocation();
 
     const QString file = loc.fileName();
     const int line = loc.lineNumber();
@@ -567,7 +628,7 @@ void DebuggerEngine::gotoLocation(const Location &loc)
     QList<IEditor *> editors = editorManager->editorsForFileName(file);
     IEditor *editor = 0;
     if (editors.isEmpty()) {
-        editor = editorManager->openEditor(file, QString(),
+        editor = editorManager->openEditor(file, Core::Id(),
             EditorManager::IgnoreNavigationHistory);
         if (editor) {
             editors.append(editor);
@@ -581,10 +642,10 @@ void DebuggerEngine::gotoLocation(const Location &loc)
         texteditor->gotoLine(line, 0);
 
     if (loc.needsMarker()) {
-        d->m_locationMark.reset(new TextEditor::BaseTextMark);
-        d->m_locationMark->setLocation(file, line);
+        d->m_locationMark.reset(new TextEditor::BaseTextMark(file, line));
         d->m_locationMark->setIcon(debuggerCore()->locationMarkIcon());
         d->m_locationMark->setPriority(TextEditor::ITextMark::HighPriority);
+        d->m_locationMark->init();
     }
 
     // FIXME: Breaks with split views.
@@ -596,7 +657,7 @@ void DebuggerEngine::gotoLocation(const Location &loc)
 // Called from RunControl.
 void DebuggerEngine::handleStartFailed()
 {
-    showMessage("HANDLE RUNCONTROL START FAILED");
+    showMessage(QLatin1String("HANDLE RUNCONTROL START FAILED"));
     d->m_runControl = 0;
     d->m_progress.setProgressValue(900);
     d->m_progress.reportCanceled();
@@ -606,7 +667,7 @@ void DebuggerEngine::handleStartFailed()
 // Called from RunControl.
 void DebuggerEngine::handleFinished()
 {
-    showMessage("HANDLE RUNCONTROL FINISHED");
+    showMessage(QLatin1String("HANDLE RUNCONTROL FINISHED"));
     d->m_runControl = 0;
     d->m_progress.setProgressValue(1000);
     d->m_progress.reportFinished();
@@ -761,6 +822,13 @@ void DebuggerEnginePrivate::doSetupEngine()
 void DebuggerEngine::notifyEngineSetupFailed()
 {
     showMessage(_("NOTE: ENGINE SETUP FAILED"));
+    QTC_ASSERT(d->remoteSetupState() == RemoteSetupNone
+               || d->remoteSetupState() == RemoteSetupRequested
+               || d->remoteSetupState() == RemoteSetupSucceeded,
+               qDebug() << this << "remoteSetupState" << d->remoteSetupState());
+    if (d->remoteSetupState() == RemoteSetupRequested)
+        d->setRemoteSetupState(RemoteSetupCancelled);
+
     QTC_ASSERT(state() == EngineSetupRequested, qDebug() << this << state());
     setState(EngineSetupFailed);
     if (isMasterEngine() && runControl())
@@ -771,6 +839,10 @@ void DebuggerEngine::notifyEngineSetupFailed()
 void DebuggerEngine::notifyEngineSetupOk()
 {
     showMessage(_("NOTE: ENGINE SETUP OK"));
+    QTC_ASSERT(d->remoteSetupState() == RemoteSetupNone
+               || d->remoteSetupState() == RemoteSetupSucceeded,
+               qDebug() << this << "remoteSetupState" << d->remoteSetupState());
+
     QTC_ASSERT(state() == EngineSetupRequested, qDebug() << this << state());
     setState(EngineSetupOk);
     showMessage(_("QUEUE: SETUP INFERIOR"));
@@ -796,6 +868,7 @@ void DebuggerEngine::notifyInferiorSetupFailed()
 {
     showMessage(_("NOTE: INFERIOR SETUP FAILED"));
     QTC_ASSERT(state() == InferiorSetupRequested, qDebug() << this << state());
+    showStatusMessage(tr("Setup failed."));
     setState(InferiorSetupFailed);
     if (isMasterEngine())
         d->queueShutdownEngine();
@@ -831,6 +904,7 @@ void DebuggerEngine::notifyInferiorUnrunnable()
     d->m_progress.setProgressValue(1000);
     d->m_progress.reportFinished();
     QTC_ASSERT(state() == EngineRunRequested, qDebug() << this << state());
+    showStatusMessage(tr("Loading finished."));
     setState(InferiorUnrunnable);
 }
 
@@ -841,17 +915,59 @@ void DebuggerEngine::notifyEngineRunFailed()
     d->m_progress.setProgressValue(900);
     d->m_progress.reportCanceled();
     d->m_progress.reportFinished();
+    showStatusMessage(tr("Run failed."));
     setState(EngineRunFailed);
     if (isMasterEngine())
         d->queueShutdownEngine();
 }
 
+void DebuggerEngine::notifyEngineRequestRemoteSetup()
+{
+    showMessage(_("NOTE: REQUEST REMOTE SETUP"));
+    QTC_ASSERT(state() == EngineSetupRequested, qDebug() << this << state());
+    QTC_ASSERT(d->remoteSetupState() == RemoteSetupNone, qDebug() << this
+               << "remoteSetupState" << d->remoteSetupState());
+
+    d->setRemoteSetupState(RemoteSetupRequested);
+    emit requestRemoteSetup();
+}
+
+void DebuggerEngine::notifyEngineRemoteSetupDone()
+{
+    showMessage(_("NOTE: REMOTE SETUP DONE"));
+    QTC_ASSERT(state() == EngineSetupRequested
+               || state() == EngineSetupFailed
+               || state() == DebuggerFinished, qDebug() << this << state());
+
+    QTC_ASSERT(d->remoteSetupState() == RemoteSetupRequested
+               || d->remoteSetupState() == RemoteSetupCancelled,
+               qDebug() << this << "remoteSetupState" << d->remoteSetupState());
+
+    if (d->remoteSetupState() == RemoteSetupCancelled)
+        return;
+
+    d->setRemoteSetupState(RemoteSetupSucceeded);
+}
+
+void DebuggerEngine::notifyEngineRemoteSetupFailed()
+{
+    showMessage(_("NOTE: REMOTE SETUP FAILED"));
+    QTC_ASSERT(state() == EngineSetupRequested
+               || state() == EngineSetupFailed
+               || state() == DebuggerFinished, qDebug() << this << state());
+
+    QTC_ASSERT(d->remoteSetupState() == RemoteSetupRequested
+               || d->remoteSetupState() == RemoteSetupCancelled,
+               qDebug() << this << "remoteSetupState" << d->remoteSetupState());
+}
+
 void DebuggerEngine::notifyEngineRunAndInferiorRunOk()
 {
     showMessage(_("NOTE: ENGINE RUN AND INFERIOR RUN OK"));
     d->m_progress.setProgressValue(1000);
     d->m_progress.reportFinished();
     QTC_ASSERT(state() == EngineRunRequested, qDebug() << this << state());
+    showStatusMessage(tr("Running."));
     setState(InferiorRunOk);
 }
 
@@ -861,6 +977,7 @@ void DebuggerEngine::notifyEngineRunAndInferiorStopOk()
     d->m_progress.setProgressValue(1000);
     d->m_progress.reportFinished();
     QTC_ASSERT(state() == EngineRunRequested, qDebug() << this << state());
+    showStatusMessage(tr("Stopped."));
     setState(InferiorStopOk);
 }
 
@@ -868,12 +985,14 @@ void DebuggerEngine::notifyInferiorRunRequested()
 {
     showMessage(_("NOTE: INFERIOR RUN REQUESTED"));
     QTC_ASSERT(state() == InferiorStopOk, qDebug() << this << state());
+    showStatusMessage(tr("Run requested..."));
     setState(InferiorRunRequested);
 }
 
 void DebuggerEngine::notifyInferiorRunOk()
 {
     showMessage(_("NOTE: INFERIOR RUN OK"));
+    showStatusMessage(tr("Running."));
     // Transition from StopRequested can happen sin remotegdbadapter.
     QTC_ASSERT(state() == InferiorRunRequested
         || state() == InferiorStopRequested, qDebug() << this << state());
@@ -910,6 +1029,7 @@ void DebuggerEngine::notifyInferiorStopOk()
         return;
     }
     QTC_ASSERT(state() == InferiorStopRequested, qDebug() << this << state());
+    showStatusMessage(tr("Stopped."));
     setState(InferiorStopOk);
 }
 
@@ -917,6 +1037,7 @@ void DebuggerEngine::notifyInferiorSpontaneousStop()
 {
     showMessage(_("NOTE: INFERIOR SPONTANEOUS STOP"));
     QTC_ASSERT(state() == InferiorRunOk, qDebug() << this << state());
+    showStatusMessage(tr("Stopped."));
     setState(InferiorStopOk);
 }
 
@@ -935,6 +1056,7 @@ void DebuggerEnginePrivate::doInterruptInferior()
     QTC_ASSERT(state() == InferiorRunOk, qDebug() << m_engine << state());
     m_engine->setState(InferiorStopRequested);
     m_engine->showMessage(_("CALL: INTERRUPT INFERIOR"));
+    m_engine->showStatusMessage(tr("Attempting to interrupt."));
     m_engine->interruptInferior();
 }
 
@@ -1024,6 +1146,27 @@ void DebuggerEnginePrivate::doFinishDebugger()
         m_runControl->debuggingFinished();
 }
 
+void DebuggerEnginePrivate::setRemoteSetupState(RemoteSetupState state)
+{
+    bool allowedTransition = true;
+    if (m_remoteSetupState == RemoteSetupNone) {
+        if (state == RemoteSetupRequested)
+            allowedTransition = true;
+    }
+    if (m_remoteSetupState == RemoteSetupRequested) {
+        if (state == RemoteSetupCancelled
+                || state == RemoteSetupSucceeded
+                || state == RemoteSetupFailed)
+            allowedTransition = true;
+    }
+
+
+    if (!allowedTransition)
+        qDebug() << "*** UNEXPECTED REMOTE SETUP TRANSITION from"
+                 << m_remoteSetupState << "to" << state;
+    m_remoteSetupState = state;
+}
+
 void DebuggerEngine::notifyEngineIll()
 {
     showMessage(_("NOTE: ENGINE ILL ******"));
@@ -1078,20 +1221,30 @@ void DebuggerEngine::slaveEngineStateChanged(DebuggerEngine *slaveEngine,
     Q_UNUSED(state);
 }
 
+static inline QString msgStateChanged(DebuggerState oldState, DebuggerState newState,
+                                      bool forced, bool master)
+{
+    QString result;
+    QTextStream str(&result);
+    str << "State changed";
+    if (forced)
+        str << " BY FORCE";
+    str << " from " << DebuggerEngine::stateName(oldState) << '(' << oldState
+        << ") to " << DebuggerEngine::stateName(newState) << '(' << newState << ')';
+    if (master)
+        str << " [master]";
+    return result;
+}
+
 void DebuggerEngine::setState(DebuggerState state, bool forced)
 {
-    if (isStateDebugging()) {
-        qDebug() << "STATUS CHANGE: " << this
-            << " FROM " << stateName(d->m_state) << " TO " << stateName(state)
-            << isMasterEngine();
-    }
+    const QString msg = msgStateChanged(d->m_state, state, forced, isMasterEngine());
+    if (isStateDebugging())
+        qDebug("%s", qPrintable(msg));
 
     DebuggerState oldState = d->m_state;
     d->m_state = state;
 
-    QString msg = _("State changed%5 from %1(%2) to %3(%4).")
-     .arg(stateName(oldState)).arg(oldState).arg(stateName(state)).arg(state)
-     .arg(forced ? " BY FORCE" : "");
     if (!forced && !isAllowedTransition(oldState, state))
         qDebug() << "*** UNEXPECTED STATE TRANSITION: " << this << msg;
 
@@ -1109,8 +1262,7 @@ void DebuggerEngine::setState(DebuggerState state, bool forced)
     showMessage(msg, LogDebug);
     updateViews();
 
-    if (isMasterEngine())
-        emit stateChanged(d->m_state);
+    emit stateChanged(d->m_state);
 
     if (isSlaveEngine())
         masterEngine()->slaveEngineStateChanged(this, state);
@@ -1144,6 +1296,16 @@ DebuggerLanguages DebuggerEngine::languages() const
     return d->m_languages;
 }
 
+QString DebuggerEngine::toFileInProject(const QUrl &fileUrl)
+{
+    // make sure file finder is properly initialized
+    d->m_fileFinder.setProjectDirectory(startParameters().projectSourceDirectory);
+    d->m_fileFinder.setProjectFiles(startParameters().projectSourceFiles);
+    d->m_fileFinder.setSysroot(startParameters().sysroot);
+
+    return d->m_fileFinder.findFile(fileUrl);
+}
+
 bool DebuggerEngine::debuggerActionsEnabled() const
 {
     return debuggerActionsEnabled(d->m_state);
@@ -1219,9 +1381,18 @@ void DebuggerEngine::quitDebugger()
     case InferiorRunOk:
         d->doInterruptInferior();
         break;
+    case EngineSetupRequested:
+        notifyEngineSetupFailed();
+        break;
+    case EngineSetupOk:
+        setState(InferiorSetupRequested);
+        notifyInferiorSetupFailed();
+        break;
     case EngineRunRequested:
         notifyEngineRunFailed();
         break;
+    case EngineShutdownRequested:
+        break;
     case EngineRunFailed:
     case DebuggerFinished:
         break;
@@ -1319,11 +1490,6 @@ void DebuggerEngine::addOptionPages(QList<Core::IOptionsPage*> *) const
 {
 }
 
-unsigned DebuggerEngine::debuggerCapabilities() const
-{
-    return 0;
-}
-
 bool DebuggerEngine::isSynchronous() const
 {
     return false;
@@ -1497,6 +1663,12 @@ void DebuggerEngine::executeDebuggerCommand(const QString &)
     showStatusMessage(tr("This debugger cannot handle user input."));
 }
 
+bool DebuggerEngine::evaluateScriptExpression(const QString &)
+{
+    showStatusMessage(tr("This debugger cannot handle user input."));
+    return false;
+}
+
 BreakHandler *DebuggerEngine::breakHandler() const
 {
     return debuggerCore()->breakHandler();
@@ -1601,20 +1773,6 @@ void DebuggerEngine::showStoppedByExceptionMessageBox(const QString &description
     showMessageBox(QMessageBox::Information, tr("Exception Triggered"), msg);
 }
 
-bool DebuggerEngine::isCppBreakpoint(const BreakpointParameters &p)
-{
-    //Qml specific breakpoint types
-    if (p.type == BreakpointAtJavaScriptThrow
-            || p.type == BreakpointOnQmlSignalHandler)
-        return false;
-
-    // Qml is currently only file
-    if (p.type != BreakpointByFileAndLine)
-        return true;
-    return !p.fileName.endsWith(QLatin1String(".qml"), Qt::CaseInsensitive)
-            && !p.fileName.endsWith(QLatin1String(".js"), Qt::CaseInsensitive);
-}
-
 void DebuggerEngine::openMemoryView(quint64 startAddr, unsigned flags,
                                     const QList<MemoryMarkup> &ml, const QPoint &pos,
                                     const QString &title, QWidget *parent)
@@ -1633,15 +1791,13 @@ void DebuggerEngine::openDisassemblerView(const Location &location)
     agent->setLocation(location);
 }
 
-void DebuggerEngine::handleRemoteSetupDone(int gdbServerPort, int qmlPort)
+void DebuggerEngine::handleRemoteSetupDone(int /*gdbServerPort*/,
+                                           int /*qmlPort*/)
 {
-    Q_UNUSED(gdbServerPort);
-    Q_UNUSED(qmlPort);
 }
 
-void DebuggerEngine::handleRemoteSetupFailed(const QString &message)
+void DebuggerEngine::handleRemoteSetupFailed(const QString &/*message*/)
 {
-    Q_UNUSED(message);
 }
 
 bool DebuggerEngine::isStateDebugging() const
@@ -1654,6 +1810,159 @@ void DebuggerEngine::setStateDebugging(bool on)
     d->m_isStateDebugging = on;
 }
 
+void DebuggerEngine::handleAutoTests()
+{
+    d->handleAutoTests();
+}
+
+bool DebuggerEngine::isAutoTestRunning() const
+{
+    return d->m_testsRunning;
+}
+
+void DebuggerEnginePrivate::handleAutoTests()
+{
+    if (!m_testsPossible)
+        return;
+
+    StackFrame frame = m_engine->stackHandler()->currentFrame();
+    if (!frame.file.endsWith(QLatin1String("debugger/simple/simple_test_app.cpp")))
+        return;
+
+    if (m_testContents.isEmpty()) {
+        QFile file(frame.file);
+        file.open(QIODevice::ReadOnly);
+        QTextStream ts(&file);
+        m_testFileName = QFileInfo(frame.file).absoluteFilePath();
+        m_testContents = ts.readAll().split(QLatin1Char('\n'));
+        if (m_testContents.isEmpty()) {
+            m_testsPossible = false;
+            return;
+        }
+        foreach (const QString &s, m_testContents) {
+            if (s.startsWith(QLatin1String("#define USE_AUTORUN"))) {
+                if (s.startsWith(QLatin1String("#define USE_AUTORUN 1"))) {
+                    m_testsPossible = true;
+                    m_breakOnError = false;
+                    m_testsRunning = true;
+                } else if (s.startsWith(QLatin1String("#define USE_AUTORUN 2"))) {
+                    m_testsPossible = true;
+                    m_testsRunning = true;
+                    m_breakOnError = true;
+                } else {
+                    m_testsPossible = false;
+                    m_testsRunning = false;
+                    m_breakOnError = false;
+                }
+                break;
+            }
+        }
+    }
+
+    if (!m_testsPossible)
+        return;
+
+    int line = frame.line;
+    if (line > 1 && line < m_testContents.size())
+        handleAutoTestLine(line);
+}
+
+void DebuggerEnginePrivate::handleAutoTestLine(int line)
+{
+    QString s = m_testContents.at(line).trimmed();
+    if (s.endsWith(QLatin1Char('.')))
+        s.chop(1);
+    int pos = s.indexOf(QLatin1String("//"));
+    if (pos == -1)
+        return;
+    s = s.mid(pos + 2).trimmed();
+    QString cmd = s.section(QLatin1Char(' '), 0, 0);
+    if (cmd == QLatin1String("Skip")) {
+        m_engine->showMessage(_("Skipping test %1").arg(line));
+        handleAutoTestLine(line + 1);
+    } else if (cmd == QLatin1String("Expand")) {
+        m_engine->showMessage(_("'Expand' found in line %1, "
+            "but is not implemented yet.").arg(line));
+        handleAutoTestLine(line + 1);
+    } else if (cmd == QLatin1String("Check")) {
+        QString name = s.section(QLatin1Char(' '), 1, 1);
+        if (name.isEmpty()) {
+            reportTestError(_("'Check' needs arguments."), line);
+        } else if (name.count(QLatin1Char('.')) >= 2) {
+            m_engine->showMessage(_("Variable %1 found in line %2 is nested "
+                "too deeply for the current implementation.").arg(name).arg(line));
+        } else {
+            QByteArray iname = "local." + name.toLatin1();
+            QString found = m_engine->watchHandler()->displayForAutoTest(iname);
+            if (found.isEmpty()) {
+                reportTestError(_("Check referes to unknown variable %1.")
+                    .arg(name), line);
+            } else {
+                QString needle = s.section(QLatin1Char(' '), 2, -1);
+                if (needle == found) {
+                    m_engine->showMessage(_("Check in line %1 for %2 was successful")
+                        .arg(line).arg(needle));
+                } else {
+                    reportTestError(_("Check for %1 failed. Got %2.")
+                        .arg(needle).arg(found), line);
+                }
+            }
+        }
+        handleAutoTestLine(line + 1);
+    } else if (cmd == QLatin1String("CheckType")) {
+        QString name = s.section(QLatin1Char(' '), 1, 1);
+        if (name.isEmpty()) {
+            reportTestError(_("'CheckType' needs arguments."), line);
+        } else if (name.count(QLatin1Char('.')) >= 2) {
+            m_engine->showMessage(_("Variable %1 found in line %2 is nested "
+                "too deeply for the current implementation.").arg(name).arg(line));
+        } else {
+            QByteArray iname = "local." + name.toLatin1();
+            QString found = m_engine->watchHandler()->displayForAutoTest(iname);
+            if (found.isEmpty()) {
+                reportTestError(_("CheckType referes to unknown variable %1.")
+                    .arg(name), line);
+            } else {
+                QString needle = s.section(QLatin1Char(' '), 2, -1);
+                if (found.endsWith(needle)) {
+                    m_engine->showMessage(_("CheckType in line %1 for %2 was successful")
+                        .arg(line).arg(needle));
+                } else {
+                    reportTestError(_("CheckType for %1 failed. Got %2.")
+                        .arg(needle).arg(found), line);
+                }
+            }
+        }
+        handleAutoTestLine(line + 1);
+    } else if (cmd == QLatin1String("Continue")) {
+        if (state() == InferiorStopOk) {
+            m_engine->showMessage(_("Continue in line %1 processed.").arg(line));
+            if (!m_breakOnError || !m_foundError)
+                m_engine->continueInferior();
+            else
+                m_foundError = false;
+        } else {
+            m_engine->showMessage(_("Auto-run aborted in line %1. State is %2.")
+                .arg(line).arg(state()));
+        }
+    }
+}
+
+void DebuggerEnginePrivate::reportTestError(const QString &msg, int line)
+{
+    m_engine->showMessage(_("### Line %1: %2").arg(line).arg(msg));
+    m_foundError = true;
+
+    if (!m_taskHub) {
+        ExtensionSystem::PluginManager *pm = ExtensionSystem::PluginManager::instance();
+        m_taskHub = pm->getObject<TaskHub>();
+        m_taskHub->addCategory(Core::Id("DebuggerTest"), tr("Debugger Test"));
+    }
+
+    Task task(Task::Error, msg, Utils::FileName::fromUserInput(m_testFileName), line + 1, Core::Id("DebuggerTest"));
+    m_taskHub->addTask(task);
+}
+
 } // namespace Debugger
 
 #include "debuggerengine.moc"
