@@ -2,7 +2,7 @@
 **
 ** This file is part of Qt Creator
 **
-** Copyright (c) 2011 Nokia Corporation and/or its subsidiary(-ies).
+** Copyright (c) 2012 Nokia Corporation and/or its subsidiary(-ies).
 **
 ** Contact: Nokia Corporation (qt-info@nokia.com)
 **
@@ -65,12 +65,17 @@
 #include <utils/fancymainwindow.h>
 #include <utils/qtcprocess.h>
 #include <coreplugin/icore.h>
+#include <coreplugin/helpmanager.h>
 #include <utils/buildablehelperlibrary.h>
 
-#include <QtCore/QDir>
-#include <QtCore/QDebug>
-#include <QtGui/QMessageBox>
-#include <QtGui/QErrorMessage>
+#include <QDir>
+#include <QCheckBox>
+#include <QSpinBox>
+#include <QDebug>
+#include <QErrorMessage>
+#include <QFormLayout>
+#include <QLabel>
+#include <QMessageBox>
 
 using namespace ProjectExplorer;
 using namespace Debugger::Internal;
@@ -147,6 +152,123 @@ static inline QString msgEngineNotAvailable(DebuggerEngineType et)
     return msgEngineNotAvailable(engineTypeName(et));
 }
 
+////////////////////////////////////////////////////////////////////////
+//
+// DebuggerRunConfigWidget
+//
+////////////////////////////////////////////////////////////////////////
+
+class DebuggerRunConfigWidget : public ProjectExplorer::RunConfigWidget
+{
+    Q_OBJECT
+
+public:
+    explicit DebuggerRunConfigWidget(RunConfiguration *runConfiguration);
+    QString displayName() const { return tr("Debugger Settings"); }
+
+private slots:
+    void useCppDebuggerToggled(bool toggled);
+    void useQmlDebuggerToggled(bool toggled);
+    void qmlDebugServerPortChanged(int port);
+
+public:
+    DebuggerRunConfigurationAspect *m_aspect; // not owned
+
+    QCheckBox *m_useCppDebugger;
+    QCheckBox *m_useQmlDebugger;
+    QSpinBox *m_debugServerPort;
+    QLabel *m_debugServerPortLabel;
+    QLabel *m_qmlDebuggerInfoLabel;
+};
+
+DebuggerRunConfigWidget::DebuggerRunConfigWidget(RunConfiguration *runConfiguration)
+{
+    m_aspect = runConfiguration->debuggerAspect();
+
+    m_useCppDebugger = new QCheckBox(tr("Enable C++"), this);
+    m_useQmlDebugger = new QCheckBox(tr("Enable QML"), this);
+
+    m_debugServerPort = new QSpinBox(this);
+    m_debugServerPort->setMinimum(1);
+    m_debugServerPort->setMaximum(65535);
+
+    m_debugServerPortLabel = new QLabel(tr("Debug port:"), this);
+    m_debugServerPortLabel->setBuddy(m_debugServerPort);
+
+    m_qmlDebuggerInfoLabel = new QLabel(tr("<a href=\""
+        "qthelp://com.nokia.qtcreator/doc/creator-debugging-qml.html"
+        "\">What are the prerequisites?</a>"));
+
+    m_useCppDebugger->setChecked(m_aspect->useCppDebugger());
+    m_useQmlDebugger->setChecked(m_aspect->useQmlDebugger());
+
+    m_debugServerPort->setValue(m_aspect->qmlDebugServerPort());
+
+    connect(m_qmlDebuggerInfoLabel, SIGNAL(linkActivated(QString)),
+            Core::HelpManager::instance(), SLOT(handleHelpRequest(QString)));
+    connect(m_useQmlDebugger, SIGNAL(toggled(bool)),
+            SLOT(useQmlDebuggerToggled(bool)));
+    connect(m_useCppDebugger, SIGNAL(toggled(bool)),
+            SLOT(useCppDebuggerToggled(bool)));
+    connect(m_debugServerPort, SIGNAL(valueChanged(int)),
+            SLOT(qmlDebugServerPortChanged(int)));
+
+    if (m_aspect->isDisplaySuppressed())
+        hide();
+
+    if (m_aspect->areQmlDebuggingOptionsSuppressed()) {
+        m_debugServerPortLabel->hide();
+        m_debugServerPort->hide();
+        m_useQmlDebugger->hide();
+    }
+
+    if (m_aspect->areCppDebuggingOptionsSuppressed())
+        m_useCppDebugger->hide();
+
+    if (m_aspect->isQmlDebuggingSpinboxSuppressed()) {
+        m_debugServerPort->hide();
+        m_debugServerPortLabel->hide();
+    }
+
+    QHBoxLayout *qmlLayout = new QHBoxLayout;
+    qmlLayout->setMargin(0);
+    qmlLayout->addWidget(m_useQmlDebugger);
+    qmlLayout->addWidget(m_debugServerPortLabel);
+    qmlLayout->addWidget(m_debugServerPort);
+    qmlLayout->addWidget(m_qmlDebuggerInfoLabel);
+    qmlLayout->addStretch();
+
+    QVBoxLayout *layout = new QVBoxLayout;
+    layout->setMargin(0);
+    layout->addWidget(m_useCppDebugger);
+    layout->addLayout(qmlLayout);
+    setLayout(layout);
+}
+
+void DebuggerRunConfigWidget::qmlDebugServerPortChanged(int port)
+{
+    m_aspect->m_qmlDebugServerPort = port;
+}
+
+void DebuggerRunConfigWidget::useCppDebuggerToggled(bool toggled)
+{
+    m_aspect->m_useCppDebugger = toggled;
+    if (!toggled && !m_useQmlDebugger->isChecked())
+        m_useQmlDebugger->setChecked(true);
+}
+
+void DebuggerRunConfigWidget::useQmlDebuggerToggled(bool toggled)
+{
+    m_debugServerPort->setEnabled(toggled);
+    m_debugServerPortLabel->setEnabled(toggled);
+
+    m_aspect->m_useQmlDebugger = toggled
+            ? DebuggerRunConfigurationAspect::EnableQmlDebugger
+            : DebuggerRunConfigurationAspect::DisableQmlDebugger;
+    if (!toggled && !m_useCppDebugger->isChecked())
+        m_useCppDebugger->setChecked(true);
+}
+
 ////////////////////////////////////////////////////////////////////////
 //
 // DebuggerRunControlPrivate
@@ -185,7 +307,7 @@ DebuggerRunControlPrivate::DebuggerRunControlPrivate(DebuggerRunControl *parent,
 DebuggerRunControl::DebuggerRunControl(RunConfiguration *runConfiguration,
                                        const DebuggerStartParameters &sp,
                                        const QPair<DebuggerEngineType, DebuggerEngineType> &masterSlaveEngineTypes)
-    : RunControl(runConfiguration, Constants::DEBUGMODE),
+    : RunControl(runConfiguration, ProjectExplorer::DebugRunMode),
       d(new DebuggerRunControlPrivate(this, runConfiguration))
 {
     connect(this, SIGNAL(finished()), SLOT(handleFinished()));
@@ -200,7 +322,7 @@ DebuggerRunControl::DebuggerRunControl(RunConfiguration *runConfiguration,
         DebuggerToolTipManager::instance()->registerEngine(d->m_engine);
     } else {
         debuggingFinished();
-        Core::ICore::instance()->showWarningWithOptions(DebuggerRunControl::tr("Debugger"), errorMessage);
+        Core::ICore::showWarningWithOptions(DebuggerRunControl::tr("Debugger"), errorMessage);
     }
 }
 
@@ -229,7 +351,7 @@ QString DebuggerRunControl::displayName() const
 
 QIcon DebuggerRunControl::icon() const
 {
-    return QIcon(ProjectExplorer::Constants::ICON_DEBUG_SMALL);
+    return QIcon(QLatin1String(ProjectExplorer::Constants::ICON_DEBUG_SMALL));
 }
 
 void DebuggerRunControl::setCustomEnvironment(Utils::Environment env)
@@ -365,7 +487,8 @@ RunConfiguration *DebuggerRunControl::runConfiguration() const
 //
 ////////////////////////////////////////////////////////////////////////
 
-static QList<DebuggerEngineType> enginesForToolChain(const Abi &toolChain)
+static QList<DebuggerEngineType> enginesForToolChain(const Abi &toolChain,
+                                                     DebuggerLanguages languages)
 {
     QList<DebuggerEngineType> result;
     switch (toolChain.binaryFormat()) {
@@ -373,6 +496,8 @@ static QList<DebuggerEngineType> enginesForToolChain(const Abi &toolChain)
     case Abi::MachOFormat:
         result.push_back(LldbEngineType);
         result.push_back(GdbEngineType);
+        if (languages & QmlLanguage)
+            result.push_back(QmlEngineType);
         break;
    case Abi::PEFormat:
         if (toolChain.osFlavor() == Abi::WindowsMSysFlavor) {
@@ -380,8 +505,10 @@ static QList<DebuggerEngineType> enginesForToolChain(const Abi &toolChain)
             result.push_back(CdbEngineType);
         } else {
             result.push_back(CdbEngineType);
-            result.push_back(GdbEngineType);
+            //result.push_back(GdbEngineType);
         }
+        if (languages & QmlLanguage)
+            result.push_back(QmlEngineType);
         break;
     case Abi::RuntimeQmlFormat:
         result.push_back(QmlEngineType);
@@ -435,9 +562,22 @@ static QList<DebuggerEngineType> enginesForExecutable(const QString &executable)
 
 // Debugger type for mode.
 static QList<DebuggerEngineType> enginesForMode(DebuggerStartMode startMode,
+                                                DebuggerLanguages languages,
                                                 bool hardConstraintsOnly)
 {
     QList<DebuggerEngineType> result;
+
+    if (languages == QmlLanguage) {
+        QTC_ASSERT(startMode == StartInternal
+                   || startMode == AttachToRemoteServer,
+                   qDebug() << "qml debugging not supported for mode"
+                            << startMode);
+
+        // Qml language only
+        result.push_back(QmlEngineType);
+        return result;
+    }
+
     switch (startMode) {
     case NoStartMode:
         break;
@@ -449,6 +589,9 @@ static QList<DebuggerEngineType> enginesForMode(DebuggerStartMode startMode,
             result.push_back(CdbEngineType); // Preferably Windows debugger for attaching locally.
 #endif
             result.push_back(GdbEngineType);
+
+            if (languages & QmlLanguage)
+                result.push_back(QmlEngineType);
         }
         break;
     case AttachCore:
@@ -457,16 +600,22 @@ static QList<DebuggerEngineType> enginesForMode(DebuggerStartMode startMode,
 #endif
         result.push_back(GdbEngineType);
         break;
-    case StartRemote:
+    case StartRemoteProcess:
     case StartRemoteGdb:
         result.push_back(GdbEngineType);
+        if (languages & QmlLanguage)
+            result.push_back(QmlEngineType);
         break;
+    case AttachToRemoteProcess:
     case AttachToRemoteServer:
         if (!hardConstraintsOnly) {
 #ifdef Q_OS_WIN
             result.push_back(CdbEngineType);
 #endif
             result.push_back(GdbEngineType);
+
+            if (languages & QmlLanguage)
+                result.push_back(QmlEngineType);
         }
         break;
     case AttachCrashedExternal:
@@ -477,9 +626,6 @@ static QList<DebuggerEngineType> enginesForMode(DebuggerStartMode startMode,
         // For now thats the only supported IPC engine.
         result.push_back(LldbEngineType);
         break;
-    case AttachToQmlPort:
-        result.push_back(QmlEngineType); // Only QML can do this
-        break;
     }
     return result;
 }
@@ -493,30 +639,32 @@ static QList<DebuggerEngineType> engineTypes(const DebuggerStartParameters &sp)
     if (!result.isEmpty())
         return result;
 
-    result = enginesForMode(sp.startMode, true);
+    result = enginesForMode(sp.startMode, sp.languages, true);
     if (!result.isEmpty())
         return result;
 
     //  'hard constraints' done (with the exception of QML ABI checked here),
     // further try to restrict available engines.
     if (sp.toolChainAbi.isValid()) {
-        result = enginesForToolChain(sp.toolChainAbi);
+        result = enginesForToolChain(sp.toolChainAbi, sp.languages);
         if (!result.isEmpty())
             return result;
     }
 
     // FIXME: 1 of 3 testing hacks.
-    if (sp.processArgs.startsWith(__("@tcf@ "))) {
+    if (sp.processArgs.startsWith(QLatin1String("@tcf@ "))) {
         result.push_back(GdbEngineType);
         return result;
     }
 
-    if (sp.startMode != AttachToRemoteServer && !sp.executable.isEmpty())
+    if (sp.startMode != AttachToRemoteServer
+            && sp.startMode != AttachToRemoteProcess
+            && !sp.executable.isEmpty())
         result = enginesForExecutable(sp.executable);
     if (!result.isEmpty())
         return result;
 
-    result = enginesForMode(sp.startMode, false);
+    result = enginesForMode(sp.startMode, sp.languages, false);
     return result;
 }
 
@@ -548,7 +696,7 @@ static inline bool canUseEngine(DebuggerEngineType et,
     // Enabled?
     if ((et & cmdLineEnabledEngines) == 0) {
         result->errorDetails.push_back(DebuggerPlugin::tr("The debugger engine '%1' is disabled.").
-                                       arg(engineTypeName(et)));
+                                       arg(QLatin1String(engineTypeName(et))));
         return false;
     }
     // Configured.
@@ -574,38 +722,32 @@ static inline bool canUseEngine(DebuggerEngineType et,
 DEBUGGER_EXPORT ConfigurationCheck checkDebugConfiguration(const DebuggerStartParameters &sp)
 {
     ConfigurationCheck result;
-    const unsigned activeLangs = debuggerCore()->activeLanguages();
-    const bool qmlLanguage = activeLangs & QmlLanguage;
-    const bool cppLanguage = activeLangs & CppLanguage;
     if (debug)
         qDebug().nospace() << "checkDebugConfiguration " << sp.toolChainAbi.toString()
                            << " Start mode=" << sp.startMode << " Executable=" << sp.executable
                            << " Debugger command=" << sp.debuggerCommand;
     // Get all applicable types.
-    QList<DebuggerEngineType> requiredTypes;
-    if (qmlLanguage && !cppLanguage) {
-        requiredTypes.push_back(QmlEngineType);
-    } else {
-        requiredTypes = engineTypes(sp);
-    }
+    QList<DebuggerEngineType> requiredTypes = engineTypes(sp);
     if (requiredTypes.isEmpty()) {
         result.errorMessage = QLatin1String("Internal error: Unable to determine debugger engine type for this configuration");
         return result;
     }
     if (debug)
         qDebug() << " Required: " << engineTypeNames(requiredTypes);
-    // Filter out disables types, command line + current settings.
+    // Filter out disabled types, command line + current settings.
     unsigned cmdLineEnabledEngines = debuggerCore()->enabledEngines();
 #ifdef WITH_LLDB
-    if (!Core::ICore::instance()->settings()->value(QLatin1String("LLDB/enabled")).toBool())
+    if (!Core::ICore::settings()->value(QLatin1String("LLDB/enabled")).toBool())
         cmdLineEnabledEngines &= ~LldbEngineType;
 #else
      cmdLineEnabledEngines &= ~LldbEngineType;
 #endif
+
     DebuggerEngineType usableType = NoEngineType;
     QList<DebuggerEngineType> unavailableTypes;
     foreach (DebuggerEngineType et, requiredTypes) {
         if (canUseEngine(et, sp, cmdLineEnabledEngines, &result)) {
+            result.errorDetails.clear();
             usableType = et;
             break;
         } else {
@@ -633,7 +775,7 @@ DEBUGGER_EXPORT ConfigurationCheck checkDebugConfiguration(const DebuggerStartPa
         const QString msg = DebuggerPlugin::tr(
             "The preferred debugger engine for debugging binaries of type '%1' is not available.\n"
             "The debugger engine '%2' will be used as a fallback.\nDetails: %3").
-                arg(sp.toolChainAbi.toString(), engineTypeName(usableType),
+                arg(sp.toolChainAbi.toString(), QLatin1String(engineTypeName(usableType)),
                     result.errorDetails.join(QString(QLatin1Char('\n'))));
         debuggerCore()->showMessage(msg, LogWarning);
         showMessageBox(QMessageBox::Warning, DebuggerPlugin::tr("Warning"), msg);
@@ -641,12 +783,17 @@ DEBUGGER_EXPORT ConfigurationCheck checkDebugConfiguration(const DebuggerStartPa
     // Anything left: Happy.
     result.errorMessage.clear();
     result.errorDetails.clear();
-    if (qmlLanguage && cppLanguage) {
-        result.masterSlaveEngineTypes.first = QmlCppEngineType;
-        result.masterSlaveEngineTypes.second = usableType;
-    } else {
-        result.masterSlaveEngineTypes.first = usableType;
-    }
+
+
+    // Could we actually use a combined qml/cpp-engine?
+     if (usableType != QmlEngineType
+             && requiredTypes.contains(QmlEngineType)) {
+         result.masterSlaveEngineTypes.first = QmlCppEngineType;
+         result.masterSlaveEngineTypes.second = usableType;
+     } else {
+         result.masterSlaveEngineTypes.first = usableType;
+     }
+
     if (debug)
         qDebug() << engineTypeName(result.masterSlaveEngineTypes.first) << engineTypeName(result.masterSlaveEngineTypes.second);
     return result;
@@ -664,10 +811,9 @@ DebuggerRunControlFactory::DebuggerRunControlFactory(QObject *parent,
     : IRunControlFactory(parent), m_enabledEngines(enabledEngines)
 {}
 
-bool DebuggerRunControlFactory::canRun(RunConfiguration *runConfiguration, const QString &mode) const
+bool DebuggerRunControlFactory::canRun(RunConfiguration *runConfiguration, RunMode mode) const
 {
-//    return mode == ProjectExplorer::Constants::DEBUGMODE;
-    return (mode == Constants::DEBUGMODE || mode == Constants::DEBUGMODE2)
+    return (mode == DebugRunMode || mode == DebugRunModeWithBreakOnMain)
             && qobject_cast<LocalApplicationRunConfiguration *>(runConfiguration);
 }
 
@@ -677,26 +823,26 @@ QString DebuggerRunControlFactory::displayName() const
 }
 
 // Find Qt installation by running qmake
-static inline QString findQtInstallPath(const QString &qmakePath)
+static inline QString findQtInstallPath(const Utils::FileName &qmakePath)
 {
     QProcess proc;
     QStringList args;
     args.append(_("-query"));
     args.append(_("QT_INSTALL_HEADERS"));
-    proc.start(qmakePath, args);
+    proc.start(qmakePath.toString(), args);
     if (!proc.waitForStarted()) {
-        qWarning("%s: Cannot start '%s': %s", Q_FUNC_INFO, qPrintable(qmakePath),
+        qWarning("%s: Cannot start '%s': %s", Q_FUNC_INFO, qPrintable(qmakePath.toString()),
            qPrintable(proc.errorString()));
         return QString();
     }
     proc.closeWriteChannel();
     if (!proc.waitForFinished()) {
         Utils::SynchronousProcess::stopProcess(proc);
-        qWarning("%s: Timeout running '%s'.", Q_FUNC_INFO, qPrintable(qmakePath));
+        qWarning("%s: Timeout running '%s'.", Q_FUNC_INFO, qPrintable(qmakePath.toString()));
         return QString();
     }
     if (proc.exitStatus() != QProcess::NormalExit) {
-        qWarning("%s: '%s' crashed.", Q_FUNC_INFO, qPrintable(qmakePath));
+        qWarning("%s: '%s' crashed.", Q_FUNC_INFO, qPrintable(qmakePath.toString()));
         return QString();
     }
     const QByteArray ba = proc.readAllStandardOutput().trimmed();
@@ -714,7 +860,6 @@ static DebuggerStartParameters localStartParameters(RunConfiguration *runConfigu
             qobject_cast<LocalApplicationRunConfiguration *>(runConfiguration);
     QTC_ASSERT(rc, return sp);
 
-    sp.startMode = StartInternal;
     sp.environment = rc->environment();
     sp.workingDirectory = rc->workingDirectory();
 
@@ -724,10 +869,13 @@ static DebuggerStartParameters localStartParameters(RunConfiguration *runConfigu
 #endif
 
     sp.executable = rc->executable();
+    if (sp.executable.isEmpty())
+        return sp;
+    sp.startMode = StartInternal;
     sp.processArgs = rc->commandLineArguments();
     sp.toolChainAbi = rc->abi();
     if (!sp.toolChainAbi.isValid()) {
-        QList<Abi> abis = Abi::abisOfBinary(sp.executable);
+        QList<Abi> abis = Abi::abisOfBinary(Utils::FileName::fromString(sp.executable));
         if (!abis.isEmpty())
             sp.toolChainAbi = abis.at(0);
     }
@@ -737,7 +885,7 @@ static DebuggerStartParameters localStartParameters(RunConfiguration *runConfigu
 
     if (const ProjectExplorer::Target *target = runConfiguration->target()) {
         if (QByteArray(target->metaObject()->className()).contains("Qt4")) {
-            const QString qmake = Utils::BuildableHelperLibrary::findSystemQt(sp.environment);
+            const Utils::FileName qmake = Utils::BuildableHelperLibrary::findSystemQt(sp.environment);
             if (!qmake.isEmpty())
                 sp.qtInstallPath = findQtInstallPath(qmake);
         }
@@ -746,15 +894,19 @@ static DebuggerStartParameters localStartParameters(RunConfiguration *runConfigu
             if (const ProjectExplorer::BuildConfiguration *buildConfig = target->activeBuildConfiguration()) {
                 sp.projectBuildDirectory = buildConfig->buildDirectory();
                 if (const ProjectExplorer::ToolChain *tc = buildConfig->toolChain())
-                    sp.debuggerCommand = tc->debuggerCommand();
+                    sp.debuggerCommand = tc->debuggerCommand().toString();
             }
             sp.projectSourceFiles = project->files(Project::ExcludeGeneratedFiles);
         }
     }
 
-    if (debuggerCore()->isActiveDebugLanguage(QmlLanguage)) {
+    if (runConfiguration->debuggerAspect()->useCppDebugger())
+        sp.languages |= CppLanguage;
+
+    if (runConfiguration->debuggerAspect()->useQmlDebugger()) {
         sp.qmlServerAddress = _("127.0.0.1");
-        sp.qmlServerPort = runConfiguration->qmlDebugServerPort();
+        sp.qmlServerPort = runConfiguration->debuggerAspect()->qmlDebugServerPort();
+        sp.languages |= QmlLanguage;
 
         // Makes sure that all bindings go through the JavaScript engine, so that
         // breakpoints are actually hit!
@@ -763,8 +915,8 @@ static DebuggerStartParameters localStartParameters(RunConfiguration *runConfigu
             sp.environment.set(optimizerKey, _("1"));
         }
 
-        Utils::QtcProcess::addArg(&sp.processArgs, QString("-qmljsdebugger=port:%1,block").arg(
-                                      sp.qmlServerPort));
+        Utils::QtcProcess::addArg(&sp.processArgs,
+                                  QString::fromLatin1("-qmljsdebugger=port:%1,block").arg(sp.qmlServerPort));
     }
 
     // FIXME: If it's not yet build this will be empty and not filled
@@ -777,11 +929,13 @@ static DebuggerStartParameters localStartParameters(RunConfiguration *runConfigu
 }
 
 RunControl *DebuggerRunControlFactory::create
-    (RunConfiguration *runConfiguration, const QString &mode)
+    (RunConfiguration *runConfiguration, RunMode mode)
 {
-    QTC_ASSERT(mode == Constants::DEBUGMODE || mode == Constants::DEBUGMODE2, return 0);
+    QTC_ASSERT(mode == DebugRunMode || mode == DebugRunModeWithBreakOnMain, return 0);
     DebuggerStartParameters sp = localStartParameters(runConfiguration);
-    if (mode == Constants::DEBUGMODE2)
+    if (sp.startMode == NoStartMode)
+        return 0;
+    if (mode == DebugRunModeWithBreakOnMain)
         sp.breakOnMain = true;
     return create(sp, runConfiguration);
 }
@@ -789,9 +943,7 @@ RunControl *DebuggerRunControlFactory::create
 RunConfigWidget *DebuggerRunControlFactory::createConfigurationWidget
     (RunConfiguration *runConfiguration)
 {
-    // NBS TODO: Add GDB-specific configuration widget
-    Q_UNUSED(runConfiguration)
-    return 0;
+    return new DebuggerRunConfigWidget(runConfiguration);
 }
 
 DebuggerRunControl *DebuggerRunControlFactory::create
@@ -801,7 +953,7 @@ DebuggerRunControl *DebuggerRunControlFactory::create
 
     if (!check) {
         //appendMessage(errorMessage, true);
-        Core::ICore::instance()->showWarningWithOptions(DebuggerRunControl::tr("Debugger"),
+        Core::ICore::showWarningWithOptions(DebuggerRunControl::tr("Debugger"),
             check.errorMessage, check.errorDetailsString(), check.settingsCategory, check.settingsPage);
         return 0;
     }
@@ -836,5 +988,6 @@ DebuggerEngine *DebuggerRunControlFactory::createEngine
     return 0;
 }
 
-
 } // namespace Debugger
+
+#include "debuggerrunner.moc"
