@@ -2,7 +2,7 @@
 **
 ** This file is part of Qt Creator
 **
-** Copyright (c) 2011 Nokia Corporation and/or its subsidiary(-ies).
+** Copyright (c) 2012 Nokia Corporation and/or its subsidiary(-ies).
 **
 ** Contact: Nokia Corporation (qt-info@nokia.com)
 **
@@ -62,10 +62,9 @@ private:
     AbstractGdbProcess *gdbProc() { return &m_gdbProc; }
     DumperHandling dumperHandling() const { return DumperLoadedByGdbPreload; }
 
-    virtual QByteArray execFilePath() const;
-    virtual bool infoTargetNecessary() const;
-    virtual QByteArray toLocalEncoding(const QString &s) const;
-    virtual QString fromLocalEncoding(const QByteArray &b) const;
+    QByteArray execFilePath() const;
+    QByteArray toLocalEncoding(const QString &s) const;
+    QString fromLocalEncoding(const QByteArray &b) const;
     void handleApplicationOutput(const QByteArray &output);
 
     RemoteGdbProcess m_gdbProc;
