@@ -2,7 +2,7 @@
 **
 ** This file is part of Qt Creator
 **
-** Copyright (c) 2011 Nokia Corporation and/or its subsidiary(-ies).
+** Copyright (c) 2012 Nokia Corporation and/or its subsidiary(-ies).
 **
 ** Contact: Nokia Corporation (qt-info@nokia.com)
 **
@@ -44,13 +44,13 @@
 #include <utils/qtcassert.h>
 #include <utils/savedaction.h>
 
-#include <QtCore/QDebug>
+#include <QDebug>
 
-#include <QtGui/QHeaderView>
-#include <QtGui/QItemDelegate>
-#include <QtGui/QMenu>
-#include <QtGui/QPainter>
-#include <QtGui/QResizeEvent>
+#include <QHeaderView>
+#include <QItemDelegate>
+#include <QMenu>
+#include <QPainter>
+#include <QResizeEvent>
 
 
 namespace Debugger {
@@ -113,8 +113,8 @@ public:
         QTC_ASSERT(lineEdit, return);
         const int base = currentHandler()->numberBase();
         QString value = lineEdit->text();
-        if (base == 16 && !value.startsWith("0x"))
-            value = "0x" + value;
+        if (base == 16 && !value.startsWith(QLatin1String("0x")))
+            value.insert(0, QLatin1String("0x"));
         currentEngine()->setRegisterValue(index.row(), value);
     }
 
@@ -179,12 +179,11 @@ void RegisterWindow::contextMenuEvent(QContextMenuEvent *ev)
     DebuggerEngine *engine = currentEngine();
     QTC_ASSERT(engine, return);
     RegisterHandler *handler = currentHandler();
-    const unsigned engineCapabilities = engine->debuggerCapabilities();
     const bool actionsEnabled = engine->debuggerActionsEnabled();
     const int state = engine->state();
 
     QAction *actReload = menu.addAction(tr("Reload Register Listing"));
-    actReload->setEnabled((engineCapabilities & RegisterCapability)
+    actReload->setEnabled(engine->hasCapability(RegisterCapability)
         && (state == InferiorStopOk || state == InferiorUnrunnable));
 
     menu.addSeparator();
@@ -201,17 +200,17 @@ void RegisterWindow::contextMenuEvent(QContextMenuEvent *ev)
 
     QAction *actShowDisassemblerAt = menu.addAction(QString());
     QAction *actShowDisassembler = menu.addAction(tr("Open Disassembler..."));
-    actShowDisassembler->setEnabled(engineCapabilities & DisassemblerCapability);
+    actShowDisassembler->setEnabled(engine->hasCapability(DisassemblerCapability));
 
     if (address) {
-        const bool canShow = actionsEnabled && (engineCapabilities & ShowMemoryCapability);
+        const bool canShow = actionsEnabled && engine->hasCapability(ShowMemoryCapability);
         actEditMemory->setText(tr("Open Memory Editor at 0x%1").arg(address, 0, 16));
         actEditMemory->setEnabled(canShow);
         actViewMemory->setText(tr("Open Memory View at Value of Register %1 0x%2")
-            .arg(QString::fromAscii(aRegister.name)).arg(address, 0, 16));
+            .arg(QString::fromLatin1(aRegister.name)).arg(address, 0, 16));
         actShowDisassemblerAt->setText(tr("Open Disassembler at 0x%1")
             .arg(address, 0, 16));
-        actShowDisassemblerAt->setEnabled(engineCapabilities & DisassemblerCapability);
+        actShowDisassemblerAt->setEnabled(engine->hasCapability(DisassemblerCapability));
     } else {
         actEditMemory->setText(tr("Open Memory Editor"));
         actViewMemory->setText(tr("Open Memory View at Value of Register"));
@@ -244,7 +243,7 @@ void RegisterWindow::contextMenuEvent(QContextMenuEvent *ev)
     if (act == actReload)
         engine->reloadRegisters();
     else if (act == actEditMemory) {
-        const QString registerName = QString::fromAscii(aRegister.name);
+        const QString registerName = QString::fromLatin1(aRegister.name);
         engine->openMemoryView(address, 0,
             RegisterMemoryView::registerMarkup(address, registerName),
             QPoint(), RegisterMemoryView::title(registerName), 0);
