@@ -2,7 +2,7 @@
 **
 ** This file is part of Qt Creator
 **
-** Copyright (c) 2011 Nokia Corporation and/or its subsidiary(-ies).
+** Copyright (c) 2012 Nokia Corporation and/or its subsidiary(-ies).
 **
 ** Contact: Nokia Corporation (qt-info@nokia.com)
 **
@@ -36,7 +36,7 @@
 
 #include <utils/qtcassert.h>
 
-#include <coreplugin/ifile.h>
+#include <coreplugin/idocument.h>
 
 #include <texteditor/basetexteditor.h>
 #include <texteditor/basetextmark.h>
@@ -55,16 +55,16 @@
 
 #include <extensionsystem/pluginmanager.h>
 
-#include <QtCore/QCoreApplication>
-#include <QtCore/QDateTime>
-#include <QtCore/QDebug>
-#include <QtCore/QHash>
-#include <QtCore/QStringList>
-#include <QtCore/QTextStream>
-#include <QtCore/QTime>
+#include <QCoreApplication>
+#include <QDateTime>
+#include <QDebug>
+#include <QHash>
+#include <QStringList>
+#include <QTextStream>
+#include <QTime>
 
-#include <QtGui/QTextCursor>
-#include <QtGui/QPlainTextEdit>
+#include <QTextCursor>
+#include <QPlainTextEdit>
 
 #include <string.h>
 #include <ctype.h>
@@ -151,7 +151,7 @@ bool isEditorDebuggable(Core::IEditor *editor)
 {
     // Only blacklist Qml. Whitelisting would fail on C++ code in files
     // with strange names, more harm would be done this way.
-    //   IFile *file = editor->file();
+    //   IDocument *file = editor->document();
     //   return !(file && file->mimeType() == "application/x-qml");
     // Nowadays, even Qml is debuggable.
     return editor;
@@ -442,9 +442,9 @@ bool getUninitializedVariables(const CPlusPlus::Snapshot &snapshot,
 
 QByteArray gdbQuoteTypes(const QByteArray &type)
 {
-    // gdb does not understand sizeof(Core::IFile*).
-    // "sizeof('Core::IFile*')" is also not acceptable,
-    // it needs to be "sizeof('Core::IFile'*)"
+    // gdb does not understand sizeof(Core::IDocument*).
+    // "sizeof('Core::IDocument*')" is also not acceptable,
+    // it needs to be "sizeof('Core::IDocument'*)"
     //
     // We never will have a perfect solution here (even if we had a full blown
     // C++ parser as we do not have information on what is a type and what is
@@ -510,10 +510,10 @@ QString quoteUnprintableLatin1(const QByteArray &ba)
     for (int i = 0, n = ba.size(); i != n; ++i) {
         const unsigned char c = ba.at(i);
         if (isprint(c)) {
-            res += c;
+            res += QLatin1Char(c);
         } else {
             qsnprintf(buf, sizeof(buf) - 1, "\\%x", int(c));
-            res += buf;
+            res += QLatin1String(buf);
         }
     }
     return res;
@@ -647,10 +647,10 @@ QString decodeData(const QByteArray &ba, int encoding)
 bool isCppEditor(Core::IEditor *editor)
 {
     using namespace CppTools::Constants;
-    const Core::IFile *file = editor->file();
-    if (!file)
+    const Core::IDocument *document= editor->document();
+    if (!document)
         return false;
-    const QByteArray mimeType = file->mimeType().toLatin1();
+    const QByteArray mimeType = document->mimeType().toLatin1();
     return mimeType == C_SOURCE_MIMETYPE
         || mimeType == CPP_SOURCE_MIMETYPE
         || mimeType == CPP_HEADER_MIMETYPE
@@ -669,7 +669,7 @@ QString cppExpressionAt(TextEditor::ITextEditor *editor, int pos,
 
     const QPlainTextEdit *plaintext = qobject_cast<QPlainTextEdit*>(editor->widget());
     if (!plaintext)
-        return QByteArray();
+        return QString();
 
     QString expr = plaintext->textCursor().selectedText();
     CppModelManagerInterface *modelManager = CppModelManagerInterface::instance();
@@ -693,12 +693,12 @@ QString cppExpressionAt(TextEditor::ITextEditor *editor, int pos,
     }
 
     if (function && !expr.isEmpty())
-        if (const Core::IFile *file = editor->file())
+        if (const Core::IDocument *document= editor->document())
             if (modelManager)
                 *function = AbstractEditorSupport::functionAt(modelManager,
-                    file->fileName(), *line, *column);
+                    document->fileName(), *line, *column);
 
-    return expr.toUtf8();
+    return expr;
 }
 
 QString cppFunctionAt(const QString &fileName, int line)
@@ -862,7 +862,7 @@ void parseWatchData(const QSet<QByteArray> &expandedINames,
 
     bool ok = false;
     qulonglong addressBase = item.findChild("addrbase").data().toULongLong(&ok, 0);
-    qulonglong addressStep = item.findChild("addrstep").data().toULongLong();
+    qulonglong addressStep = item.findChild("addrstep").data().toULongLong(&ok, 0);
 
     // Try not to repeat data too often.
     WatchData childtemplate;
@@ -888,7 +888,7 @@ void parseWatchData(const QSet<QByteArray> &expandedINames,
             data1.iname += data1.name.toLatin1();
         }
         if (!data1.name.isEmpty() && data1.name.at(0).isDigit())
-            data1.name = _c('[') + data1.name + _c(']');
+            data1.name = QLatin1Char('[') + data1.name + QLatin1Char(']');
         if (addressStep) {
             setWatchDataAddress(data1, addressBase);
             addressBase += addressStep;
