@@ -2,7 +2,7 @@
 **
 ** This file is part of Qt Creator
 **
-** Copyright (c) 2011 Nokia Corporation and/or its subsidiary(-ies).
+** Copyright (c) 2012 Nokia Corporation and/or its subsidiary(-ies).
 **
 ** Contact: Nokia Corporation (qt-info@nokia.com)
 **
@@ -34,7 +34,7 @@
 #define SETTINGSMANAGER_H
 
 #if QT_VERSION >= 0x050000
-#    include <QtDesigner/QDesignerSettingsInterface>
+#    include <QDesignerSettingsInterface>
 #else
 #    include "qt_private/abstractsettings_p.h"
 #endif
