@@ -2,7 +2,7 @@
 **
 ** This file is part of Qt Creator
 **
-** Copyright (c) 2011 Nokia Corporation and/or its subsidiary(-ies).
+** Copyright (c) 2012 Nokia Corporation and/or its subsidiary(-ies).
 **
 ** Contact: Nokia Corporation (qt-info@nokia.com)
 **
@@ -44,9 +44,9 @@
 #include <coreplugin/dialogs/ioptionspage.h>
 #include <coreplugin/editormanager/editormanager.h>
 #include <coreplugin/editormanager/openeditorsmodel.h>
-#include <coreplugin/filemanager.h>
+#include <coreplugin/documentmanager.h>
 #include <coreplugin/icore.h>
-#include <coreplugin/ifile.h>
+#include <coreplugin/idocument.h>
 #include <coreplugin/messagemanager.h>
 #include <coreplugin/id.h>
 #include <coreplugin/statusbarwidget.h>
@@ -82,23 +82,23 @@
 
 #include <cpptools/cpptoolsconstants.h>
 
-#include <QtCore/QAbstractTableModel>
-#include <QtCore/QDebug>
-#include <QtCore/QFile>
-#include <QtCore/QtPlugin>
-#include <QtCore/QObject>
-#include <QtCore/QSettings>
-#include <QtCore/QTextStream>
+#include <QAbstractTableModel>
+#include <QDebug>
+#include <QFile>
+#include <QtPlugin>
+#include <QObject>
+#include <QSettings>
+#include <QTextStream>
 
-#include <QtGui/QDesktopServices>
-#include <QtGui/QItemDelegate>
-#include <QtGui/QMessageBox>
-#include <QtGui/QPlainTextEdit>
-#include <QtGui/QShortcut>
-#include <QtGui/QTextBlock>
-#include <QtGui/QTextCursor>
-#include <QtGui/QTextEdit>
-#include <QtGui/QTreeWidgetItem>
+#include <QDesktopServices>
+#include <QItemDelegate>
+#include <QMessageBox>
+#include <QPlainTextEdit>
+#include <QShortcut>
+#include <QTextBlock>
+#include <QTextCursor>
+#include <QTextEdit>
+#include <QTreeWidgetItem>
 
 using namespace FakeVim::Internal;
 using namespace TextEditor;
@@ -194,7 +194,7 @@ public:
         { return QIcon(_(Constants::SETTINGS_CATEGORY_FAKEVIM_ICON)); }
 
     QWidget *createPage(QWidget *parent);
-    void apply() { m_group.apply(ICore::instance()->settings()); }
+    void apply() { m_group.apply(ICore::settings()); }
     void finish() { m_group.finish(); }
     virtual bool matches(const QString &) const;
 
@@ -380,7 +380,7 @@ QWidget *FakeVimExCommandsPage::createPage(QWidget *parent)
 
 void FakeVimExCommandsPage::initialize()
 {
-    ActionManager *am = ICore::instance()->actionManager();
+    ActionManager *am = ICore::actionManager();
     QTC_ASSERT(am, return);
 
     QMap<QString, QTreeWidgetItem *> sections;
@@ -390,7 +390,7 @@ void FakeVimExCommandsPage::initialize()
             continue;
 
         QTreeWidgetItem *item = new QTreeWidgetItem;
-        const QString name = Id::fromUniqueIdentifier(c->id()).toString();
+        const QString name = c->id().toString();
         const int pos = name.indexOf(QLatin1Char('.'));
         const QString section = name.left(pos);
         const QString subId = name.mid(pos + 1);
@@ -408,16 +408,7 @@ void FakeVimExCommandsPage::initialize()
         sections[section]->addChild(item);
 
         item->setText(0, subId);
-
-        if (c->action()) {
-            QString text = c->hasAttribute(Command::CA_UpdateText)
-                    && !c->defaultText().isNull()
-                ? c->defaultText() : c->action()->text();
-            text.remove(QRegExp("&(?!&)"));
-            item->setText(1, text);
-        } else {
-            item->setText(1, c->shortcut()->whatsThis());
-        }
+        item->setText(1, c->description());
 
         QString regex;
         if (exCommandMap().contains(name))
@@ -630,12 +621,12 @@ void FakeVimUserCommandsPage::apply()
 class FakeVimCompletionAssistProvider : public TextEditor::CompletionAssistProvider
 {
 public:
-    virtual bool supportsEditor(const QString &) const
+    bool supportsEditor(const Core::Id &) const
     {
         return false;
     }
 
-    virtual TextEditor::IAssistProcessor *createProcessor() const;
+    TextEditor::IAssistProcessor *createProcessor() const;
 
     void setActive(const QString &needle, bool forward, FakeVimHandler *handler)
     {
@@ -735,7 +726,7 @@ public:
 
         const int basePosition = interface->position() - needle.size();
 
-        QTextCursor tc(interface->document());
+        QTextCursor tc(interface->textDocument());
         tc.setPosition(interface->position());
         tc.movePosition(QTextCursor::Start, QTextCursor::MoveAnchor);
 
@@ -844,8 +835,8 @@ private:
     FakeVimUserCommandsPage *m_fakeVimUserCommandsPage;
     QHash<Core::IEditor *, FakeVimHandler *> m_editorToHandler;
 
-    void triggerAction(const QString &code);
-    void setActionChecked(const QString &code, bool check);
+    void triggerAction(const Core::Id &id);
+    void setActionChecked(const Core::Id &id, bool check);
 
     typedef int (*DistFunction)(const QRect &cursor, const QRect &other);
     void moveSomewhere(DistFunction f);
@@ -938,7 +929,7 @@ FakeVimPluginPrivate::~FakeVimPluginPrivate()
 void FakeVimPluginPrivate::onCoreAboutToClose()
 {
     // Don't attach to editors anymore.
-    disconnect(ICore::instance()->editorManager(), SIGNAL(editorOpened(Core::IEditor*)),
+    disconnect(ICore::editorManager(), SIGNAL(editorOpened(Core::IEditor*)),
         this, SLOT(editorOpened(Core::IEditor*)));
 }
 
@@ -948,9 +939,8 @@ void FakeVimPluginPrivate::aboutToShutdown()
 
 bool FakeVimPluginPrivate::initialize()
 {
-    Core::ICore *core = Core::ICore::instance();
-    EditorManager *editorManager = core->editorManager();
-    ActionManager *actionManager = core->actionManager();
+    EditorManager *editorManager = ICore::editorManager();
+    ActionManager *actionManager = ICore::actionManager();
     QTC_ASSERT(actionManager, return false);
 
     //m_wordCompletion = new WordCompletion;
@@ -995,12 +985,12 @@ bool FakeVimPluginPrivate::initialize()
         act->setData(i);
         QString id = QString("FakeVim.UserAction%1").arg(i);
         QString keys = QString("Alt+V,%1").arg(i);
-        cmd = actionManager->registerAction(act, id, globalcontext);
+        cmd = actionManager->registerAction(act, Core::Id(id), globalcontext);
         cmd->setDefaultKeySequence(QKeySequence(keys));
         connect(act, SIGNAL(triggered()), SLOT(userActionTriggered()));
     }
 
-    connect(core, SIGNAL(coreAboutToClose()), this, SLOT(onCoreAboutToClose()));
+    connect(ICore::instance(), SIGNAL(coreAboutToClose()), this, SLOT(onCoreAboutToClose()));
 
     // EditorManager
     connect(editorManager, SIGNAL(editorAboutToClose(Core::IEditor*)),
@@ -1033,9 +1023,7 @@ void FakeVimPluginPrivate::userActionTriggered()
     if (!key)
         return;
     QString cmd = userCommandMap().value(key);
-    Core::ICore *core = Core::ICore::instance();
-    EditorManager *editorManager = core->editorManager();
-    IEditor *editor = editorManager->currentEditor();
+    IEditor *editor = ICore::editorManager()->currentEditor();
     FakeVimHandler *handler = m_editorToHandler[editor];
     if (handler)
         handler->handleInput(cmd);
@@ -1049,7 +1037,7 @@ static const char idKey[] = "Command";
 
 void FakeVimPluginPrivate::writeSettings()
 {
-    QSettings *settings = ICore::instance()->settings();
+    QSettings *settings = ICore::settings();
 
     theFakeVimSettings()->writeSettings(settings);
 
@@ -1097,7 +1085,7 @@ void FakeVimPluginPrivate::writeSettings()
 
 void FakeVimPluginPrivate::readSettings()
 {
-    QSettings *settings = ICore::instance()->settings();
+    QSettings *settings = ICore::settings();
 
     theFakeVimSettings()->readSettings(settings);
 
@@ -1143,27 +1131,27 @@ void FakeVimPluginPrivate::maybeReadVimRc()
 
 void FakeVimPluginPrivate::showSettingsDialog()
 {
-    ICore::instance()->showOptionsDialog(
+    ICore::showOptionsDialog(
         _(Constants::SETTINGS_CATEGORY),
         _(Constants::SETTINGS_ID));
 }
 
-void FakeVimPluginPrivate::triggerAction(const QString &code)
+void FakeVimPluginPrivate::triggerAction(const Id &id)
 {
-    Core::ActionManager *am = ICore::instance()->actionManager();
+    Core::ActionManager *am = ICore::actionManager();
     QTC_ASSERT(am, return);
-    Core::Command *cmd = am->command(code);
-    QTC_ASSERT(cmd, qDebug() << "UNKNOWN CODE: " << code; return);
+    Core::Command *cmd = am->command(id);
+    QTC_ASSERT(cmd, qDebug() << "UNKNOWN CODE: " << id.name(); return);
     QAction *action = cmd->action();
     QTC_ASSERT(action, return);
     action->trigger();
 }
 
-void FakeVimPluginPrivate::setActionChecked(const QString &code, bool check)
+void FakeVimPluginPrivate::setActionChecked(const Id &id, bool check)
 {
-    Core::ActionManager *am = ICore::instance()->actionManager();
+    Core::ActionManager *am = ICore::actionManager();
     QTC_ASSERT(am, return);
-    Core::Command *cmd = am->command(code);
+    Core::Command *cmd = am->command(id);
     QTC_ASSERT(cmd, return);
     QAction *action = cmd->action();
     QTC_ASSERT(action, return);
@@ -1269,7 +1257,7 @@ void FakeVimPluginPrivate::windowCommand(int key)
 
 void FakeVimPluginPrivate::moveSomewhere(DistFunction f)
 {
-    EditorManager *editorManager = ICore::instance()->editorManager();
+    EditorManager *editorManager = ICore::editorManager();
     IEditor *editor = editorManager->currentEditor();
     QWidget *w = editor->widget();
     QPlainTextEdit *pe =
@@ -1400,7 +1388,7 @@ void FakeVimPluginPrivate::editorOpened(Core::IEditor *editor)
     connect(ICore::instance(), SIGNAL(saveSettingsRequested()),
         SLOT(writeSettings()));
 
-    handler->setCurrentFileName(editor->file()->fileName());
+    handler->setCurrentFileName(editor->document()->fileName());
     handler->installEventFilter();
 
     // pop up the bar
@@ -1507,9 +1495,9 @@ void FakeVimPluginPrivate::handleExCommand(bool *handled, const ExCommand &cmd)
         // :w[rite]
         Core::IEditor *editor = m_editorToHandler.key(handler);
         const QString fileName = handler->currentFileName();
-        if (editor && editor->file()->fileName() == fileName) {
+        if (editor && editor->document()->fileName() == fileName) {
             // Handle that as a special case for nicer interaction with core
-            Core::ICore::instance()->fileManager()->saveFile(editor->file());
+            DocumentManager::saveDocument(editor->document());
             // Check result by reading back.
             QFile file3(fileName);
             file3.open(QIODevice::ReadOnly);
@@ -1524,9 +1512,8 @@ void FakeVimPluginPrivate::handleExCommand(bool *handled, const ExCommand &cmd)
         }
     } else if (cmd.matches("wa", "wall")) {
         // :w[all]
-        FileManager *fm = ICore::instance()->fileManager();
-        QList<IFile *> toSave = fm->modifiedFiles();
-        QList<IFile *> failed = fm->saveModifiedFilesSilently(toSave);
+        QList<IDocument *> toSave = DocumentManager::modifiedDocuments();
+        QList<IDocument *> failed = DocumentManager::saveModifiedDocumentsSilently(toSave);
         if (failed.isEmpty())
             handler->showBlackMessage(tr("Saving succeeded"));
         else
@@ -1575,6 +1562,12 @@ void FakeVimPluginPrivate::handleExCommand(bool *handled, const ExCommand &cmd)
         // :on[ly]
         //triggerAction(Core::Constants::REMOVE_ALL_SPLITS);
         triggerAction(Core::Constants::REMOVE_CURRENT_SPLIT);
+    } else if (cmd.cmd == "AS") {
+        triggerAction(Core::Constants::SPLIT);
+        triggerAction(CppTools::Constants::SWITCH_HEADER_SOURCE);
+    } else if (cmd.cmd == "AV") {
+        triggerAction(Core::Constants::SPLIT_SIDE_BY_SIDE);
+        triggerAction(CppTools::Constants::SWITCH_HEADER_SOURCE);
     } else {
         // Check whether one of the configure commands matches.
         typedef ExCommandMap::const_iterator Iterator;
@@ -1583,7 +1576,7 @@ void FakeVimPluginPrivate::handleExCommand(bool *handled, const ExCommand &cmd)
             const QString &id = it.key();
             const QRegExp &re = it.value();
             if (!re.pattern().isEmpty() && re.indexIn(cmd.cmd) != -1) {
-                triggerAction(id);
+                triggerAction(Core::Id(id));
                 return;
             }
         }
@@ -1595,7 +1588,7 @@ void FakeVimPluginPrivate::handleDelayedQuit(bool forced, Core::IEditor *editor)
 {
     // This tries to simulate vim behaviour. But the models of vim and
     // Qt Creator core do not match well...
-    EditorManager *editorManager = ICore::instance()->editorManager();
+    EditorManager *editorManager = ICore::editorManager();
     if (editorManager->hasSplitter()) {
         triggerAction(Core::Constants::REMOVE_CURRENT_SPLIT);
     } else {
@@ -1608,7 +1601,7 @@ void FakeVimPluginPrivate::handleDelayedQuit(bool forced, Core::IEditor *editor)
 void FakeVimPluginPrivate::handleDelayedQuitAll(bool forced)
 {
     triggerAction(Core::Constants::REMOVE_ALL_SPLITS);
-    ICore::instance()->editorManager()->closeAllEditors(!forced);
+    ICore::editorManager()->closeAllEditors(!forced);
 }
 
 void FakeVimPluginPrivate::moveToMatchingParenthesis(bool *moved, bool *forward,
@@ -1714,7 +1707,7 @@ void FakeVimPluginPrivate::changeSelection
 
 int FakeVimPluginPrivate::currentFile() const
 {
-    EditorManager *editorManager = ICore::instance()->editorManager();
+    EditorManager *editorManager = ICore::editorManager();
     OpenEditorsModel *model = editorManager->openedEditorsModel();
     IEditor *editor = editorManager->currentEditor();
     return model->indexOf(editor).row();
@@ -1722,7 +1715,7 @@ int FakeVimPluginPrivate::currentFile() const
 
 void FakeVimPluginPrivate::switchToFile(int n)
 {
-    EditorManager *editorManager = ICore::instance()->editorManager();
+    EditorManager *editorManager = ICore::editorManager();
     Core::OpenEditorsModel *model = editorManager->openedEditorsModel();
     int size = model->rowCount();
     QTC_ASSERT(size, return);
