@@ -2,7 +2,7 @@
 **
 ** This file is part of Qt Creator
 **
-** Copyright (c) 2011 Nokia Corporation and/or its subsidiary(-ies).
+** Copyright (c) 2012 Nokia Corporation and/or its subsidiary(-ies).
 **
 ** Contact: Nokia Corporation (qt-info@nokia.com)
 **
@@ -34,12 +34,11 @@
 
 #include <utils/synchronousprocess.h>
 
-#include <QtCore/QCoreApplication>
+#include <QCoreApplication>
 
 namespace Git {
 namespace Internal {
 
-const QLatin1String GitSettings::adoptPathKey("SysEnv");
 const QLatin1String GitSettings::pathKey("Path");
 const QLatin1String GitSettings::pullRebaseKey("PullRebase");
 const QLatin1String GitSettings::omitAnnotationDateKey("OmitAnnotationDate");
@@ -49,6 +48,7 @@ const QLatin1String GitSettings::diffPatienceKey("DiffPatience");
 const QLatin1String GitSettings::winSetHomeEnvironmentKey("WinSetHomeEnvironment");
 const QLatin1String GitSettings::showPrettyFormatKey("DiffPrettyFormat");
 const QLatin1String GitSettings::gitkOptionsKey("GitKOptions");
+const QLatin1String GitSettings::logDiffKey("LogDiff");
 
 GitSettings::GitSettings()
 {
@@ -60,7 +60,6 @@ GitSettings::GitSettings()
 #else
     declareKey(timeoutKey, 30);
 #endif
-    declareKey(adoptPathKey, false);
     declareKey(pathKey, QString());
     declareKey(pullRebaseKey, false);
     declareKey(omitAnnotationDateKey, false);
@@ -69,7 +68,8 @@ GitSettings::GitSettings()
     declareKey(diffPatienceKey, true);
     declareKey(winSetHomeEnvironmentKey, false);
     declareKey(gitkOptionsKey, QString());
-    declareKey(showPrettyFormatKey, 5);
+    declareKey(showPrettyFormatKey, 2);
+    declareKey(logDiffKey, false);
 }
 
 QString GitSettings::gitBinaryPath(bool *ok, QString *errorMessage) const
@@ -85,7 +85,7 @@ QString GitSettings::gitBinaryPath(bool *ok, QString *errorMessage) const
         const QString binary = stringValue(binaryPathKey);
         QString currentPath = stringValue(pathKey);
         // Easy, git is assumed to be elsewhere accessible
-        if (!boolValue(adoptPathKey))
+        if (currentPath.isEmpty())
             currentPath = QString::fromLocal8Bit(qgetenv("PATH"));
         // Search in path?
         m_binaryPath = Utils::SynchronousProcess::locateBinary(currentPath, binary);
@@ -103,7 +103,7 @@ QString GitSettings::gitBinaryPath(bool *ok, QString *errorMessage) const
 
 GitSettings &GitSettings::operator = (const GitSettings &s)
 {
-    VCSBaseClientSettings::operator =(s);
+    VcsBaseClientSettings::operator =(s);
     m_binaryPath.clear();
     return *this;
 }
