@@ -2,7 +2,7 @@
 **
 ** This file is part of Qt Creator
 **
-** Copyright (c) 2011 Nokia Corporation and/or its subsidiary(-ies).
+** Copyright (c) 2012 Nokia Corporation and/or its subsidiary(-ies).
 **
 ** Contact: Nokia Corporation (qt-info@nokia.com)
 **
@@ -33,7 +33,7 @@
 #ifndef REUSE_H
 #define REUSE_H
 
-#include <QtCore/QtGlobal>
+#include <QtGlobal>
 
 namespace GLSLEditor {
 namespace Internal {
