@@ -2,7 +2,7 @@
 **
 ** This file is part of Qt Creator
 **
-** Copyright (c) 2011 Nokia Corporation and/or its subsidiary(-ies).
+** Copyright (c) 2012 Nokia Corporation and/or its subsidiary(-ies).
 ** Copyright (c) 2010 Denis Mingulov.
 **
 ** Contact: Nokia Corporation (qt-info@nokia.com)
@@ -36,9 +36,10 @@
 #include "imageviewerconstants.h"
 #include "imageviewer.h"
 
-#include <QtCore/QMap>
-#include <QtGui/QImageReader>
-#include <QtCore/QtDebug>
+#include <QCoreApplication>
+#include <QMap>
+#include <QImageReader>
+#include <QtDebug>
 
 namespace ImageViewer {
 namespace Internal {
@@ -102,10 +103,10 @@ Core::Id ImageViewerFactory::id() const
 
 QString ImageViewerFactory::displayName() const
 {
-    return tr(Constants::IMAGEVIEWER_DISPLAY_NAME);
+    return qApp->translate("OpenWith::Editors", Constants::IMAGEVIEWER_DISPLAY_NAME);
 }
 
-Core::IFile *ImageViewerFactory::open(const QString & /*fileName*/)
+Core::IDocument *ImageViewerFactory::open(const QString & /*fileName*/)
 {
     return 0;
 }
