@@ -2,7 +2,7 @@
 **
 ** This file is part of Qt Creator
 **
-** Copyright (c) 2011 Nokia Corporation and/or its subsidiary(-ies).
+** Copyright (c) 2012 Nokia Corporation and/or its subsidiary(-ies).
 ** Copyright (c) 2010 Denis Mingulov.
 **
 ** Contact: Nokia Corporation (qt-info@nokia.com)
@@ -34,16 +34,16 @@
 #ifndef IMAGEVIEWERFILE_H
 #define IMAGEVIEWERFILE_H
 
-#include <coreplugin/ifile.h>
+#include <coreplugin/idocument.h>
 
-#include <QtCore/QScopedPointer>
+#include <QScopedPointer>
 
 namespace ImageViewer {
 namespace Internal {
 
 class ImageViewer;
 
-class ImageViewerFile : public Core::IFile
+class ImageViewerFile : public Core::IDocument
 {
     Q_OBJECT
 public:
@@ -59,9 +59,9 @@ public:
     QString mimeType() const;
 
     bool isModified() const;
-    bool isReadOnly() const;
     bool isSaveAsAllowed() const;
 
+    virtual ReloadBehavior reloadBehavior(ChangeTrigger state, ChangeType type) const;
     bool reload(QString *errorString, ReloadFlag flag, ChangeType type);
 
     void setMimetype(const QString &mimetype);
