@@ -2,7 +2,7 @@
 **
 ** This file is part of Qt Creator
 **
-** Copyright (c) 2011 Nokia Corporation and/or its subsidiary(-ies).
+** Copyright (c) 2012 Nokia Corporation and/or its subsidiary(-ies).
 **
 ** Contact: Nokia Corporation (qt-info@nokia.com)
 **
@@ -35,9 +35,9 @@
 
 #include "locator_global.h"
 
-#include <QtCore/QVariant>
-#include <QtCore/QFutureInterface>
-#include <QtGui/QIcon>
+#include <QVariant>
+#include <QFutureInterface>
+#include <QIcon>
 
 namespace Locator {
 
@@ -133,19 +133,28 @@ public:
     /* Returns whether the filter should be hidden from configuration and menus. */
     bool isHidden() const;
 
+    /* Returns whether the filter should be enabled and used in menus. */
+    bool isEnabled() const;
+
     static QString trimWildcards(const QString &str) {
         if (str.isEmpty())
             return str;
         int first = 0, last = str.size()-1;
-        while (first < str.size() && (str.at(first) == '*' || str.at(first) == '?'))
+        const QChar asterisk = QLatin1Char('*');
+        const QChar question = QLatin1Char('?');
+        while (first < str.size() && (str.at(first) == asterisk || str.at(first) == question))
             ++first;
-        while (last >= 0 && (str.at(last) == '*' || str.at(last) == '?'))
+        while (last >= 0 && (str.at(last) == asterisk || str.at(last) == question))
             --last;
         if (first > last)
             return QString();
         return str.mid(first, last-first+1);
     }
 
+public slots:
+    /* Enable or disable the filter. */
+    void setEnabled(bool enabled);
+
 protected:
     void setShortcutString(const QString &shortcut);
     void setIncludedByDefault(bool includedByDefault);
@@ -155,6 +164,7 @@ private:
     QString m_shortcut;
     bool m_includedByDefault;
     bool m_hidden;
+    bool m_enabled;
 };
 
 } // namespace Locator
