@@ -2,7 +2,7 @@
 **
 ** This file is part of Qt Creator
 **
-** Copyright (c) 2011 Nokia Corporation and/or its subsidiary(-ies).
+** Copyright (c) 2012 Nokia Corporation and/or its subsidiary(-ies).
 **
 ** Contact: Nokia Corporation (qt-info@nokia.com)
 **
@@ -37,7 +37,7 @@
 #include "qt4maemodeployconfiguration.h"
 #include "qt4maemotarget.h"
 
-#include <coreplugin/ifile.h>
+#include <coreplugin/idocument.h>
 #include <projectexplorer/project.h>
 #include <projectexplorer/target.h>
 #include <qt4projectmanager/qmakestep.h>
@@ -47,12 +47,14 @@
 #include <remotelinux/deploymentinfo.h>
 #include <utils/fileutils.h>
 #include <utils/qtcassert.h>
+#include <utils/ssh/sshremoteprocessrunner.h>
 
-#include <QtCore/QCoreApplication>
-#include <QtCore/QDir>
-#include <QtCore/QFileInfo>
-#include <QtCore/QStringList>
-#include <QtGui/QIcon>
+#include <QCoreApplication>
+#include <QDir>
+#include <QFile>
+#include <QFileInfo>
+#include <QStringList>
+#include <QIcon>
 
 using namespace Core;
 using namespace Qt4ProjectManager;
@@ -67,7 +69,8 @@ MaemoPublisherFremantleFree::MaemoPublisherFremantleFree(const ProjectExplorer::
     QObject(parent),
     m_project(project),
     m_state(Inactive),
-    m_sshParams(SshConnectionParameters::DefaultProxy)
+    m_sshParams(SshConnectionParameters::DefaultProxy),
+    m_uploader(0)
 {
     m_sshParams.authenticationType = SshConnectionParameters::AuthenticationByKey;
     m_sshParams.timeout = 30;
@@ -125,7 +128,7 @@ void MaemoPublisherFremantleFree::createPackage()
     m_tmpProjectDir = tmpDirContainer() + QLatin1Char('/')
         + m_project->displayName();
     if (QFileInfo(tmpDirContainer()).exists()) {
-        emit progressReport(tr("Removing left-over temporary directory ..."));
+        emit progressReport(tr("Removing left-over temporary directory..."));
         QString error;
         if (!Utils::FileUtils::removeRecursively(tmpDirContainer(), &error)) {
             finishWithFailure(tr("Error removing temporary directory: %1").arg(error),
@@ -134,7 +137,7 @@ void MaemoPublisherFremantleFree::createPackage()
         }
     }
 
-    emit progressReport(tr("Setting up temporary directory ..."));
+    emit progressReport(tr("Setting up temporary directory..."));
     if (!QDir::temp().mkdir(QFileInfo(tmpDirContainer()).fileName())) {
         finishWithFailure(tr("Error: Could not create temporary directory."),
             tr("Publishing failed: Could not create source package."));
@@ -158,7 +161,7 @@ void MaemoPublisherFremantleFree::createPackage()
         return;
     }
 
-    emit progressReport(tr("Cleaning up temporary directory ..."));
+    emit progressReport(tr("Cleaning up temporary directory..."));
     AbstractMaemoPackageCreationStep::preparePackagingProcess(m_process,
             m_buildConfig, m_tmpProjectDir);
     setState(RunningQmake);
@@ -216,7 +219,7 @@ bool MaemoPublisherFremantleFree::copyRecursively(const QString &srcFilePath,
             QByteArray rulesContents = reader.data();
             rulesContents.replace("$(MAKE) clean", "# $(MAKE) clean");
             rulesContents.replace("# Add here commands to configure the package.",
-                "qmake " + QFileInfo(m_project->file()->fileName()).fileName().toLocal8Bit());
+                "qmake " + QFileInfo(m_project->document()->fileName()).fileName().toLocal8Bit());
             MaemoDebianPackageCreationStep::ensureShlibdeps(rulesContents);
             Utils::FileSaver saver(tgtFilePath);
             saver.write(rulesContents);
@@ -224,6 +227,12 @@ bool MaemoPublisherFremantleFree::copyRecursively(const QString &srcFilePath,
                 emit progressReport(saver.errorString(), ErrorOutput);
                 return false;
             }
+            QFile rulesFile(tgtFilePath);
+            if (!rulesFile.setPermissions(rulesFile.permissions() | QFile::ExeUser)) {
+                emit progressReport(tr("Could not set execute permissions for rules file: %1")
+                    .arg(rulesFile.errorString()));
+                return false;
+            }
         } else {
             QFile srcFile(srcFilePath);
             if (!srcFile.copy(tgtFilePath)) {
@@ -376,25 +385,23 @@ void MaemoPublisherFremantleFree::runDpkgBuildPackage()
     emit progressReport(tr("Building source package..."));
     const QStringList args = QStringList() << QLatin1String("dpkg-buildpackage")
         << QLatin1String("-S") << QLatin1String("-us") << QLatin1String("-uc");
-    MaemoGlobal::callMad(*m_process, args, lqt->qmakeCommand(), true);
+    MaemoGlobal::callMad(*m_process, args, lqt->qmakeCommand().toString(), true);
 }
 
 // We have to implement the SCP protocol, because the maemo.org
 // webmaster refuses to enable SFTP "for security reasons" ...
 void MaemoPublisherFremantleFree::uploadPackage()
 {
-    m_uploader = SshRemoteProcessRunner::create(m_sshParams);
-    connect(m_uploader.data(), SIGNAL(processStarted()),
-        SLOT(handleScpStarted()));
-    connect(m_uploader.data(), SIGNAL(connectionError(Utils::SshError)),
-        SLOT(handleConnectionError()));
-    connect(m_uploader.data(), SIGNAL(processClosed(int)),
-        SLOT(handleUploadJobFinished(int)));
-    connect(m_uploader.data(), SIGNAL(processOutputAvailable(QByteArray)),
+    if (!m_uploader)
+        m_uploader = new SshRemoteProcessRunner(this);
+    connect(m_uploader, SIGNAL(processStarted()), SLOT(handleScpStarted()));
+    connect(m_uploader, SIGNAL(connectionError()), SLOT(handleConnectionError()));
+    connect(m_uploader, SIGNAL(processClosed(int)), SLOT(handleUploadJobFinished(int)));
+    connect(m_uploader, SIGNAL(processOutputAvailable(QByteArray)),
         SLOT(handleScpStdOut(QByteArray)));
-    emit progressReport(tr("Starting scp ..."));
+    emit progressReport(tr("Starting scp..."));
     setState(StartingScp);
-    m_uploader->run("scp -td " + m_remoteDir.toUtf8());
+    m_uploader->run("scp -td " + m_remoteDir.toUtf8(), m_sshParams);
 }
 
 void MaemoPublisherFremantleFree::handleScpStarted()
@@ -408,7 +415,7 @@ void MaemoPublisherFremantleFree::handleScpStarted()
 void MaemoPublisherFremantleFree::handleConnectionError()
 {
     if (m_state != Inactive) {
-        finishWithFailure(tr("SSH error: %1").arg(m_uploader->connection()->errorString()),
+        finishWithFailure(tr("SSH error: %1").arg(m_uploader->lastConnectionErrorString()),
             tr("Upload failed."));
     }
 }
@@ -419,11 +426,11 @@ void MaemoPublisherFremantleFree::handleUploadJobFinished(int exitStatus)
         return);
 
     if (m_state != Inactive && (exitStatus != SshRemoteProcess::ExitedNormally
-            || m_uploader->process()->exitCode() != 0)) {
+            || m_uploader->processExitCode() != 0)) {
         QString error;
         if (exitStatus != SshRemoteProcess::ExitedNormally) {
             error = tr("Error uploading file: %1.")
-                .arg(m_uploader->process()->errorString());
+                .arg(m_uploader->processErrorString());
         } else {
             error = tr("Error uploading file.");
         }
@@ -444,10 +451,10 @@ void MaemoPublisherFremantleFree::prepareToSendFile()
 
     setState(PreparingToUploadFile);
     const QString &nextFilePath = m_filesToUpload.first();
-    emit progressReport(tr("Uploading file %1 ...")
+    emit progressReport(tr("Uploading file %1...")
         .arg(QDir::toNativeSeparators(nextFilePath)));
     QFileInfo info(nextFilePath);
-    m_uploader->process()->sendInput("C0644 " + QByteArray::number(info.size())
+    m_uploader->writeDataToProcess("C0644 " + QByteArray::number(info.size())
         + ' ' + info.fileName().toUtf8() + '\n');
 }
 
@@ -473,13 +480,13 @@ void MaemoPublisherFremantleFree::sendFile()
                 tr("Upload failed."));
             return;
         }
-        m_uploader->process()->sendInput(data);
+        m_uploader->writeDataToProcess(data);
         bytesToSend -= data.size();
         QCoreApplication::processEvents();
         if (m_state == Inactive)
             return;
     }
-    m_uploader->process()->sendInput(QByteArray(1, '\0'));
+    m_uploader->writeDataToProcess(QByteArray(1, '\0'));
 }
 
 void MaemoPublisherFremantleFree::handleScpStdOut(const QByteArray &output)
@@ -536,8 +543,7 @@ bool MaemoPublisherFremantleFree::updateDesktopFiles(QString *error) const
     bool success = true;
     const Qt4MaemoDeployConfiguration * const deployConfig
         = qobject_cast<Qt4MaemoDeployConfiguration *>(m_buildConfig->target()->activeDeployConfiguration());
-    const QSharedPointer<DeploymentInfo> deploymentInfo
-        = deployConfig->deploymentInfo();
+    const DeploymentInfo * const deploymentInfo = deployConfig->deploymentInfo();
     for (int i = 0; i < deploymentInfo->modelCount(); ++i) {
         const DeployableFilesPerProFile * const model = deploymentInfo->modelAt(i);
         QString desktopFilePath = deployConfig->localDesktopFilePath(model);
@@ -631,8 +637,7 @@ void MaemoPublisherFremantleFree::setState(State newState)
             // TODO: Can we ensure the remote scp exits, e.g. by sending
             //       an illegal sequence of bytes? (Probably not, if
             //       we are currently uploading a file.)
-            disconnect(m_uploader.data(), 0, this, 0);
-            m_uploader = SshRemoteProcessRunner::Ptr();
+            disconnect(m_uploader, 0, this, 0);
             break;
         default:
             break;
