@@ -2,7 +2,7 @@
 **
 ** This file is part of Qt Creator
 **
-** Copyright (c) 2011 Nokia Corporation and/or its subsidiary(-ies).
+** Copyright (c) 2012 Nokia Corporation and/or its subsidiary(-ies).
 **
 ** Contact: Nokia Corporation (qt-info@nokia.com)
 **
@@ -37,10 +37,10 @@
 
 #include <vcsbase/vcsbaseconstants.h>
 
-#include <QtGui/QApplication>
-#include <QtGui/QLineEdit>
-#include <QtGui/QFileDialog>
-#include <QtCore/QTextStream>
+#include <QApplication>
+#include <QLineEdit>
+#include <QFileDialog>
+#include <QTextStream>
 
 using namespace Perforce::Internal;
 using namespace Utils;
@@ -144,7 +144,7 @@ SettingsPage::SettingsPage()
 
 QString SettingsPage::id() const
 {
-    return QLatin1String(VCSBase::Constants::VCS_ID_PERFORCE);
+    return QLatin1String(VcsBase::Constants::VCS_ID_PERFORCE);
 }
 
 QString SettingsPage::displayName() const
