@@ -2,7 +2,7 @@
 **
 ** This file is part of Qt Creator
 **
-** Copyright (c) 2011 Nokia Corporation and/or its subsidiary(-ies).
+** Copyright (c) 2012 Nokia Corporation and/or its subsidiary(-ies).
 **
 ** Contact: Nokia Corporation (qt-info@nokia.com)
 **
@@ -52,14 +52,14 @@
 #include <utils/qtcassert.h>
 #include <utils/outputformatter.h>
 
-#include <QtGui/QAction>
-#include <QtGui/QVBoxLayout>
-#include <QtGui/QTabWidget>
-#include <QtGui/QToolButton>
-#include <QtGui/QTabBar>
-#include <QtGui/QMenu>
+#include <QAction>
+#include <QVBoxLayout>
+#include <QTabWidget>
+#include <QToolButton>
+#include <QTabBar>
+#include <QMenu>
 
-#include <QtCore/QDebug>
+#include <QDebug>
 
 enum { debug = 0 };
 
@@ -68,7 +68,7 @@ using namespace ProjectExplorer::Internal;
 
 static QObject *debuggerCore()
 {
-    return ExtensionSystem::PluginManager::instance()->getObjectByName("DebuggerCore");
+    return ExtensionSystem::PluginManager::instance()->getObjectByName(QLatin1String("DebuggerCore"));
 }
 
 static QString msgAttachDebuggerTooltip(const QString &handleDescription = QString())
@@ -125,7 +125,7 @@ AppOutputPane::AppOutputPane() :
     m_attachButton(new QToolButton)
 {
     // Rerun
-    m_reRunButton->setIcon(QIcon(ProjectExplorer::Constants::ICON_RUN_SMALL));
+    m_reRunButton->setIcon(QIcon(QLatin1String(ProjectExplorer::Constants::ICON_RUN_SMALL)));
     m_reRunButton->setToolTip(tr("Re-run this run-configuration"));
     m_reRunButton->setAutoRaise(true);
     m_reRunButton->setEnabled(false);
@@ -133,7 +133,7 @@ AppOutputPane::AppOutputPane() :
             this, SLOT(reRunRunControl()));
 
     // Stop
-    Core::ActionManager *am = Core::ICore::instance()->actionManager();
+    Core::ActionManager *am = Core::ICore::actionManager();
     Core::Context globalcontext(Core::Constants::C_GLOBAL);
 
     QIcon stopIcon = QIcon(QLatin1String(Constants::ICON_STOP));
@@ -153,7 +153,7 @@ AppOutputPane::AppOutputPane() :
     // Attach
     m_attachButton->setToolTip(msgAttachDebuggerTooltip());
     m_attachButton->setEnabled(false);
-    m_attachButton->setIcon(QIcon(ProjectExplorer::Constants::ICON_DEBUG_SMALL));
+    m_attachButton->setIcon(QIcon(QLatin1String(ProjectExplorer::Constants::ICON_DEBUG_SMALL)));
     m_attachButton->setAutoRaise(true);
 
     connect(m_attachButton, SIGNAL(clicked()),
@@ -174,7 +174,7 @@ AppOutputPane::AppOutputPane() :
 
     m_mainWidget->setLayout(layout);
 
-    connect(ProjectExplorerPlugin::instance()->session(), SIGNAL(aboutToUnloadSession()),
+    connect(ProjectExplorerPlugin::instance()->session(), SIGNAL(aboutToUnloadSession(QString)),
             this, SLOT(aboutToUnloadSession()));
     connect(ProjectExplorerPlugin::instance(), SIGNAL(settingsChanged()),
             this, SLOT(updateFromSettings()));
