@@ -2,7 +2,7 @@
 **
 ** This file is part of Qt Creator
 **
-** Copyright (c) 2011 Nokia Corporation and/or its subsidiary(-ies).
+** Copyright (c) 2012 Nokia Corporation and/or its subsidiary(-ies).
 **
 ** Contact: Nokia Corporation (qt-info@nokia.com)
 **
@@ -35,9 +35,9 @@
 
 #include "iprojectproperties.h"
 
-#include <QtCore/QAbstractListModel>
+#include <QAbstractListModel>
 
-#include <QtGui/QTreeView>
+#include <QTreeView>
 
 namespace Utils {
     class DetailsWidget;
@@ -50,7 +50,7 @@ class SessionManager;
 
 namespace Internal {
 
-const char * const DEPENDENCIES_PANEL_ID("ProjectExplorer.DependenciesPanel");
+const char DEPENDENCIES_PANEL_ID[] = "ProjectExplorer.DependenciesPanel";
 
 class DependenciesWidget;
 
@@ -61,6 +61,7 @@ public:
 
     QString id() const;
     QString displayName() const;
+    int priority() const;
     bool supports(Project *project);
     PropertiesPanel *createPanel(Project *project);
 private:
