@@ -2,7 +2,7 @@
 **
 ** This file is part of Qt Creator
 **
-** Copyright (c) 2011 Nokia Corporation and/or its subsidiary(-ies).
+** Copyright (c) 2012 Nokia Corporation and/or its subsidiary(-ies).
 **
 ** Contact: Nokia Corporation (qt-info@nokia.com)
 **
@@ -43,15 +43,15 @@
 #include <utils/qtcassert.h>
 #include <utils/pathchooser.h>
 
-#include <QtCore/QBuffer>
-#include <QtCore/QCoreApplication>
-#include <QtCore/QFileInfo>
-#include <QtCore/QProcess>
-#include <QtCore/QScopedPointer>
+#include <QBuffer>
+#include <QCoreApplication>
+#include <QFileInfo>
+#include <QProcess>
+#include <QScopedPointer>
 
-#include <QtGui/QComboBox>
-#include <QtGui/QFormLayout>
-#include <QtGui/QLabel>
+#include <QComboBox>
+#include <QFormLayout>
+#include <QLabel>
 
 namespace ProjectExplorer {
 
@@ -59,14 +59,14 @@ namespace ProjectExplorer {
 // Helpers:
 // --------------------------------------------------------------------------
 
-static const char compilerPathKeyC[] = "ProjectExplorer.GccToolChain.Path";
+static const char compilerCommandKeyC[] = "ProjectExplorer.GccToolChain.Path";
 static const char targetAbiKeyC[] = "ProjectExplorer.GccToolChain.TargetAbi";
 static const char supportedAbisKeyC[] = "ProjectExplorer.GccToolChain.SupportedAbis";
 static const char debuggerCommandKeyC[] = "ProjectExplorer.GccToolChain.Debugger";
 
-static QByteArray runGcc(const QString &gcc, const QStringList &arguments, const QStringList &env)
+static QByteArray runGcc(const Utils::FileName &gcc, const QStringList &arguments, const QStringList &env)
 {
-    if (gcc.isEmpty() || !QFileInfo(gcc).isExecutable())
+    if (gcc.isEmpty() || !gcc.toFileInfo().isExecutable())
         return QByteArray();
 
     QProcess cpp;
@@ -75,33 +75,59 @@ static QByteArray runGcc(const QString &gcc, const QStringList &arguments, const
     environment.append(QLatin1String("LC_ALL=C"));
 
     cpp.setEnvironment(environment);
-    cpp.start(gcc, arguments);
+    cpp.start(gcc.toString(), arguments);
     if (!cpp.waitForStarted()) {
-        qWarning("%s: Cannot start '%s': %s", Q_FUNC_INFO, qPrintable(gcc),
+        qWarning("%s: Cannot start '%s': %s", Q_FUNC_INFO, qPrintable(gcc.toUserOutput()),
             qPrintable(cpp.errorString()));
         return QByteArray();
     }
     cpp.closeWriteChannel();
     if (!cpp.waitForFinished()) {
         Utils::SynchronousProcess::stopProcess(cpp);
-        qWarning("%s: Timeout running '%s'.", Q_FUNC_INFO, qPrintable(gcc));
+        qWarning("%s: Timeout running '%s'.", Q_FUNC_INFO, qPrintable(gcc.toUserOutput()));
         return QByteArray();
     }
     if (cpp.exitStatus() != QProcess::NormalExit) {
-        qWarning("%s: '%s' crashed.", Q_FUNC_INFO, qPrintable(gcc));
+        qWarning("%s: '%s' crashed.", Q_FUNC_INFO, qPrintable(gcc.toUserOutput()));
         return QByteArray();
     }
 
     return cpp.readAllStandardOutput() + '\n' + cpp.readAllStandardError();
 }
 
-static QByteArray gccPredefinedMacros(const QString &gcc, const QStringList &env)
+static QByteArray gccPredefinedMacros(const Utils::FileName &gcc, const QStringList &args, const QStringList &env)
 {
     QStringList arguments;
     arguments << QLatin1String("-xc++")
               << QLatin1String("-E")
-              << QLatin1String("-dM")
-              << QLatin1String("-");
+              << QLatin1String("-dM");
+    foreach (const QString &a, args) {
+        if (a == QLatin1String("-m128bit-long-double") || a == QLatin1String("-m32")
+                || a == QLatin1String("-m3dnow") || a == QLatin1String("-m3dnowa")
+                || a == QLatin1String("-m64") || a == QLatin1String("-m96bit-long-double")
+                || a == QLatin1String("-mabm") || a == QLatin1String("-maes")
+                || a.startsWith(QLatin1String("-march=")) || a == QLatin1String("-mavx")
+                || a.startsWith(QLatin1String("-masm=")) || a == QLatin1String("-mcx16")
+                || a == QLatin1String("-mfma") || a == QLatin1String("-mfma4")
+                || a == QLatin1String("-mlwp") || a == QLatin1String("-mpclmul")
+                || a == QLatin1String("-mpopcnt") || a == QLatin1String("-msse")
+                || a == QLatin1String("-msse2") || a == QLatin1String("-msse2avx")
+                || a == QLatin1String("-msse3") || a == QLatin1String("-msse4")
+                || a == QLatin1String("-msse4.1") || a == QLatin1String("-msse4.2")
+                || a == QLatin1String("-msse4a") || a == QLatin1String("-mssse3")
+                || a.startsWith(QLatin1String("-mtune=")) || a == QLatin1String("-mxop")
+                || a == QLatin1String("-Os") || a == QLatin1String("-O0") || a == QLatin1String("-O1")
+                || a == QLatin1String("-O2") || a == QLatin1String("-O3")
+                || a == QLatin1String("-ffinite-math-only") || a == QLatin1String("-fshort-double")
+                || a == QLatin1String("-fshort-wchar") || a == QLatin1String("-fsignaling-nans")
+                || a.startsWith(QLatin1String("-std=")) || a.startsWith(QLatin1String("-specs="))
+                || a == QLatin1String("-ansi")
+                || a.startsWith(QLatin1String("-D")) || a.startsWith(QLatin1String("-U"))
+                || a == QLatin1String("-undef"))
+            arguments << a;
+    }
+
+    arguments << QLatin1String("-");
 
     QByteArray predefinedMacros = runGcc(gcc, arguments, env);
 #ifdef Q_OS_MAC
@@ -120,7 +146,7 @@ static QByteArray gccPredefinedMacros(const QString &gcc, const QStringList &env
     return predefinedMacros;
 }
 
-static QList<HeaderPath> gccHeaderPathes(const QString &gcc, const QStringList &env)
+static QList<HeaderPath> gccHeaderPathes(const Utils::FileName &gcc, const QStringList &env)
 {
     QList<HeaderPath> systemHeaderPaths;
     QStringList arguments;
@@ -145,7 +171,7 @@ static QList<HeaderPath> gccHeaderPathes(const QString &gcc, const QStringList &
             line = cpp.readLine();
             if (line.startsWith("#include")) {
                 kind = HeaderPath::GlobalHeaderPath;
-            } else if (! line.isEmpty() && QChar(line.at(0)).isSpace()) {
+            } else if (! line.isEmpty() && QChar(QLatin1Char(line.at(0))).isSpace()) {
                 HeaderPath::Kind thisHeaderKind = kind;
 
                 line = line.trimmed();
@@ -160,7 +186,7 @@ static QList<HeaderPath> gccHeaderPathes(const QString &gcc, const QStringList &
             } else if (line.startsWith("End of search list.")) {
                 break;
             } else {
-                qWarning() << "ignore line:" << line;
+                qWarning("%s: Ignoring line: %s", __FUNCTION__, line.constData());
             }
         }
     }
@@ -175,7 +201,7 @@ static QList<ProjectExplorer::Abi> guessGccAbi(const QString &m)
     if (machine.isEmpty())
         return abiList;
 
-    QStringList parts = machine.split(QRegExp("[ /-]"));
+    QStringList parts = machine.split(QRegExp(QLatin1String("[ /-]")));
 
     ProjectExplorer::Abi::Architecture arch = ProjectExplorer::Abi::UnknownArchitecture;
     ProjectExplorer::Abi::OS os = ProjectExplorer::Abi::UnknownOS;
@@ -199,7 +225,7 @@ static QList<ProjectExplorer::Abi> guessGccAbi(const QString &m)
         } else if (p == QLatin1String("mipsel")) {
             arch = ProjectExplorer::Abi::MipsArchitecture;
             width = 32;
-        } else if (p == QLatin1String("x86_64")) {
+        } else if (p == QLatin1String("x86_64") || p == QLatin1String("amd64")) {
             arch = ProjectExplorer::Abi::X86Architecture;
             width = 64;
         } else if (p == QLatin1String("powerpc")) {
@@ -211,7 +237,7 @@ static QList<ProjectExplorer::Abi> guessGccAbi(const QString &m)
             if (flavor == Abi::UnknownFlavor)
                 flavor = ProjectExplorer::Abi::GenericLinuxFlavor;
             format = ProjectExplorer::Abi::ElfFormat;
-        } else if (p.startsWith("freebsd")) {
+        } else if (p.startsWith(QLatin1String("freebsd"))) {
             os = ProjectExplorer::Abi::BsdOS;
             if (flavor == Abi::UnknownFlavor)
                 flavor = ProjectExplorer::Abi::FreeBsdFlavor;
@@ -225,7 +251,7 @@ static QList<ProjectExplorer::Abi> guessGccAbi(const QString &m)
             flavor = ProjectExplorer::Abi::SymbianDeviceFlavor;
             format = ProjectExplorer::Abi::ElfFormat;
             width = 32;
-        } else if (p == QLatin1String("mingw32") || p == QLatin1String("win32")) {
+        } else if (p == QLatin1String("mingw32") || p == QLatin1String("win32") || p == QLatin1String("mingw32msvc")) {
             arch = ProjectExplorer::Abi::X86Architecture;
             os = ProjectExplorer::Abi::WindowsOS;
             flavor = ProjectExplorer::Abi::WindowsMSysFlavor;
@@ -265,14 +291,17 @@ static QList<ProjectExplorer::Abi> guessGccAbi(const QString &m)
     return abiList;
 }
 
-static QList<ProjectExplorer::Abi> guessGccAbi(const QString &path, const QStringList &env)
+static QList<ProjectExplorer::Abi> guessGccAbi(const Utils::FileName &path, const QStringList &env)
 {
+    if (path.isEmpty())
+        return QList<ProjectExplorer::Abi>();
+
     QStringList arguments(QLatin1String("-dumpmachine"));
     QString machine = QString::fromLocal8Bit(runGcc(path, arguments, env)).trimmed();
     return guessGccAbi(machine);
 }
 
-static QString gccVersion(const QString &path, const QStringList &env)
+static QString gccVersion(const Utils::FileName &path, const QStringList &env)
 {
     QStringList arguments(QLatin1String("-dumpversion"));
     return QString::fromLocal8Bit(runGcc(path, arguments, env)).trimmed();
@@ -292,32 +321,39 @@ GccToolChain::GccToolChain(const QString &id, bool autodetect) :
 
 GccToolChain::GccToolChain(const GccToolChain &tc) :
     ToolChain(tc),
-    m_compilerPath(tc.compilerPath()),
+    m_predefinedMacros(tc.predefinedMacros(QStringList())),
+    m_compilerCommand(tc.compilerCommand()),
     m_debuggerCommand(tc.debuggerCommand()),
-    m_targetAbi(tc.m_targetAbi)
-{
-    setCompilerPath(tc.m_compilerPath);
-}
+    m_targetAbi(tc.m_targetAbi),
+    m_supportedAbis(tc.m_supportedAbis),
+    m_headerPathes(tc.m_headerPathes),
+    m_version(tc.m_version)
+{ }
 
 QString GccToolChain::defaultDisplayName() const
 {
     if (!m_targetAbi.isValid())
-        return typeName();
-    return QString::fromLatin1("%1 (%2 %3)").arg(typeName(),
+        return typeDisplayName();
+    return QString::fromLatin1("%1 (%2 %3)").arg(typeDisplayName(),
                                                  ProjectExplorer::Abi::toString(m_targetAbi.architecture()),
                                                  ProjectExplorer::Abi::toString(m_targetAbi.wordWidth()));
 }
 
-void GccToolChain::updateId()
+QString GccToolChain::legacyId() const
 {
     QString i = id();
     i = i.left(i.indexOf(QLatin1Char(':')));
-    setId(QString::fromLatin1("%1:%2.%3.%4")
-          .arg(i).arg(m_compilerPath)
-          .arg(m_targetAbi.toString()).arg(m_debuggerCommand));
+    return QString::fromLatin1("%1:%2.%3.%4")
+               .arg(i).arg(m_compilerCommand.toString())
+               .arg(m_targetAbi.toString()).arg(m_debuggerCommand.toString());
 }
 
-QString GccToolChain::typeName() const
+QString GccToolChain::type() const
+{
+    return QLatin1String("gcc");
+}
+
+QString GccToolChain::typeDisplayName() const
 {
     return Internal::GccToolChainFactory::tr("GCC");
 }
@@ -339,99 +375,103 @@ void GccToolChain::setTargetAbi(const Abi &abi)
     if (abi == m_targetAbi)
         return;
 
-    updateSupportedAbis();
     m_targetAbi = abi;
-    updateId();
     toolChainUpdated();
 }
 
 QList<Abi> GccToolChain::supportedAbis() const
 {
-    updateSupportedAbis();
     return m_supportedAbis;
 }
 
 bool GccToolChain::isValid() const
 {
-    return !m_compilerPath.isNull();
+    return !m_compilerCommand.isNull();
 }
 
-QByteArray GccToolChain::predefinedMacros() const
+QByteArray GccToolChain::predefinedMacros(const QStringList &cxxflags) const
 {
     if (m_predefinedMacros.isEmpty()) {
         // Using a clean environment breaks ccache/distcc/etc.
         Utils::Environment env = Utils::Environment::systemEnvironment();
         addToEnvironment(env);
-        m_predefinedMacros = gccPredefinedMacros(m_compilerPath, env.toStringList());
+        m_predefinedMacros = gccPredefinedMacros(m_compilerCommand, cxxflags, env.toStringList());
     }
     return m_predefinedMacros;
 }
 
+ProjectExplorer::ToolChain::CompilerFlags GccToolChain::compilerFlags(const QStringList &cxxflags) const
+{
+    if (cxxflags.contains("-std=c++0x") || cxxflags.contains("-std=gnu++0x"))
+        return STD_CXX11;
+    return NO_FLAGS;
+}
+
 QList<HeaderPath> GccToolChain::systemHeaderPaths() const
 {
     if (m_headerPathes.isEmpty()) {
         // Using a clean environment breaks ccache/distcc/etc.
         Utils::Environment env = Utils::Environment::systemEnvironment();
         addToEnvironment(env);
-        m_headerPathes = gccHeaderPathes(m_compilerPath, env.toStringList());
+        m_headerPathes = gccHeaderPathes(m_compilerCommand, env.toStringList());
     }
     return m_headerPathes;
 }
 
 void GccToolChain::addToEnvironment(Utils::Environment &env) const
 {
-    if (!m_compilerPath.isEmpty())
-        env.prependOrSetPath(QFileInfo(m_compilerPath).absolutePath());
+    if (!m_compilerCommand.isEmpty()) {
+        Utils::FileName path = m_compilerCommand.parentDir();
+        env.prependOrSetPath(path.toString());
+    }
 }
 
-void GccToolChain::setDebuggerCommand(const QString &d)
+void GccToolChain::setDebuggerCommand(const Utils::FileName &d)
 {
     if (m_debuggerCommand == d)
         return;
     m_debuggerCommand = d;
-    updateId();
     toolChainUpdated();
 }
 
-QString GccToolChain::debuggerCommand() const
+Utils::FileName GccToolChain::debuggerCommand() const
 {
     return m_debuggerCommand;
 }
 
-QString GccToolChain::mkspec() const
+QList<Utils::FileName> GccToolChain::suggestedMkspecList() const
 {
     Abi abi = targetAbi();
+    Abi host = Abi::hostAbi();
+
+    // Cross compile: Leave the mkspec alone!
+    if (abi.architecture() != host.architecture()
+            || abi.os() != host.os()
+            || abi.osFlavor() != host.osFlavor()) // Note: This can fail:-(
+        return QList<Utils::FileName>();
+
     if (abi.os() == Abi::MacOS) {
         QString v = version();
         // prefer versioned g++ on mac. This is required to enable building for older Mac OS versions
-        if (v.startsWith(QLatin1String("4.0")))
-            return QLatin1String("macx-g++40");
-        if (v.startsWith(QLatin1String("4.2")))
-            return QLatin1String("macx-g++42");
-        return QLatin1String("macx-g++");
+        if (v.startsWith(QLatin1String("4.0")) && m_compilerCommand.endsWith(QLatin1String("-4.0")))
+            return QList<Utils::FileName>() << Utils::FileName::fromString(QLatin1String("macx-g++40"));
+        if (v.startsWith(QLatin1String("4.2")) && m_compilerCommand.endsWith(QLatin1String("-4.2")))
+            return QList<Utils::FileName>() << Utils::FileName::fromString(QLatin1String("macx-g++42"));
+        return QList<Utils::FileName>() << Utils::FileName::fromString(QLatin1String("macx-g++"));
     }
 
-    QList<Abi> gccAbiList = Abi::abisOfBinary(m_compilerPath);
-    Abi gccAbi;
-    if (!gccAbiList.isEmpty())
-        gccAbi  = gccAbiList.first();
-    if (!gccAbi.isNull()
-            && (gccAbi.architecture() != abi.architecture()
-                || gccAbi.os() != abi.os()
-                || gccAbi.osFlavor() != abi.osFlavor())) {
-        // Note: This can fail:-(
-        return QString(); // this is a cross-compiler, leave the mkspec alone!
-    }
     if (abi.os() == Abi::LinuxOS) {
         if (abi.osFlavor() != Abi::GenericLinuxFlavor)
-            return QString(); // most likely not a desktop, so leave the mkspec alone.
-        if (abi.wordWidth() == gccAbi.wordWidth())
-            return QLatin1String("linux-g++"); // no need to explicitly set the word width
-        return QLatin1String("linux-g++-") + QString::number(m_targetAbi.wordWidth());
+            return QList<Utils::FileName>(); // most likely not a desktop, so leave the mkspec alone.
+        if (abi.wordWidth() == host.wordWidth())
+            return QList<Utils::FileName>() << Utils::FileName::fromString(QLatin1String("linux-g++")); // no need to explicitly set the word width
+        return QList<Utils::FileName>() << Utils::FileName::fromString(QLatin1String("linux-g++-") + QString::number(m_targetAbi.wordWidth()));
     }
+
     if (abi.os() == Abi::BsdOS && abi.osFlavor() == Abi::FreeBsdFlavor)
-        return QLatin1String("freebsd-g++");
-    return QString();
+        return QList<Utils::FileName>() << Utils::FileName::fromString(QLatin1String("freebsd-g++"));
+
+    return QList<Utils::FileName>();
 }
 
 QString GccToolChain::makeCommand() const
@@ -444,37 +484,35 @@ IOutputParser *GccToolChain::outputParser() const
     return new GccParser;
 }
 
-void GccToolChain::setCompilerPath(const QString &path)
+void GccToolChain::setCompilerCommand(const Utils::FileName &path)
 {
-    if (path == m_compilerPath)
+    if (path == m_compilerCommand)
         return;
 
     bool resetDisplayName = displayName() == defaultDisplayName();
 
-    m_compilerPath = path;
-    m_supportedAbis.clear();
+    m_compilerCommand = path;
 
     Abi currentAbi = m_targetAbi;
+    m_supportedAbis = detectSupportedAbis();
 
     m_targetAbi = Abi();
-    if (!m_compilerPath.isEmpty()) {
-        updateSupportedAbis();
-        if (!m_supportedAbis.isEmpty()) {
-            if (m_supportedAbis.contains(currentAbi))
-                m_targetAbi = currentAbi;
-            else
-                m_targetAbi = m_supportedAbis.at(0);
-        }
-
-        if (resetDisplayName)
-            setDisplayName(defaultDisplayName());
+    if (!m_supportedAbis.isEmpty()) {
+        if (m_supportedAbis.contains(currentAbi))
+            m_targetAbi = currentAbi;
+        else
+            m_targetAbi = m_supportedAbis.at(0);
     }
-    updateId(); // Will trigger toolChainUpdated()!
+
+    if (resetDisplayName)
+        setDisplayName(defaultDisplayName()); // calls toolChainUpdated()!
+    else
+        toolChainUpdated();
 }
 
-QString GccToolChain::compilerPath() const
+Utils::FileName GccToolChain::compilerCommand() const
 {
-    return m_compilerPath;
+    return m_compilerCommand;
 }
 
 ToolChain *GccToolChain::clone() const
@@ -485,13 +523,13 @@ ToolChain *GccToolChain::clone() const
 QVariantMap GccToolChain::toMap() const
 {
     QVariantMap data = ToolChain::toMap();
-    data.insert(QLatin1String(compilerPathKeyC), m_compilerPath);
+    data.insert(QLatin1String(compilerCommandKeyC), m_compilerCommand.toString());
     data.insert(QLatin1String(targetAbiKeyC), m_targetAbi.toString());
     QStringList abiList;
     foreach (const ProjectExplorer::Abi &a, m_supportedAbis)
         abiList.append(a.toString());
     data.insert(QLatin1String(supportedAbisKeyC), abiList);
-    data.insert(QLatin1String(debuggerCommandKeyC), m_debuggerCommand);
+    data.insert(QLatin1String(debuggerCommandKeyC), m_debuggerCommand.toString());
     return data;
 }
 
@@ -500,7 +538,7 @@ bool GccToolChain::fromMap(const QVariantMap &data)
     if (!ToolChain::fromMap(data))
         return false;
 
-    m_compilerPath = data.value(QLatin1String(compilerPathKeyC)).toString();
+    m_compilerCommand = Utils::FileName::fromString(data.value(QLatin1String(compilerCommandKeyC)).toString());
     m_targetAbi = Abi(data.value(QLatin1String(targetAbiKeyC)).toString());
     QStringList abiList = data.value(QLatin1String(supportedAbisKeyC)).toStringList();
     m_supportedAbis.clear();
@@ -510,8 +548,7 @@ bool GccToolChain::fromMap(const QVariantMap &data)
             continue;
         m_supportedAbis.append(abi);
     }
-    m_debuggerCommand = data.value(QLatin1String(debuggerCommandKeyC)).toString();
-    updateId();
+    m_debuggerCommand = Utils::FileName::fromString(data.value(QLatin1String(debuggerCommandKeyC)).toString());
     return true;
 }
 
@@ -521,7 +558,7 @@ bool GccToolChain::operator ==(const ToolChain &other) const
         return false;
 
     const GccToolChain *gccTc = static_cast<const GccToolChain *>(&other);
-    return m_compilerPath == gccTc->m_compilerPath && m_targetAbi == gccTc->m_targetAbi
+    return m_compilerCommand == gccTc->m_compilerCommand && m_targetAbi == gccTc->m_targetAbi
             && m_debuggerCommand == gccTc->m_debuggerCommand;
 }
 
@@ -540,14 +577,14 @@ QList<Abi> GccToolChain::detectSupportedAbis() const
 {
     Utils::Environment env = Utils::Environment::systemEnvironment();
     addToEnvironment(env);
-    return guessGccAbi(m_compilerPath, env.toStringList());
+    return guessGccAbi(m_compilerCommand, env.toStringList());
 }
 
 QString GccToolChain::detectVersion() const
 {
     Utils::Environment env = Utils::Environment::systemEnvironment();
     addToEnvironment(env);
-    return gccVersion(m_compilerPath, env.toStringList());
+    return gccVersion(m_compilerCommand, env.toStringList());
 }
 
 // --------------------------------------------------------------------------
@@ -618,7 +655,7 @@ QList<ToolChain *> Internal::GccToolChainFactory::autoDetectToolchains(const QSt
     QList<ToolChain *> result;
 
     const Utils::Environment systemEnvironment = Utils::Environment::systemEnvironment();
-    const QString compilerPath = systemEnvironment.searchInPath(compiler);
+    const Utils::FileName compilerPath = Utils::FileName::fromString(systemEnvironment.searchInPath(compiler));
     if (compilerPath.isEmpty())
         return result;
 
@@ -630,10 +667,10 @@ QList<ToolChain *> Internal::GccToolChainFactory::autoDetectToolchains(const QSt
             return result;
     }
 
-    QString debuggerPath = ToolChainManager::instance()->defaultDebugger(requiredAbi); // Find the first debugger
+    Utils::FileName debuggerPath = ToolChainManager::instance()->defaultDebugger(requiredAbi);
     if (debuggerPath.isEmpty()) {
         foreach (const QString &debugger, debuggers) {
-            debuggerPath = systemEnvironment.searchInPath(debugger);
+            debuggerPath = Utils::FileName::fromString(systemEnvironment.searchInPath(debugger));
             if (!debuggerPath.isEmpty())
                 break;
         }
@@ -644,7 +681,7 @@ QList<ToolChain *> Internal::GccToolChainFactory::autoDetectToolchains(const QSt
         if (tc.isNull())
             return result;
 
-        tc->setCompilerPath(compilerPath);
+        tc->setCompilerCommand(compilerPath);
         tc->setDebuggerCommand(debuggerPath);
         tc->setTargetAbi(abi);
         tc->setDisplayName(tc->defaultDisplayName()); // reset displayname
@@ -661,7 +698,7 @@ QList<ToolChain *> Internal::GccToolChainFactory::autoDetectToolchains(const QSt
 
 Internal::GccToolChainConfigWidget::GccToolChainConfigWidget(GccToolChain *tc) :
     ToolChainConfigWidget(tc),
-    m_compilerPath(new Utils::PathChooser),
+    m_compilerCommand(new Utils::PathChooser),
     m_abiWidget(new AbiWidget),
     m_isReadOnly(false)
 {
@@ -670,18 +707,19 @@ Internal::GccToolChainConfigWidget::GccToolChainConfigWidget(GccToolChain *tc) :
     QFormLayout *layout = new QFormLayout(this);
 
     const QStringList gnuVersionArgs = QStringList(QLatin1String("--version"));
-    m_compilerPath->setExpectedKind(Utils::PathChooser::ExistingCommand);
-    m_compilerPath->setCommandVersionArguments(gnuVersionArgs);
-    layout->addRow(tr("&Compiler path:"), m_compilerPath);
+    m_compilerCommand->setExpectedKind(Utils::PathChooser::ExistingCommand);
+    m_compilerCommand->setCommandVersionArguments(gnuVersionArgs);
+    layout->addRow(tr("&Compiler path:"), m_compilerCommand);
     layout->addRow(tr("&ABI:"), m_abiWidget);
     m_abiWidget->setEnabled(false);
 
     addDebuggerCommandControls(layout, gnuVersionArgs);
+    addMkspecControls(layout);
     addErrorLabel(layout);
 
     setFromToolchain();
 
-    connect(m_compilerPath, SIGNAL(changed(QString)), this, SLOT(handlePathChange()));
+    connect(m_compilerCommand, SIGNAL(changed(QString)), this, SLOT(handleCompilerCommandChange()));
     connect(m_abiWidget, SIGNAL(abiChanged()), this, SLOT(handleAbiChange()));
 }
 
@@ -693,14 +731,12 @@ void Internal::GccToolChainConfigWidget::apply()
     GccToolChain *tc = static_cast<GccToolChain *>(toolChain());
     Q_ASSERT(tc);
     QString displayName = tc->displayName();
-    QString path = m_compilerPath->path();
-    if (path.isEmpty())
-        path = m_compilerPath->rawPath();
-    tc->setCompilerPath(path);
+    tc->setCompilerCommand(m_compilerCommand->fileName());
     tc->setTargetAbi(m_abiWidget->currentAbi());
     tc->setDisplayName(displayName); // reset display name
     tc->setDebuggerCommand(debuggerCommand());
-    m_autoDebuggerCommand = QLatin1String("<manually set>");
+    tc->setMkspecList(mkspecList());
+    m_autoDebuggerCommand = Utils::FileName::fromString(QLatin1String("<manually set>"));
 }
 
 void Internal::GccToolChainConfigWidget::setFromToolchain()
@@ -708,11 +744,12 @@ void Internal::GccToolChainConfigWidget::setFromToolchain()
     // subwidgets are not yet connected!
     bool blocked = blockSignals(true);
     GccToolChain *tc = static_cast<GccToolChain *>(toolChain());
-    m_compilerPath->setPath(tc->compilerPath());
+    m_compilerCommand->setFileName(tc->compilerCommand());
     m_abiWidget->setAbis(tc->supportedAbis(), tc->targetAbi());
-    if (!m_isReadOnly && !m_compilerPath->path().isEmpty())
+    if (!m_isReadOnly && !m_compilerCommand->path().isEmpty())
         m_abiWidget->setEnabled(true);
     setDebuggerCommand(tc->debuggerCommand());
+    setMkspecList(tc->mkspecList());
     blockSignals(blocked);
 }
 
@@ -720,25 +757,27 @@ bool Internal::GccToolChainConfigWidget::isDirty() const
 {
     GccToolChain *tc = static_cast<GccToolChain *>(toolChain());
     Q_ASSERT(tc);
-    return m_compilerPath->path() != tc->compilerPath()
-            || m_abiWidget->currentAbi() != tc->targetAbi();
+    return m_compilerCommand->fileName() != tc->compilerCommand()
+            || m_abiWidget->currentAbi() != tc->targetAbi()
+            || debuggerCommand() != tc->debuggerCommand()
+            || mkspecList() != tc->mkspecList();
 }
 
 void Internal::GccToolChainConfigWidget::makeReadOnly()
 {
-    m_compilerPath->setEnabled(false);
+    m_compilerCommand->setEnabled(false);
     m_abiWidget->setEnabled(false);
     m_isReadOnly = true;
     ToolChainConfigWidget::makeReadOnly();
 }
 
-void Internal::GccToolChainConfigWidget::handlePathChange()
+void Internal::GccToolChainConfigWidget::handleCompilerCommandChange()
 {
-    QString path = m_compilerPath->path();
+    Utils::FileName path = m_compilerCommand->fileName();
     QList<Abi> abiList;
     bool haveCompiler = false;
     if (!path.isEmpty()) {
-        QFileInfo fi(path);
+        QFileInfo fi(path.toFileInfo());
         haveCompiler = fi.isExecutable() && fi.isFile();
     }
     if (haveCompiler)
@@ -756,7 +795,7 @@ void Internal::GccToolChainConfigWidget::handleAbiChange()
         m_autoDebuggerCommand = ToolChainManager::instance()->defaultDebugger(abi);
         setDebuggerCommand(m_autoDebuggerCommand);
     }
-    emit dirty(toolChain());
+    emit dirty();
 }
 
 // --------------------------------------------------------------------------
@@ -767,7 +806,12 @@ ClangToolChain::ClangToolChain(bool autodetect) :
     GccToolChain(QLatin1String(Constants::CLANG_TOOLCHAIN_ID), autodetect)
 { }
 
-QString ClangToolChain::typeName() const
+QString ClangToolChain::type() const
+{
+    return QLatin1String("clang");
+}
+
+QString ClangToolChain::typeDisplayName() const
 {
     return Internal::ClangToolChainFactory::tr("Clang");
 }
@@ -781,14 +825,18 @@ QString ClangToolChain::makeCommand() const
 #endif
 }
 
-QString ClangToolChain::mkspec() const
+QList<Utils::FileName> ClangToolChain::suggestedMkspecList() const
 {
     Abi abi = targetAbi();
     if (abi.os() == Abi::MacOS)
-        return QLatin1String("unsupported/macx-clang");
+        return QList<Utils::FileName>()
+                << Utils::FileName::fromString(QLatin1String("macx-clang"))
+                << Utils::FileName::fromString(QLatin1String("unsupported/macx-clang"));
     else if (abi.os() == Abi::LinuxOS)
-        return QLatin1String("unsupported/linux-clang");
-    return QString(); // Note: Not supported by Qt yet, so default to the mkspec the Qt was build with
+        return QList<Utils::FileName>()
+                << Utils::FileName::fromString(QLatin1String("linux-clang"))
+                << Utils::FileName::fromString(QLatin1String("unsupported/linux-clang"));
+    return QList<Utils::FileName>(); // Note: Not supported by Qt yet, so default to the mkspec the Qt was build with
 }
 
 IOutputParser *ClangToolChain::outputParser() const
@@ -859,19 +907,41 @@ MingwToolChain::MingwToolChain(bool autodetect) :
     GccToolChain(QLatin1String(Constants::MINGW_TOOLCHAIN_ID), autodetect)
 { }
 
-QString MingwToolChain::typeName() const
+QString MingwToolChain::type() const
+{
+    return QLatin1String("mingw");
+}
+
+QString MingwToolChain::typeDisplayName() const
 {
     return Internal::MingwToolChainFactory::tr("MinGW");
 }
 
-QString MingwToolChain::mkspec() const
+QList<Utils::FileName> MingwToolChain::suggestedMkspecList() const
 {
-    return QLatin1String("win32-g++");
+#if defined(Q_OS_WIN)
+    return QList<Utils::FileName>() << Utils::FileName::fromString(QLatin1String("win32-g++"));
+#elif defined(Q_OS_LINUX)
+    if (version().startsWith("4.6."))
+        return QList<Utils::FileName>()
+                << Utils::FileName::fromString(QLatin1String("win32-g++-4.6-cross"))
+                << Utils::FileName::fromString(QLatin1String("unsupported/win32-g++-4.6-cross"));
+    else
+        return QList<Utils::FileName>()
+                << Utils::FileName::fromString(QLatin1String("win32-g++-cross"))
+                << Utils::FileName::fromString(QLatin1String("unsupported/win32-g++-cross"));
+#else
+    return QList<Utils::FileName>();
+#endif
 }
 
 QString MingwToolChain::makeCommand() const
 {
+#ifdef Q_OS_WIN
     return QLatin1String("mingw32-make.exe");
+#else
+    return QLatin1String("make");
+#endif
 }
 
 ToolChain *MingwToolChain::clone() const
@@ -948,7 +1018,12 @@ LinuxIccToolChain::LinuxIccToolChain(bool autodetect) :
     GccToolChain(QLatin1String(Constants::LINUXICC_TOOLCHAIN_ID), autodetect)
 { }
 
-QString LinuxIccToolChain::typeName() const
+QString LinuxIccToolChain::type() const
+{
+    return QLatin1String("icc");
+}
+
+QString LinuxIccToolChain::typeDisplayName() const
 {
     return Internal::LinuxIccToolChainFactory::tr("Linux ICC");
 }
@@ -958,9 +1033,10 @@ IOutputParser *LinuxIccToolChain::outputParser() const
     return new LinuxIccParser;
 }
 
-QString LinuxIccToolChain::mkspec() const
+QList<Utils::FileName> LinuxIccToolChain::suggestedMkspecList() const
 {
-    return QLatin1String("linux-icc-") + QString::number(targetAbi().wordWidth());
+    return QList<Utils::FileName>()
+            << Utils::FileName::fromString(QLatin1String("linux-icc-") + QString::number(targetAbi().wordWidth()));
 }
 
 ToolChain *LinuxIccToolChain::clone() const
@@ -1022,7 +1098,7 @@ GccToolChain *Internal::LinuxIccToolChainFactory::createToolChain(bool autoDetec
 #   include "projectexplorer.h"
 
 #   include <QTest>
-#   include <QtCore/QUrl>
+#   include <QUrl>
 
 namespace ProjectExplorer {
 void ProjectExplorerPlugin::testGccAbiGuessing_data()
@@ -1084,6 +1160,13 @@ void ProjectExplorerPlugin::testGccAbiGuessing_data()
     QTest::newRow("Mingw 2")
             << QString::fromLatin1("mingw32")
             << (QStringList() << QLatin1String("x86-windows-msys-pe-32bit"));
+    QTest::newRow("Cross Mingw 1")
+            << QString::fromLatin1("amd64-mingw32msvc")
+            << (QStringList() << QLatin1String("x86-windows-msys-pe-64bit")
+                              << QLatin1String("x86-windows-msys-pe-32bit"));
+    QTest::newRow("Cross Mingw 2")
+            << QString::fromLatin1("i586-mingw32msvc")
+            << (QStringList() << QLatin1String("x86-windows-msys-pe-32bit"));
     QTest::newRow("Clang 1: windows")
             << QString::fromLatin1("x86_64-pc-win32")
             << (QStringList() << QLatin1String("x86-windows-msys-pe-64bit")
