@@ -1,8 +1,40 @@
+/**************************************************************************
+**
+** This file is part of Qt Creator
+**
+** Copyright (c) 2012 Nokia Corporation and/or its subsidiary(-ies).
+**
+** Contact: Nokia Corporation (qt-info@nokia.com)
+**
+**
+** GNU Lesser General Public License Usage
+**
+** This file may be used under the terms of the GNU Lesser General Public
+** License version 2.1 as published by the Free Software Foundation and
+** appearing in the file LICENSE.LGPL included in the packaging of this file.
+** Please review the following information to ensure the GNU Lesser General
+** Public License version 2.1 requirements will be met:
+** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+**
+** In addition, as a special exception, Nokia gives you certain additional
+** rights. These rights are described in the Nokia Qt LGPL Exception
+** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+**
+** Other Usage
+**
+** Alternatively, this file may be used in accordance with the terms and
+** conditions contained in a signed written agreement between you and Nokia.
+**
+** If you have questions regarding the use of this file, please contact
+** Nokia at qt-info@nokia.com.
+**
+**************************************************************************/
+
 #ifndef TARGETSELECTOR_H
 #define TARGETSELECTOR_H
 
-#include <QtGui/QWidget>
-#include <QtGui/QPixmap>
+#include <QWidget>
+#include <QPixmap>
 
 QT_BEGIN_NAMESPACE
 class QMenu;
@@ -36,8 +68,7 @@ public:
     bool isAddButtonEnabled() const;
     bool isRemoveButtonEnabled() const;
 
-public slots:
-    void addTarget(const QString &name);
+public:
     void insertTarget(int index, const QString &name);
     void removeTarget(int index);
     void setCurrentIndex(int index);
@@ -48,7 +79,7 @@ public slots:
 
 signals:
     void removeButtonClicked();
-    // This signal is emited whenever the target pointed to by the indices
+    // This signal is emitted whenever the target pointed to by the indices
     // has changed.
     void currentChanged(int targetIndex, int subIndex);
 
