@@ -2,7 +2,7 @@
 **
 ** This file is part of Qt Creator
 **
-** Copyright (c) 2011 Nokia Corporation and/or its subsidiary(-ies).
+** Copyright (c) 2012 Nokia Corporation and/or its subsidiary(-ies).
 **
 ** Contact: Nokia Corporation (qt-info@nokia.com)
 **
@@ -39,7 +39,7 @@
 
 #include <texteditor/basetexteditor.h>
 
-#include <QtCore/QStringList>
+#include <QStringList>
 
 namespace QmlJS {
 class Snapshot;
