@@ -2,28 +2,31 @@
 **
 ** This file is part of Qt Creator
 **
-** Copyright (c) 2010 Nokia Corporation and/or its subsidiary(-ies).
+** Copyright (c) 2012 Nokia Corporation and/or its subsidiary(-ies).
 **
 ** Contact: Nokia Corporation (qt-info@nokia.com)
 **
-** Commercial Usage
-**
-** Licensees holding valid Qt Commercial licenses may use this file in
-** accordance with the Qt Commercial License Agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Nokia.
 **
 ** GNU Lesser General Public License Usage
 **
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU Lesser General Public License version 2.1 requirements
-** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** This file may be used under the terms of the GNU Lesser General Public
+** License version 2.1 as published by the Free Software Foundation and
+** appearing in the file LICENSE.LGPL included in the packaging of this file.
+** Please review the following information to ensure the GNU Lesser General
+** Public License version 2.1 requirements will be met:
+** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** If you are unsure which license is appropriate for your use, please
-** contact the sales department at http://qt.nokia.com/contact.
+** In addition, as a special exception, Nokia gives you certain additional
+** rights. These rights are described in the Nokia Qt LGPL Exception
+** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+**
+** Other Usage
+**
+** Alternatively, this file may be used in accordance with the terms and
+** conditions contained in a signed written agreement between you and Nokia.
+**
+** If you have questions regarding the use of this file, please contact
+** Nokia at qt-info@nokia.com.
 **
 **************************************************************************/
 
@@ -32,6 +35,7 @@
 #include <abstractview.h>
 #include <model.h>
 #include <metainfo.h>
+#include <nodemetainfo.h>
 #include "internalnode_p.h"
 #include <QHash>
 #include <QTextStream>
@@ -48,6 +52,7 @@
 #include "nodeabstractproperty.h"
 #include "nodelistproperty.h"
 #include "nodeproperty.h"
+#include <rewriterview.h>
 
 namespace QmlDesigner {
 using namespace QmlDesigner::Internal;
@@ -160,15 +165,23 @@ QString ModelNode::validId()
     return id();
 }
 
+static bool idIsQmlKeyWord(const QString& id)
+{
+    QStringList keywords;
+    keywords << "import" << "as";
+
+    return keywords.contains(id);
+}
+
 static bool idContainsWrongLetter(const QString& id)
 {
-    static QRegExp idExpr(QLatin1String("[a-z][a-zA-Z0-9_]*"));
+    static QRegExp idExpr(QLatin1String("[a-z_][a-zA-Z0-9_]*"));
     return !idExpr.exactMatch(id);
 }
 
 bool ModelNode::isValidId(const QString &id)
 {
-    return id.isEmpty() || !idContainsWrongLetter(id);
+    return id.isEmpty() || (!idContainsWrongLetter(id) && !idIsQmlKeyWord(id));
 }
 
 void ModelNode::setId(const QString& id)
@@ -188,7 +201,7 @@ void ModelNode::setId(const QString& id)
     if (view()->hasId(id))
         throw InvalidIdException(__LINE__, __FUNCTION__, __FILE__, id, InvalidIdException::DuplicateId);
 
-    m_model.data()->m_d->changeNodeId(internalNode(), id);
+    m_model.data()->d->changeNodeId(internalNode(), id);
 }
 
 /*! \brief the fully-qualified type name of the node is represented as string
@@ -227,6 +240,29 @@ int ModelNode::majorVersion() const
     return m_internalNode->majorVersion();
 }
 
+/*! \brief major number of the QtQuick version used
+\return major number of QtQuickVersion
+*/
+int ModelNode::majorQtQuickVersion() const
+{
+    if (!isValid()) {
+        Q_ASSERT_X(isValid(), Q_FUNC_INFO, "model node is invalid");
+        throw InvalidModelNodeException(__LINE__, __FUNCTION__, __FILE__);
+    }
+
+    if (metaInfo().isValid()) {
+        NodeMetaInfo superClass = metaInfo().directSuperClass();
+
+        while (superClass.isValid()) {
+            if (superClass.typeName() == "QtQuick.Item")
+                return superClass.majorVersion();
+            superClass = superClass.directSuperClass();
+        }
+        return 1; //default
+    }
+    return 1; //default
+}
+
 
 /*! \return the short-hand type name of the node. */
 QString ModelNode::simplifiedTypeName() const
@@ -236,7 +272,7 @@ QString ModelNode::simplifiedTypeName() const
         throw InvalidModelNodeException(__LINE__, __FUNCTION__, __FILE__);
     }
 
-    return type().split(QLatin1Char('/')).last();
+    return type().split(QLatin1Char('.')).last();
 }
 
 /*! \brief Returns whether the node is valid
@@ -250,7 +286,7 @@ A node might become invalid if e.g. it or one of its ancestors is deleted.
 */
 bool ModelNode::isValid() const
 {
-    return !m_model.isNull() && !m_view.isNull() && m_internalNode &&m_internalNode->isValid() /*&& model()->metaInfo().hasNodeMetaInfo(m_internalNode->type(), m_internalNode->majorVersion(), m_internalNode->minorVersion())*/;
+    return !m_model.isNull() && !m_view.isNull() && m_internalNode && m_internalNode->isValid();
 }
 
 /*!
@@ -553,10 +589,10 @@ void ModelNode::removeProperty(const QString &name)
     if (!isValid())
         throw InvalidModelNodeException(__LINE__, __FUNCTION__, __FILE__);
 
-    model()->m_d->checkPropertyName(name);
+    model()->d->checkPropertyName(name);
 
     if (internalNode()->hasProperty(name))
-        model()->m_d->removeProperty(internalNode()->property(name));
+        model()->d->removeProperty(internalNode()->property(name));
 }
 
 
@@ -601,7 +637,7 @@ void ModelNode::destroy()
     }
 
     removeModelNodeFromSelection(*this);
-    model()->m_d->removeNode(internalNode());
+    model()->d->removeNode(internalNode());
 }
 //\}
 
@@ -620,7 +656,7 @@ bool operator ==(const ModelNode &firstNode, const ModelNode &secondNode)
         throw InvalidModelNodeException(__LINE__, __FUNCTION__, __FILE__);
     }
 
-    return firstNode.m_internalNode.data() == secondNode.m_internalNode.data();
+    return firstNode.internalId() == secondNode.internalId();
 }
 
 /*!
@@ -633,7 +669,7 @@ bool operator !=(const ModelNode &firstNode, const ModelNode &secondNode)
         throw InvalidModelNodeException(__LINE__, __FUNCTION__, __FILE__);
     }
 
-    return firstNode.m_internalNode != secondNode.m_internalNode;
+    return firstNode.internalId() != secondNode.internalId();
 }
 
 bool operator <(const ModelNode &firstNode, const ModelNode &secondNode)
@@ -643,7 +679,7 @@ bool operator <(const ModelNode &firstNode, const ModelNode &secondNode)
         throw InvalidModelNodeException(__LINE__, __FUNCTION__, __FILE__);
     }
 
-    return firstNode.internalNode().data() < secondNode.internalNode().data();
+    return firstNode.internalId() < secondNode.internalId();
 }
 
 
@@ -659,11 +695,7 @@ Internal::InternalNodePointer ModelNode::internalNode() const
 
 uint qHash(const ModelNode &node)
 {
-//    if (!node.isValid()) {
-//        Q_ASSERT_X(node.isValid(), Q_FUNC_INFO, "model node is invalid");
-//        throw InvalidModelNodeException(__LINE__, __FUNCTION__, __FILE__);
-//    }
-    return ::qHash(node.m_internalNode.data());
+    return ::qHash(node.internalId());
 }
 
 /*!
@@ -721,16 +753,14 @@ bool ModelNode::hasAnySubModelNodes() const
     return !nodeAbstractProperties().isEmpty();
 }
 
-/*! \brief returns the meta info of the node
-\return meta info of the node
-*/
 const NodeMetaInfo ModelNode::metaInfo() const
 {
     if (!isValid()) {
         Q_ASSERT_X(isValid(), Q_FUNC_INFO, "model node is invalid");
         throw InvalidModelNodeException(__LINE__, __FUNCTION__, __FILE__);
     }
-    return model()->metaInfo().nodeMetaInfo(type(), majorVersion(), minorVersion());
+
+    return NodeMetaInfo(model()->metaInfoProxyModel(), type(), majorVersion(), minorVersion());
 }
 
 /*! \brief has a node the selection of the model
@@ -827,7 +857,7 @@ QDebug operator<<(QDebug debug, const ModelNode &modelNode)
 {
     if (modelNode.isValid()) {
         debug.nospace() << "ModelNode("
-                << modelNode.internalNode().data() << ", "
+                << modelNode.internalId() << ", "
                 << modelNode.type() << ", "
                 << modelNode.id() << ')';
     } else {
@@ -893,10 +923,10 @@ QVariant ModelNode::auxiliaryData(const QString &name) const
     return internalNode()->auxiliaryData(name);
 }
 
-void ModelNode::setAuxiliaryData(const QString &name, const QVariant &data)
+void ModelNode::setAuxiliaryData(const QString &name, const QVariant &data) const
 {
     Internal::WriteLocker locker(m_model.data());
-    m_model.data()->m_d->setAuxiliaryData(internalNode(), name, data);
+    m_model.data()->d->setAuxiliaryData(internalNode(), name, data);
 }
 
 bool ModelNode::hasAuxiliaryData(const QString &name) const
@@ -908,9 +938,18 @@ bool ModelNode::hasAuxiliaryData(const QString &name) const
     return internalNode()->hasAuxiliaryData(name);
 }
 
+QHash<QString, QVariant> ModelNode::auxiliaryData() const
+{
+    if (!isValid()) {
+        throw InvalidModelNodeException(__LINE__, __FUNCTION__, __FILE__);
+    }
+
+    return internalNode()->auxiliaryData();
+}
+
 void  ModelNode::setScriptFunctions(const QStringList &scriptFunctionList)
 {
-    model()->m_d->setScriptFunctions(internalNode(), scriptFunctionList);
+    model()->d->setScriptFunctions(internalNode(), scriptFunctionList);
 }
 
 QStringList  ModelNode::scriptFunctions() const
@@ -918,4 +957,58 @@ QStringList  ModelNode::scriptFunctions() const
     return internalNode()->scriptFunctions();
 }
 
+qint32 ModelNode::internalId() const
+{
+    if (m_internalNode.isNull())
+        return -1;
+
+    return m_internalNode->internalId();
+}
+
+void ModelNode::setNodeSource(const QString &newNodeSource)
+{
+    Internal::WriteLocker locker(m_model.data());
+
+    if (!isValid()) {
+        Q_ASSERT_X(isValid(), Q_FUNC_INFO, "model node is invalid");
+        throw InvalidModelNodeException(__LINE__, __FUNCTION__, __FILE__);
+    }
+
+    if (internalNode()->nodeSource() == newNodeSource)
+        return;
+
+    m_model.data()->d->setNodeSource(internalNode(), newNodeSource);
+}
+
+QString ModelNode::nodeSource() const
+{
+    if (!isValid()) {
+        throw InvalidModelNodeException(__LINE__, __FUNCTION__, __FILE__);
+    }
+
+    return internalNode()->nodeSource();
+}
+
+QString ModelNode::convertTypeToImportAlias() const
+{
+    if (!isValid()) {
+        throw InvalidModelNodeException(__LINE__, __FUNCTION__, __FILE__);
+    }
+
+    if (model()->rewriterView())
+        return model()->rewriterView()->convertTypeToImportAlias(type());
+
+    return type();
+}
+
+ModelNode::NodeSourceType ModelNode::nodeSourceType() const
+{
+    if (!isValid()) {
+        throw InvalidModelNodeException(__LINE__, __FUNCTION__, __FILE__);
+    }
+
+    return static_cast<ModelNode::NodeSourceType>(internalNode()->nodeSourceType());
+
+}
+
 }
