@@ -2,7 +2,7 @@
 **
 ** This file is part of Qt Creator
 **
-** Copyright (c) 2011 Nokia Corporation and/or its subsidiary(-ies).
+** Copyright (c) 2012 Nokia Corporation and/or its subsidiary(-ies).
 **
 ** Contact: Nokia Corporation (qt-info@nokia.com)
 **
@@ -53,17 +53,17 @@
 #include <projectexplorer/session.h>
 #include <qtsupport/baseqtversion.h>
 
-#include <QtCore/QDir>
-#include <QtCore/QFile>
-#include <QtCore/QFileInfo>
-#include <QtCore/QLibraryInfo>
-#include <QtCore/QtConcurrentRun>
-#include <qtconcurrent/runextensions.h>
-#include <QtCore/QTextStream>
-#include <QtCore/QCoreApplication>
-#include <QtCore/QTimer>
+#include <QDir>
+#include <QFile>
+#include <QFileInfo>
+#include <QLibraryInfo>
+#include <QtConcurrentRun>
+#include <utils/runextensions.h>
+#include <QTextStream>
+#include <QCoreApplication>
+#include <QTimer>
 
-#include <QtCore/QDebug>
+#include <QDebug>
 
 using namespace QmlJS;
 using namespace QmlJSTools;
@@ -75,13 +75,16 @@ QmlJS::Document::Language QmlJSTools::languageOfFile(const QString &fileName)
 {
     QStringList jsSuffixes("js");
     QStringList qmlSuffixes("qml");
+    QStringList jsonSuffixes("json");
 
     if (Core::ICore::instance()) {
-        Core::MimeDatabase *db = Core::ICore::instance()->mimeDatabase();
+        Core::MimeDatabase *db = Core::ICore::mimeDatabase();
         Core::MimeType jsSourceTy = db->findByType(Constants::JS_MIMETYPE);
         jsSuffixes = jsSourceTy.suffixes();
         Core::MimeType qmlSourceTy = db->findByType(Constants::QML_MIMETYPE);
         qmlSuffixes = qmlSourceTy.suffixes();
+        Core::MimeType jsonSourceTy = db->findByType(Constants::JSON_MIMETYPE);
+        jsonSuffixes = jsonSourceTy.suffixes();
     }
 
     const QFileInfo info(fileName);
@@ -90,6 +93,8 @@ QmlJS::Document::Language QmlJSTools::languageOfFile(const QString &fileName)
         return QmlJS::Document::JavaScriptLanguage;
     if (qmlSuffixes.contains(fileSuffix))
         return QmlJS::Document::QmlLanguage;
+    if (jsonSuffixes.contains(fileSuffix))
+        return QmlJS::Document::JsonLanguage;
     return QmlJS::Document::UnknownLanguage;
 }
 
@@ -97,7 +102,7 @@ QStringList QmlJSTools::qmlAndJsGlobPatterns()
 {
     QStringList pattern;
     if (Core::ICore::instance()) {
-        Core::MimeDatabase *db = Core::ICore::instance()->mimeDatabase();
+        Core::MimeDatabase *db = Core::ICore::mimeDatabase();
         Core::MimeType jsSourceTy = db->findByType(Constants::JS_MIMETYPE);
         Core::MimeType qmlSourceTy = db->findByType(Constants::QML_MIMETYPE);
 
@@ -114,7 +119,6 @@ QStringList QmlJSTools::qmlAndJsGlobPatterns()
 
 ModelManager::ModelManager(QObject *parent):
         ModelManagerInterface(parent),
-        m_core(Core::ICore::instance()),
         m_pluginDumper(new PluginDumper(this))
 {
     m_synchronizer.setCancelOnWait(true);
@@ -151,17 +155,15 @@ void ModelManager::delayedInitialization()
     }
 
     ProjectExplorer::SessionManager *sessionManager = ProjectExplorer::ProjectExplorerPlugin::instance()->session();
-    if (sessionManager) {
-        connect(sessionManager, SIGNAL(projectRemoved(ProjectExplorer::Project*)),
-                this, SLOT(removeProjectInfo(ProjectExplorer::Project*)));
-    }
+    connect(sessionManager, SIGNAL(projectRemoved(ProjectExplorer::Project*)),
+            this, SLOT(removeProjectInfo(ProjectExplorer::Project*)));
 }
 
 void ModelManager::loadQmlTypeDescriptions()
 {
     if (Core::ICore::instance()) {
-        loadQmlTypeDescriptions(Core::ICore::instance()->resourcePath());
-        loadQmlTypeDescriptions(Core::ICore::instance()->userResourcePath());
+        loadQmlTypeDescriptions(Core::ICore::resourcePath());
+        loadQmlTypeDescriptions(Core::ICore::userResourcePath());
     }
 }
 
@@ -203,13 +205,11 @@ void ModelManager::loadQmlTypeDescriptions(const QString &resourcePath)
 ModelManagerInterface::WorkingCopy ModelManager::workingCopy() const
 {
     WorkingCopy workingCopy;
-    if (!m_core)
+    if (!Core::ICore::instance())
         return workingCopy;
 
-    Core::EditorManager *editorManager = m_core->editorManager();
-
-    foreach (Core::IEditor *editor, editorManager->openedEditors()) {
-        const QString key = editor->file()->fileName();
+    foreach (Core::IEditor *editor, Core::ICore::editorManager()->openedEditors()) {
+        const QString key = editor->document()->fileName();
 
         if (TextEditor::ITextEditor *textEditor = qobject_cast<TextEditor::ITextEditor*>(editor)) {
             if (textEditor->context().contains(ProjectExplorer::Constants::LANG_QMLJS)) {
@@ -267,7 +267,7 @@ QFuture<void> ModelManager::refreshSourceFiles(const QStringList &sourceFiles,
     m_synchronizer.addFuture(result);
 
     if (sourceFiles.count() > 1) {
-        m_core->progressManager()->addTask(result, tr("Indexing"),
+        Core::ICore::progressManager()->addTask(result, tr("Indexing"),
                         Constants::TASK_INDEX);
     }
 
@@ -594,7 +594,7 @@ void ModelManager::parse(QFutureInterface<void> &future,
             }
         }
 
-        Document::Ptr doc = Document::create(fileName, language);
+        Document::MutablePtr doc = Document::create(fileName, language);
         doc->setEditorRevision(documentRevision);
         doc->setSource(contents);
         doc->parse();
@@ -626,7 +626,7 @@ void ModelManager::parse(QFutureInterface<void> &future,
 // Check whether fileMimeType is the same or extends knownMimeType
 bool ModelManager::matchesMimeType(const Core::MimeType &fileMimeType, const Core::MimeType &knownMimeType)
 {
-    Core::MimeDatabase *db = Core::ICore::instance()->mimeDatabase();
+    Core::MimeDatabase *db = Core::ICore::mimeDatabase();
 
     const QStringList knownTypeNames = QStringList(knownMimeType.type()) + knownMimeType.aliases();
 
@@ -645,6 +645,7 @@ bool ModelManager::matchesMimeType(const Core::MimeType &fileMimeType, const Cor
 
 QStringList ModelManager::importPaths() const
 {
+    QMutexLocker l(&m_mutex);
     return m_allImportPaths;
 }
 
@@ -670,18 +671,24 @@ static QStringList environmentImportPaths()
 
 void ModelManager::updateImportPaths()
 {
-    m_allImportPaths.clear();
+    QStringList allImportPaths;
     QMapIterator<ProjectExplorer::Project *, ProjectInfo> it(m_projects);
     while (it.hasNext()) {
         it.next();
         foreach (const QString &path, it.value().importPaths) {
             const QString canonicalPath = QFileInfo(path).canonicalFilePath();
             if (!canonicalPath.isEmpty())
-                m_allImportPaths += canonicalPath;
+                allImportPaths += canonicalPath;
         }
     }
-    m_allImportPaths += m_defaultImportPaths;
-    m_allImportPaths.removeDuplicates();
+    allImportPaths += m_defaultImportPaths;
+    allImportPaths.removeDuplicates();
+
+    {
+        QMutexLocker l(&m_mutex);
+        m_allImportPaths = allImportPaths;
+    }
+
 
     // check if any file in the snapshot imports something new in the new paths
     Snapshot snapshot = _validSnapshot;
@@ -705,7 +712,7 @@ void ModelManager::maybeQueueCppQmlTypeUpdate(const CPlusPlus::Document::Ptr &do
 {
     // avoid scanning documents without source code available
     doc->keepSourceAndAST();
-    if (doc->source().isEmpty()) {
+    if (doc->utf8Source().isEmpty()) {
         doc->releaseSourceAndAST();
         return;
     }
@@ -798,8 +805,6 @@ ModelManager::CppDataHash ModelManager::cppData() const
 LibraryInfo ModelManager::builtins(const Document::Ptr &doc) const
 {
     ProjectExplorer::SessionManager *sessionManager = ProjectExplorer::ProjectExplorerPlugin::instance()->session();
-    if (!sessionManager)
-        return LibraryInfo();
     ProjectExplorer::Project *project = sessionManager->projectForFile(doc->fileName());
     if (!project)
         return LibraryInfo();
