@@ -2,7 +2,7 @@
 **
 ** This file is part of Qt Creator
 **
-** Copyright (c) 2011 Nokia Corporation and/or its subsidiary(-ies).
+** Copyright (c) 2012 Nokia Corporation and/or its subsidiary(-ies).
 **
 ** Contact: Nokia Corporation (qt-info@nokia.com)
 **
@@ -39,7 +39,7 @@ namespace QmlProjectManager {
 namespace Internal {
 
 QmlProjectFile::QmlProjectFile(QmlProject *parent, QString fileName)
-    : Core::IFile(parent),
+    : Core::IDocument(parent),
       m_project(parent),
       m_fileName(fileName)
 {
@@ -88,17 +88,12 @@ bool QmlProjectFile::isModified() const
     return false;
 }
 
-bool QmlProjectFile::isReadOnly() const
-{
-    return true;
-}
-
 bool QmlProjectFile::isSaveAsAllowed() const
 {
     return false;
 }
 
-Core::IFile::ReloadBehavior QmlProjectFile::reloadBehavior(ChangeTrigger state, ChangeType type) const
+Core::IDocument::ReloadBehavior QmlProjectFile::reloadBehavior(ChangeTrigger state, ChangeType type) const
 {
     Q_UNUSED(state)
     Q_UNUSED(type)
