@@ -2,7 +2,7 @@
 **
 ** This file is part of Qt Creator
 **
-** Copyright (c) 2011 Nokia Corporation and/or its subsidiary(-ies).
+** Copyright (c) 2012 Nokia Corporation and/or its subsidiary(-ies).
 **
 ** Contact: Nokia Corporation (qt-info@nokia.com)
 **
@@ -37,11 +37,11 @@
 
 #include <coreplugin/coreconstants.h>
 
-#include <QtCore/QFileInfo>
-#include <QtCore/QTimer>
+#include <QFileInfo>
+#include <QTimer>
 
-#include <QtGui/QStackedLayout>
-#include <QtGui/QIcon>
+#include <QStackedLayout>
+#include <QIcon>
 
 namespace Qt4ProjectManager {
 namespace Internal {
