@@ -2,7 +2,7 @@
 **
 ** This file is part of Qt Creator
 **
-** Copyright (c) 2011 Nokia Corporation and/or its subsidiary(-ies).
+** Copyright (c) 2012 Nokia Corporation and/or its subsidiary(-ies).
 **
 ** Contact: Nokia Corporation (qt-info@nokia.com)
 **
@@ -38,29 +38,36 @@
 
 #include <projectexplorer/projectexplorerconstants.h>
 
-#include <QtGui/QIcon>
+#include <qtsupport/qtsupportconstants.h>
+
+#include <QIcon>
 
 namespace Qt4ProjectManager {
 namespace Internal {
 
 CustomWidgetWizard::CustomWidgetWizard() :
     QtWizard(QLatin1String("P.Qt4CustomWidget"),
-             QLatin1String(ProjectExplorer::Constants::PROJECT_WIZARD_CATEGORY),
-             QLatin1String(ProjectExplorer::Constants::PROJECT_WIZARD_TR_SCOPE),
-             QLatin1String(ProjectExplorer::Constants::PROJECT_WIZARD_TR_CATEGORY),
+             QLatin1String(ProjectExplorer::Constants::QT_PROJECT_WIZARD_CATEGORY),
+             QLatin1String(ProjectExplorer::Constants::QT_PROJECT_WIZARD_CATEGORY_DISPLAY),
              tr("Qt Custom Designer Widget"),
              tr("Creates a Qt Custom Designer Widget or a Custom Widget Collection."),
              QIcon(QLatin1String(":/wizards/images/gui.png")))
 {
 }
 
-QWizard *CustomWidgetWizard::createWizardDialog(QWidget *parent,
-                                                const QString &defaultPath,
-                                                const WizardPageList &extensionPages) const
+Core::FeatureSet CustomWidgetWizard::requiredFeatures() const
 {
-    CustomWidgetWizardDialog *rc = new CustomWidgetWizardDialog(displayName(), icon(), extensionPages, parent);
-    rc->setPath(defaultPath);
-    rc->setProjectName(CustomWidgetWizardDialog::uniqueProjectName(defaultPath));
+    return Core::Feature(QtSupport::Constants::FEATURE_QWIDGETS);
+}
+
+QWizard *CustomWidgetWizard::createWizardDialog(QWidget *parent,
+                                                const Core::WizardDialogParameters &wizardDialogParameters) const
+{
+    CustomWidgetWizardDialog *rc = new CustomWidgetWizardDialog(displayName(),
+                                                                icon(),
+                                                                parent,
+                                                                wizardDialogParameters);
+    rc->setProjectName(CustomWidgetWizardDialog::uniqueProjectName(wizardDialogParameters.defaultPath()));
     rc->setFileNamingParameters(FileNamingParameters(headerSuffix(), sourceSuffix(), QtWizard::lowerCaseFiles()));
     return rc;
 }
