@@ -2,7 +2,7 @@
 **
 ** This file is part of Qt Creator
 **
-** Copyright (c) 2011 Nokia Corporation and/or its subsidiary(-ies).
+** Copyright (c) 2012 Nokia Corporation and/or its subsidiary(-ies).
 **
 ** Contact: Nokia Corporation (qt-info@nokia.com)
 **
@@ -42,9 +42,9 @@
 #include <utils/environment.h>
 #include <qtsupport/qtversionmanager.h>
 
-#include <QtCore/QDir>
-#include <QtCore/QFileInfo>
-#include <QtCore/QStringList>
+#include <QDir>
+#include <QFileInfo>
+#include <QStringList>
 
 namespace Qt4ProjectManager {
 namespace Internal {
@@ -70,17 +70,17 @@ static const char *const WINSCW_DEFAULT_SYSTEM_LIBRARIES[] = {
     0
 };
 
-static QString winscwRoot(const QString &path)
+static Utils::FileName winscwRoot(const Utils::FileName &pathIn)
 {
+    Utils::FileName path = pathIn;
     if (path.isEmpty())
-        return QString();
+        return Utils::FileName();
 
-    QDir dir(path);
-    dir.cdUp();
-    dir.cdUp();
-    dir.cdUp();
-    dir.cd("Symbian_Support");
-    return dir.absolutePath();
+    path = path.parentDir();
+    path = path.parentDir();
+    path = path.parentDir();
+    path.appendPath(QLatin1String("Symbian_Support"));
+    return path;
 }
 
 static QString toNativePath(const QStringList &list)
@@ -90,28 +90,27 @@ static QString toNativePath(const QStringList &list)
 
 static QStringList fromNativePath(const QString &list)
 {
-    QString tmp = QDir::fromNativeSeparators(list);
-    return tmp.split(';');
+    return QDir::fromNativeSeparators(list).split(QLatin1Char(';'));
 }
 
-static QStringList detectIncludesFor(const QString path)
+static QStringList detectIncludesFor(const Utils::FileName &path)
 {
-    QString root = winscwRoot(path);
+    Utils::FileName root = winscwRoot(path);
     QStringList result;
     for (int i = 0; WINSCW_DEFAULT_SYSTEM_INCLUDES[i] != 0; ++i) {
-        QDir dir(root + QLatin1String(WINSCW_DEFAULT_SYSTEM_INCLUDES[i]));
+        QDir dir(root.toString() + QLatin1String(WINSCW_DEFAULT_SYSTEM_INCLUDES[i]));
         if (dir.exists())
             result.append(dir.absolutePath());
     }
     return result;
 }
 
-static QStringList detectLibrariesFor(const QString path)
+static QStringList detectLibrariesFor(const Utils::FileName &path)
 {
-    QString root = winscwRoot(path);
+    Utils::FileName root = winscwRoot(path);
     QStringList result;
     for (int i = 0; WINSCW_DEFAULT_SYSTEM_LIBRARIES[i] != 0; ++i) {
-        QDir dir(root + QLatin1String(WINSCW_DEFAULT_SYSTEM_LIBRARIES[i]));
+        QDir dir(root.toString() + QLatin1String(WINSCW_DEFAULT_SYSTEM_LIBRARIES[i]));
         if (dir.exists())
             result.append(dir.absolutePath());
     }
@@ -136,7 +135,12 @@ WinscwToolChain::WinscwToolChain(const WinscwToolChain &tc) :
 WinscwToolChain::~WinscwToolChain()
 { }
 
-QString WinscwToolChain::typeName() const
+QString WinscwToolChain::type() const
+{
+    return QLatin1String("winscw");
+}
+
+QString WinscwToolChain::typeDisplayName() const
 {
     return WinscwToolChainFactory::tr("WINSCW");
 }
@@ -148,20 +152,32 @@ ProjectExplorer::Abi WinscwToolChain::targetAbi() const
                                 ProjectExplorer::Abi::ElfFormat, 32);
 }
 
+QString WinscwToolChain::legacyId() const
+{
+    return QLatin1String(Constants::WINSCW_TOOLCHAIN_ID) + QLatin1Char(':')  + m_compilerPath.toString();
+}
+
 bool WinscwToolChain::isValid() const
 {
     if (m_compilerPath.isEmpty())
         return false;
 
-    QFileInfo fi(m_compilerPath);
+    QFileInfo fi(m_compilerPath.toFileInfo());
     return fi.exists() && fi.isExecutable();
 }
 
-QByteArray WinscwToolChain::predefinedMacros() const
+QByteArray WinscwToolChain::predefinedMacros(const QStringList &cxxflags) const
 {
+    Q_UNUSED(cxxflags);
     return QByteArray("#define __SYMBIAN32__\n");
 }
 
+ProjectExplorer::ToolChain::CompilerFlags WinscwToolChain::compilerFlags(const QStringList &cxxflags) const
+{
+    Q_UNUSED(cxxflags);
+    return NO_FLAGS;
+}
+
 QList<ProjectExplorer::HeaderPath> WinscwToolChain::systemHeaderPaths() const
 {
     QList<ProjectExplorer::HeaderPath> result;
@@ -179,13 +195,7 @@ void WinscwToolChain::addToEnvironment(Utils::Environment &env) const
     env.set(QLatin1String("MWSYM2LIBRARIES"), toNativePath(m_systemLibraryPathes));
     env.set(QLatin1String("MWSYM2LIBRARYFILES"),
             QLatin1String("MSL_All_MSE_Symbian_D.lib;gdi32.lib;user32.lib;kernel32.lib"));
-    env.prependOrSetPath(QFileInfo(m_compilerPath).absolutePath());
-}
-
-
-QString WinscwToolChain::mkspec() const
-{
-    return QString(); // Always use default from Qt version
+    env.prependOrSetPath(m_compilerPath.toString());
 }
 
 QString WinscwToolChain::makeCommand() const
@@ -197,9 +207,9 @@ QString WinscwToolChain::makeCommand() const
 #endif
 }
 
-QString WinscwToolChain::debuggerCommand() const
+Utils::FileName WinscwToolChain::debuggerCommand() const
 {
-    return QString();
+    return Utils::FileName();
 }
 
 QString WinscwToolChain::defaultMakeTarget() const
@@ -237,7 +247,7 @@ ProjectExplorer::ToolChain *WinscwToolChain::clone() const
 QVariantMap WinscwToolChain::toMap() const
 {
     QVariantMap result = ToolChain::toMap();
-    result.insert(QLatin1String(winscwCompilerPathKeyC), m_compilerPath);
+    result.insert(QLatin1String(winscwCompilerPathKeyC), m_compilerPath.toString());
     const QString semicolon = QString(QLatin1Char(';'));
     result.insert(QLatin1String(winscwSystemIncludePathKeyC), m_systemIncludePathes.join(semicolon));
     result.insert(QLatin1String(winscwSystemLibraryPathKeyC), m_systemLibraryPathes.join(semicolon));
@@ -248,7 +258,7 @@ bool WinscwToolChain::fromMap(const QVariantMap &data)
 {
     if (!ToolChain::fromMap(data))
         return false;
-    m_compilerPath = data.value(QLatin1String(winscwCompilerPathKeyC)).toString();
+    m_compilerPath = Utils::FileName::fromString(data.value(QLatin1String(winscwCompilerPathKeyC)).toString());
     const QChar semicolon = QLatin1Char(';');
     m_systemIncludePathes = data.value(QLatin1String(winscwSystemIncludePathKeyC)).toString().split(semicolon);
     m_systemLibraryPathes = data.value(QLatin1String(winscwSystemLibraryPathKeyC)).toString().split(semicolon);
@@ -281,25 +291,20 @@ QStringList WinscwToolChain::systemLibraryPathes() const
     return m_systemLibraryPathes;
 }
 
-void WinscwToolChain::setCompilerPath(const QString &path)
+void WinscwToolChain::setCompilerCommand(const Utils::FileName &path)
 {
     if (m_compilerPath == path)
         return;
 
     m_compilerPath = path;
-    updateId(); // Will trigger topolChainUpdated()!
+    toolChainUpdated();
 }
 
-QString WinscwToolChain::compilerPath() const
+Utils::FileName WinscwToolChain::compilerCommand() const
 {
     return m_compilerPath;
 }
 
-void WinscwToolChain::updateId()
-{
-    setId(QString::fromLatin1("%1:%2").arg(Constants::WINSCW_TOOLCHAIN_ID).arg(m_compilerPath));
-}
-
 // --------------------------------------------------------------------------
 // ToolChainConfigWidget
 // --------------------------------------------------------------------------
@@ -328,7 +333,7 @@ void WinscwToolChainConfigWidget::apply()
 {
     WinscwToolChain *tc = static_cast<WinscwToolChain *>(toolChain());
     Q_ASSERT(tc);
-    tc->setCompilerPath(m_ui->compilerPath->path());
+    tc->setCompilerCommand(m_ui->compilerPath->fileName());
     tc->setSystemIncludePathes(fromNativePath(m_ui->includeEdit->text()));
     tc->setSystemLibraryPathes(fromNativePath(m_ui->libraryEdit->text()));
 }
@@ -337,7 +342,7 @@ void WinscwToolChainConfigWidget::discard()
 {
     WinscwToolChain *tc = static_cast<WinscwToolChain *>(toolChain());
     Q_ASSERT(tc);
-    m_ui->compilerPath->setPath(tc->compilerPath());
+    m_ui->compilerPath->setFileName(tc->compilerCommand());
     m_ui->includeEdit->setText(toNativePath(tc->systemIncludePathes()));
     m_ui->libraryEdit->setText(toNativePath(tc->systemLibraryPathes()));
 }
@@ -346,18 +351,17 @@ bool WinscwToolChainConfigWidget::isDirty() const
 {
     WinscwToolChain *tc = static_cast<WinscwToolChain *>(toolChain());
     Q_ASSERT(tc);
-    return tc->compilerPath() != m_ui->compilerPath->path()
+    return tc->compilerCommand() != m_ui->compilerPath->fileName()
             || tc->systemIncludePathes() != fromNativePath(m_ui->includeEdit->text())
             || tc->systemLibraryPathes() != fromNativePath(m_ui->libraryEdit->text());
 }
 
 void WinscwToolChainConfigWidget::handleCompilerPathUpdate()
 {
-    QString path = m_ui->compilerPath->path();
+    Utils::FileName path = m_ui->compilerPath->fileName();
     if (path.isEmpty())
         return;
-    QFileInfo fi(path);
-    if (!fi.exists())
+    if (!path.toFileInfo().exists())
         return;
     m_ui->includeEdit->setText(toNativePath(detectIncludesFor(path)));
     m_ui->libraryEdit->setText(toNativePath(detectLibrariesFor(path)));
@@ -365,7 +369,7 @@ void WinscwToolChainConfigWidget::handleCompilerPathUpdate()
 
 void WinscwToolChainConfigWidget::makeDirty()
 {
-    emit dirty(toolChain());
+    emit dirty();
 }
 
 // --------------------------------------------------------------------------
@@ -399,16 +403,16 @@ QList<ProjectExplorer::ToolChain *> WinscwToolChainFactory::autoDetect()
         QFileInfo fi(path + QLatin1String("/x86Build/Symbian_Tools/Command_Line_Tools/mwwinrc.exe"));
         if (fi.exists() && fi.isExecutable()) {
             WinscwToolChain *tc = new WinscwToolChain(false);
-            tc->setCompilerPath(fi.absoluteFilePath());
+            tc->setCompilerCommand(Utils::FileName(fi));
             tc->setDisplayName(tr("WINSCW from Qt version"));
             result.append(tc);
         }
     }
 
-    QString cc = Utils::Environment::systemEnvironment().searchInPath(QLatin1String("mwwinrc"));
+    Utils::FileName cc = Utils::FileName::fromString(Utils::Environment::systemEnvironment().searchInPath(QLatin1String("mwwinrc")));
     if (!cc.isEmpty()) {
         WinscwToolChain *tc = new WinscwToolChain(true);
-        tc->setCompilerPath(cc);
+        tc->setCompilerCommand(cc);
         tc->setSystemIncludePathes(detectIncludesFor(cc));
         tc->setSystemLibraryPathes(detectLibrariesFor(cc));
         result.append(tc);
