@@ -2,7 +2,7 @@
 **
 ** This file is part of Qt Creator
 **
-** Copyright (c) 2011 Nokia Corporation and/or its subsidiary(-ies).
+** Copyright (c) 2012 Nokia Corporation and/or its subsidiary(-ies).
 **
 ** Contact: Nokia Corporation (qt-info@nokia.com)
 **
@@ -32,10 +32,10 @@
 
 #include "abstractmobileapp.h"
 
-#include <QtCore/QCoreApplication>
-#include <QtCore/QDir>
-#include <QtCore/QFile>
-#include <QtCore/QTextStream>
+#include <QCoreApplication>
+#include <QDir>
+#include <QFile>
+#include <QTextStream>
 
 #ifndef CREATORLESSTEST
 #include <coreplugin/icore.h>
@@ -349,7 +349,7 @@ QList<AbstractGeneratedFileInfo> AbstractMobileApp::fileUpdates(const QString &m
         QFile readFile(newFile.fileInfo.absoluteFilePath());
         if (!readFile.open(QIODevice::ReadOnly))
            continue;
-        const QString firstLine = readFile.readLine();
+        const QString firstLine = QString::fromUtf8(readFile.readLine());
         const QStringList elements = firstLine.split(QLatin1Char(' '));
         if (elements.count() != 5 || elements.at(1) != FileChecksum
                 || elements.at(3) != FileStubVersion)
@@ -390,7 +390,7 @@ bool AbstractMobileApp::updateFiles(const QList<AbstractGeneratedFileInfo> &list
 // CREATORLESSTEST is in tests/manual/appwizards/helpers.cpp
 QString AbstractMobileApp::templatesRoot()
 {
-    return Core::ICore::instance()->resourcePath()
+    return Core::ICore::resourcePath()
         + QLatin1String("/templates/");
 }
 
