@@ -2,7 +2,7 @@
 **
 ** This file is part of Qt Creator
 **
-** Copyright (c) 2011 Nokia Corporation and/or its subsidiary(-ies).
+** Copyright (c) 2012 Nokia Corporation and/or its subsidiary(-ies).
 **
 ** Contact: Nokia Corporation (qt-info@nokia.com)
 **
@@ -38,11 +38,11 @@
 #include <coreplugin/coreconstants.h>
 #include <utils/fileutils.h>
 
-#include <QtCore/QTemporaryFile>
-#include <QtGui/QDesktopServices>
-#include <QtGui/QFileDialog>
-#include <QtGui/QFileDialog>
-#include <QtGui/QMessageBox>
+#include <QTemporaryFile>
+#include <QDesktopServices>
+#include <QFileDialog>
+#include <QFileDialog>
+#include <QMessageBox>
 
 namespace Qt4ProjectManager {
 namespace Internal {
