@@ -2,7 +2,7 @@
 **
 ** This file is part of Qt Creator
 **
-** Copyright (c) 2011 Nokia Corporation and/or its subsidiary(-ies).
+** Copyright (c) 2012 Nokia Corporation and/or its subsidiary(-ies).
 **
 ** Contact: Nokia Corporation (qt-info@nokia.com)
 **
@@ -37,10 +37,10 @@
 
 #include <find/ifindfilter.h>
 
-#include <QtCore/QPointer>
-#include <QtGui/QLabel>
-#include <QtGui/QComboBox>
-#include <QtGui/QStringListModel>
+#include <QPointer>
+#include <QLabel>
+#include <QComboBox>
+#include <QStringListModel>
 
 
 namespace TextEditor {
@@ -62,7 +62,9 @@ public:
     void setDirectory(const QString &directory);
 
 protected:
-    Utils::FileIterator *files() const;
+    Utils::FileIterator *files(const QStringList &nameFilters,
+                               const QVariant &additionalParameters) const;
+    QVariant additionalParameters() const;
     QString label() const;
     QString toolTip() const;
 
