@@ -2,7 +2,7 @@
 **
 ** This file is part of Qt Creator
 **
-** Copyright (c) 2011 Nokia Corporation and/or its subsidiary(-ies).
+** Copyright (c) 2012 Nokia Corporation and/or its subsidiary(-ies).
 **
 ** Contact: Nokia Corporation (qt-info@nokia.com)
 **
@@ -49,10 +49,10 @@
 #include <coreplugin/icore.h>
 #include <coreplugin/mimedatabase.h>
 
-#include <QtCore/QSharedPointer>
-#include <QtCore/QFileInfo>
+#include <QSharedPointer>
+#include <QFileInfo>
 
-#include <QtCore/QDebug>
+#include <QDebug>
 
 using namespace TextEditor;
 using namespace TextEditor::Internal;
@@ -80,7 +80,7 @@ PlainTextEditorWidget::PlainTextEditorWidget(QWidget *parent)
 
     m_commentDefinition.clearCommentStyles();
 
-    connect(file(), SIGNAL(changed()), this, SLOT(configure()));
+    connect(editorDocument(), SIGNAL(changed()), this, SLOT(configure()));
     connect(Manager::instance(), SIGNAL(mimeTypesRegistered()), this, SLOT(configure()));
 }
 
@@ -92,9 +92,9 @@ Core::IEditor *PlainTextEditor::duplicate(QWidget *parent)
     return newWidget->editor();
 }
 
-QString PlainTextEditor::id() const
+Core::Id PlainTextEditor::id() const
 {
-    return QLatin1String(Core::Constants::K_DEFAULT_TEXT_EDITOR_ID);
+    return Core::Constants::K_DEFAULT_TEXT_EDITOR_ID;
 }
 
 void PlainTextEditorWidget::unCommentSelection()
@@ -149,14 +149,14 @@ void PlainTextEditorWidget::setTabSettings(const TextEditor::TabSettings &ts)
 void PlainTextEditorWidget::configure()
 {
     Core::MimeType mimeType;
-    if (file())
-        mimeType = Core::ICore::instance()->mimeDatabase()->findByFile(file()->fileName());
+    if (editorDocument())
+        mimeType = Core::ICore::mimeDatabase()->findByFile(editorDocument()->fileName());
     configure(mimeType);
 }
 
 void PlainTextEditorWidget::configure(const QString &mimeType)
 {
-    configure(Core::ICore::instance()->mimeDatabase()->findByType(mimeType));
+    configure(Core::ICore::mimeDatabase()->findByType(mimeType));
 }
 
 void PlainTextEditorWidget::configure(const Core::MimeType &mimeType)
@@ -180,7 +180,7 @@ void PlainTextEditorWidget::configure(const Core::MimeType &mimeType)
             m_isMissingSyntaxDefinition = false;
             const QSharedPointer<HighlightDefinition> &definition =
                 Manager::instance()->definition(definitionId);
-            if (!definition.isNull()) {
+            if (!definition.isNull() && definition->isValid()) {
                 highlighter->setDefaultContext(definition->initialContext());
 
                 m_commentDefinition.setAfterWhiteSpaces(definition->isCommentAfterWhiteSpaces());
@@ -190,8 +190,8 @@ void PlainTextEditorWidget::configure(const Core::MimeType &mimeType)
 
                 setCodeFoldingSupported(true);
             }
-        } else if (file()) {
-            const QString &fileName = file()->fileName();
+        } else if (editorDocument()) {
+            const QString &fileName = editorDocument()->fileName();
             if (TextEditorSettings::instance()->highlighterSettings().isIgnoredFilePattern(fileName))
                 m_isMissingSyntaxDefinition = false;
         }
@@ -221,7 +221,7 @@ QString PlainTextEditorWidget::findDefinitionId(const Core::MimeType &mimeType,
         if (definitionId.isEmpty()) {
             foreach (const QString &parent, mimeType.subClassesOf()) {
                 const Core::MimeType &parentMimeType =
-                    Core::ICore::instance()->mimeDatabase()->findByType(parent);
+                    Core::ICore::mimeDatabase()->findByType(parent);
                 definitionId = findDefinitionId(parentMimeType, considerParents);
             }
         }
@@ -231,8 +231,8 @@ QString PlainTextEditorWidget::findDefinitionId(const Core::MimeType &mimeType,
 
 void PlainTextEditorWidget::acceptMissingSyntaxDefinitionInfo()
 {
-    Core::ICore::instance()->showOptionsDialog(Constants::TEXT_EDITOR_SETTINGS_CATEGORY,
-                                               Constants::TEXT_EDITOR_HIGHLIGHTER_SETTINGS);
+    Core::ICore::showOptionsDialog(QLatin1String(Constants::TEXT_EDITOR_SETTINGS_CATEGORY),
+                                               QLatin1String(Constants::TEXT_EDITOR_HIGHLIGHTER_SETTINGS));
 }
 
 void PlainTextEditorWidget::ignoreMissingSyntaxDefinitionInfo()
