@@ -2,7 +2,7 @@
 **
 ** This file is part of Qt Creator
 **
-** Copyright (c) 2011 Nokia Corporation and/or its subsidiary(-ies).
+** Copyright (c) 2012 Nokia Corporation and/or its subsidiary(-ies).
 **
 ** Contact: Nokia Corporation (qt-info@nokia.com)
 **
@@ -44,14 +44,14 @@
 #include <texteditor/fontsettings.h>
 #include <extensionsystem/pluginmanager.h>
 
-#include <QtCore/QModelIndex>
-#include <QtCore/QAbstractTableModel>
-#include <QtCore/QList>
-#include <QtCore/QSettings>
-#include <QtCore/QTextStream>
-#include <QtCore/QHash>
-#include <QtGui/QMessageBox>
-#include <QtGui/QMainWindow>
+#include <QModelIndex>
+#include <QAbstractTableModel>
+#include <QList>
+#include <QSettings>
+#include <QTextStream>
+#include <QHash>
+#include <QMessageBox>
+#include <QMainWindow>
 
 namespace TextEditor {
 namespace Internal {
@@ -360,15 +360,15 @@ void SnippetsSettingsPagePrivate::configureUi(QWidget *w)
     loadSettings();
     loadSnippetGroup(m_ui.groupCombo->currentIndex());
 
-    connect(m_model, SIGNAL(rowsInserted(QModelIndex, int, int)),
+    connect(m_model, SIGNAL(rowsInserted(QModelIndex,int,int)),
             this, SLOT(selectSnippet(QModelIndex,int)));
-    connect(m_model, SIGNAL(rowsInserted(QModelIndex, int, int)),
+    connect(m_model, SIGNAL(rowsInserted(QModelIndex,int,int)),
             this, SLOT(markSnippetsCollection()));
-    connect(m_model, SIGNAL(rowsRemoved(QModelIndex, int, int)),
+    connect(m_model, SIGNAL(rowsRemoved(QModelIndex,int,int)),
             this, SLOT(markSnippetsCollection()));
-    connect(m_model, SIGNAL(rowsMoved(QModelIndex, int, int, QModelIndex, int)),
+    connect(m_model, SIGNAL(rowsMoved(QModelIndex,int,int,QModelIndex,int)),
             this, SLOT(selectMovedSnippet(QModelIndex,int,int,QModelIndex,int)));
-    connect(m_model, SIGNAL(rowsMoved(QModelIndex, int, int, QModelIndex, int)),
+    connect(m_model, SIGNAL(rowsMoved(QModelIndex,int,int,QModelIndex,int)),
             this, SLOT(markSnippetsCollection()));
     connect(m_model, SIGNAL(dataChanged(QModelIndex,QModelIndex)),
             this, SLOT(markSnippetsCollection()));
@@ -402,7 +402,7 @@ void SnippetsSettingsPagePrivate::apply()
         if (SnippetsCollection::instance()->synchronize(&errorString))
             m_snippetsCollectionChanged = false;
         else
-            QMessageBox::critical(Core::ICore::instance()->mainWindow(),
+            QMessageBox::critical(Core::ICore::mainWindow(),
                     tr("Error While Saving Snippet Collection"), errorString);
     }
 }
@@ -422,7 +422,7 @@ void SnippetsSettingsPagePrivate::loadSettings()
     if (m_ui.groupCombo->count() == 0)
         return;
 
-    if (QSettings *s = Core::ICore::instance()->settings()) {
+    if (QSettings *s = Core::ICore::settings()) {
         m_settings.fromSettings(m_settingsPrefix, s);
         const QString &lastGroupName = m_settings.lastUsedSnippetGroup();
         const int index = m_ui.groupCombo->findText(lastGroupName);
@@ -438,7 +438,7 @@ void SnippetsSettingsPagePrivate::writeSettings()
     if (m_ui.groupCombo->count() == 0)
         return;
 
-    if (QSettings *s = Core::ICore::instance()->settings()) {
+    if (QSettings *s = Core::ICore::settings()) {
         m_settings.setLastUsedSnippetGroup(m_ui.groupCombo->currentText());
         m_settings.toSettings(m_settingsPrefix, s);
     }
