@@ -2,7 +2,7 @@
 **
 ** This file is part of Qt Creator
 **
-** Copyright (c) 2011 Nokia Corporation and/or its subsidiary(-ies).
+** Copyright (c) 2012 Nokia Corporation and/or its subsidiary(-ies).
 **
 ** Contact: Nokia Corporation (qt-info@nokia.com)
 **
@@ -35,8 +35,8 @@
 
 #include "texteditor_global.h"
 
-#include <QtCore/QVariant>
-#include <QtGui/QTextBlock>
+#include <QVariant>
+#include <QTextBlock>
 
 QT_BEGIN_NAMESPACE
 class QSettings;
