@@ -2,7 +2,7 @@
 **
 ** This file is part of Qt Creator
 **
-** Copyright (c) 2011 Nokia Corporation and/or its subsidiary(-ies).
+** Copyright (c) 2012 Nokia Corporation and/or its subsidiary(-ies).
 **
 ** Contact: Nokia Corporation (qt-info@nokia.com)
 **
@@ -33,10 +33,10 @@
 #include "texteditoroverlay.h"
 #include "basetexteditor.h"
 
-#include <QtCore/QDebug>
-#include <QtCore/QMap>
-#include <QtGui/QPainter>
-#include <QtGui/QTextBlock>
+#include <QDebug>
+#include <QMap>
+#include <QPainter>
+#include <QTextBlock>
 
 using namespace TextEditor;
 using namespace TextEditor::Internal;
