@@ -2,7 +2,7 @@
 **
 ** This file is part of Qt Creator
 **
-** Copyright (c) 2011 Nokia Corporation and/or its subsidiary(-ies).
+** Copyright (c) 2012 Nokia Corporation and/or its subsidiary(-ies).
 **
 ** Contact: Nokia Corporation (qt-info@nokia.com)
 **
@@ -39,10 +39,10 @@
 
 #include <utils/qtcassert.h>
 
-#include <QtCore/QHash>
-#include <QtCore/QVector>
-#include <QtCore/QStringList>
-#include <QtCore/QDebug>
+#include <QHash>
+#include <QVector>
+#include <QStringList>
+#include <QDebug>
 
 // #define DEBUG_PARSER
 
