@@ -2,7 +2,7 @@
 **
 ** This file is part of Qt Creator
 **
-** Copyright (c) 2011 Nokia Corporation and/or its subsidiary(-ies).
+** Copyright (c) 2012 Nokia Corporation and/or its subsidiary(-ies).
 **
 ** Contact: Nokia Corporation (qt-info@nokia.com)
 **
@@ -31,7 +31,6 @@
 **************************************************************************/
 
 #include "welcomeplugin.h"
-#include "multifeedrssmodel.h"
 
 #include <extensionsystem/pluginmanager.h>
 
@@ -48,22 +47,22 @@
 #include <utils/iwelcomepage.h>
 #include <utils/networkaccessmanager.h>
 
-#include <QtGui/QScrollArea>
-#include <QtGui/QDesktopServices>
-#include <QtGui/QPainter>
-#include <QtGui/QHBoxLayout>
+#include <QScrollArea>
+#include <QDesktopServices>
+#include <QPainter>
+#include <QHBoxLayout>
 
-#include <QtCore/QCoreApplication>
-#include <QtCore/QDir>
-#include <QtCore/QSettings>
-#include <QtCore/QDebug>
-#include <QtCore/QUrl>
-#include <QtCore/QtPlugin>
+#include <QCoreApplication>
+#include <QDir>
+#include <QSettings>
+#include <QDebug>
+#include <QUrl>
+#include <QtPlugin>
 
-#include <QtDeclarative/QDeclarativeView>
-#include <QtDeclarative/QDeclarativeContext>
-#include <QtDeclarative/QDeclarativeEngine>
-#include <QtDeclarative/QDeclarativeNetworkAccessManagerFactory>
+#include <QDeclarativeView>
+#include <QDeclarativeContext>
+#include <QDeclarativeEngine>
+#include <QDeclarativeNetworkAccessManagerFactory>
 
 enum { debug = 0 };
 
@@ -154,15 +153,21 @@ WelcomeMode::WelcomeMode() :
     QVBoxLayout *layout = new QVBoxLayout;
     layout->setMargin(0);
     layout->setSpacing(0);
-//    layout->addWidget(new Utils::StyledBar);
-    layout->addWidget(m_welcomePage);
     m_modeWidget->setLayout(layout);
 
+    Utils::StyledBar* styledBar = new Utils::StyledBar(m_modeWidget);
+    layout->addWidget(styledBar);
+    QScrollArea *scrollArea = new QScrollArea(m_modeWidget);
+    scrollArea->setFrameShape(QFrame::NoFrame);
+    layout->addWidget(scrollArea);
+    scrollArea->setWidget(m_welcomePage);
+    scrollArea->setWidgetResizable(true);
+    m_welcomePage->setMinimumWidth(860);
+    m_welcomePage->setMinimumHeight(548);
     PluginManager *pluginManager = PluginManager::instance();
     connect(pluginManager, SIGNAL(objectAdded(QObject*)), SLOT(welcomePluginAdded(QObject*)));
 
-    Core::ModeManager *modeManager = Core::ModeManager::instance();
-    connect(modeManager, SIGNAL(currentModeChanged(Core::IMode*)), SLOT(modeChanged(Core::IMode*)));
+    connect(Core::ModeManager::instance(), SIGNAL(currentModeChanged(Core::IMode*)), SLOT(modeChanged(Core::IMode*)));
 
     setWidget(m_modeWidget);
 }
@@ -178,7 +183,7 @@ bool WelcomeMode::eventFilter(QObject *, QEvent *e)
 
 WelcomeMode::~WelcomeMode()
 {
-    QSettings *settings = Core::ICore::instance()->settings();
+    QSettings *settings = Core::ICore::settings();
     settings->setValue(QLatin1String(currentPageSettingsKeyC), activePlugin());
     delete m_modeWidget;
 }
@@ -188,35 +193,21 @@ bool sortFunction(Utils::IWelcomePage * a, Utils::IWelcomePage *b)
     return a->priority() < b->priority();
 }
 
-void WelcomeMode::facilitateQml(QDeclarativeEngine *engine)
+void WelcomeMode::facilitateQml(QDeclarativeEngine * /*engine*/)
 {
-    static const char feedGroupName[] = "Feeds";
-
-    MultiFeedRssModel *rssModel = new MultiFeedRssModel(this);
-    QSettings *settings = Core::ICore::instance()->settings();
-    if (settings->childGroups().contains(feedGroupName)) {
-        int size = settings->beginReadArray(feedGroupName);
-        for (int i = 0; i < size; ++i)
-        {
-            settings->setArrayIndex(i);
-            rssModel->addFeed(settings->value("url").toString());
-        }
-        settings->endArray();
-    } else {
-        rssModel->addFeed(QLatin1String("http://labs.trolltech.com/blogs/feed"));
-        rssModel->addFeed(QLatin1String("http://feeds.feedburner.com/TheQtBlog?format=xml"));
-    }
-
-    engine->rootContext()->setContextProperty("aggregatedFeedsModel", rssModel);
 }
 
 void WelcomeMode::initPlugins()
 {
-    QSettings *settings = Core::ICore::instance()->settings();
+    QSettings *settings = Core::ICore::settings();
     setActivePlugin(settings->value(QLatin1String(currentPageSettingsKeyC)).toInt());
 
+    // TODO: re-enable reading from Settings when possible. See QTCREATORBUG-6803
+    if (activePlugin() > 1)
+        setActivePlugin(1);
+
     QDeclarativeContext *ctx = m_welcomePage->rootContext();
-    ctx->setContextProperty("welcomeMode", this);
+    ctx->setContextProperty(QLatin1String("welcomeMode"), this);
 
     QList<Utils::IWelcomePage*> plugins = PluginManager::instance()->getObjects<Utils::IWelcomePage>();
     qSort(plugins.begin(), plugins.end(), &sortFunction);
@@ -238,11 +229,11 @@ void WelcomeMode::initPlugins()
         m_pluginList.append(plugin);
     }
 
-    ctx->setContextProperty("pagesModel", QVariant::fromValue(m_pluginList));
+    ctx->setContextProperty(QLatin1String("pagesModel"), QVariant::fromValue(m_pluginList));
 
     // finally, load the root page
     m_welcomePage->setSource(
-            QUrl::fromLocalFile(Core::ICore::instance()->resourcePath() + "/welcomescreen/welcomescreen.qml"));
+            QUrl::fromLocalFile(Core::ICore::resourcePath() + QLatin1String("/welcomescreen/welcomescreen.qml")));
 }
 
 QString WelcomeMode::platform() const
@@ -273,7 +264,7 @@ void WelcomeMode::welcomePluginAdded(QObject *obj)
         m_pluginList.insert(insertPos, plugin);
         // update model through reset
         QDeclarativeContext *ctx = m_welcomePage->rootContext();
-        ctx->setContextProperty("pagesModel", QVariant::fromValue(m_pluginList));
+        ctx->setContextProperty(QLatin1String("pagesModel"), QVariant::fromValue(m_pluginList));
     }
 }
 
@@ -285,7 +276,7 @@ void WelcomeMode::sendFeedback()
 
 void WelcomeMode::newProject()
 {
-    Core::ICore::instance()->showNewItemDialog(tr("New Project"),
+    Core::ICore::showNewItemDialog(tr("New Project"),
                                                Core::IWizard::wizardsOfKind(Core::IWizard::ProjectWizard));
 }
 
@@ -301,12 +292,11 @@ void WelcomeMode::modeChanged(Core::IMode *mode)
 // Eike doesn't like this, but I do...
 
 //    ProjectExplorer::ProjectExplorerPlugin *projectExplorer = ProjectExplorer::ProjectExplorerPlugin::instance();
-//    Core::ModeManager *modeManager = Core::ICore::instance()->modeManager();
-//    Core::EditorManager *editorManager = Core::ICore::instance()->editorManager();
+//    Core::EditorManager *editorManager = Core::ICore::editorManager();
 //    if (mode->id() == id() && (!projectExplorer->currentProject() && editorManager->openedEditors().isEmpty()))
-//        modeManager->setModeBarHidden(true);
+//        ModeManager::setModeBarHidden(true);
 //    else
-//        modeManager->setModeBarHidden(false);
+//        ModeManager::setModeBarHidden(false);
 }
 
 //
@@ -344,7 +334,7 @@ bool WelcomePlugin::initialize(const QStringList & /* arguments */, QString * /*
 void WelcomePlugin::extensionsInitialized()
 {
     m_welcomeMode->initPlugins();
-    Core::ModeManager::instance()->activateMode(m_welcomeMode->id());
+    Core::ModeManager::activateMode(m_welcomeMode->id());
 }
 
 } // namespace Internal
