@@ -2,7 +2,7 @@
 **
 ** This file is part of Qt Creator
 **
-** Copyright (c) 2011 Nokia Corporation and/or its subsidiary(-ies).
+** Copyright (c) 2012 Nokia Corporation and/or its subsidiary(-ies).
 **
 ** Contact: Nokia Corporation (qt-info@nokia.com)
 **
@@ -33,11 +33,11 @@
 #include "codautils.h"
 #include <ctype.h>
 
-#include <QtCore/QCoreApplication>
-#include <QtCore/QDebug>
-#include <QtCore/QDate>
-#include <QtCore/QDateTime>
-#include <QtCore/QTime>
+#include <QCoreApplication>
+#include <QDebug>
+#include <QDate>
+#include <QDateTime>
+#include <QTime>
 
 #define logMessage(s)  do { qDebug() << "CODACLIENT: " << s; } while (0)
 
